@@ -5,13 +5,13 @@
  *
  *      Description:  Header File allocator Module.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  none
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -45,6 +45,7 @@ extern void allocator_allocateVector(void** ptr,
                 int offset,
                 DataType type,
                 int stride,
-                bstring domain);
+                bstring domain,
+                int init_per_thread);
 
 #endif /*ALLOCATOR_H*/
