@@ -5,13 +5,13 @@
  *
  *      Description:  Header file of pthread interface module
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -32,6 +32,7 @@
 
 #include <pthread.h>
 #include <threads_types.h>
+#include <strUtil.h>
 
 #define THREADS_BARRIER pthread_barrier_wait(&threads_barrier)
 #define MIN_ITERATIONS 10
@@ -107,7 +108,8 @@ extern void threads_destroy(int numberOfGroups, int numberOfStreams);
 /**
  * @brief  Create Thread groups
  * @param  numberOfGroups The number of groups to create
+ * @param  groups Pointer to the groups data
  */
-extern void threads_createGroups(int numberOfGroups);
+extern void threads_createGroups(int numberOfGroups, Workgroup *groups);
 
 #endif /* THREADS_H */
