@@ -5,13 +5,13 @@
  *
  *      Description:  A flexible and extensible benchmarking toolbox
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -39,6 +39,7 @@
 #include <ctype.h>
 #include <inttypes.h>
 #include <math.h>
+#include <signal.h>
 
 #include <bstrlib.h>
 #include <errno.h>
@@ -47,8 +48,10 @@
 #include <testcases.h>
 #include <strUtil.h>
 #include <allocator.h>
+#include <ptt2asm.h>
 
 #include <likwid.h>
+#include <likwid-marker.h>
 
 #define STRINGIFY(x) #x
 #define TOSTRING(x) STRINGIFY(x)
@@ -63,16 +66,24 @@ extern void* getIterSingle(void* arg);
     printf("Supported Options:\n"); \
     printf("-h\t\t Help message\n"); \
     printf("-a\t\t List available benchmarks \n"); \
-    printf("-d\t\t Delimiter used for physical core list (default ,) \n"); \
+    printf("-d\t\t Delimiter used for physical hwthread list (default ,) \n"); \
     printf("-p\t\t List available thread domains\n"); \
-    printf("\t\t or the physical ids of the cores selected by the -c expression \n"); \
+    printf("\t\t or the physical ids of the hwthreads selected by the -c expression \n"); \
     printf("-s <TIME>\t Seconds to run the test minimally (default 1)\n");\
     printf("\t\t If resulting iteration count is below 10, it is normalized to 10.\n");\
     printf("-i <ITERS>\t Specify the number of iterations per thread manually. \n"); \
     printf("-l <TEST>\t list properties of benchmark \n"); \
     printf("-t <TEST>\t type of test \n"); \
     printf("-w\t\t <thread_domain>:<size>[:<num_threads>[:<chunk size>:<stride>]-<streamId>:<domain_id>[:<offset>]\n"); \
-    printf("\t\t <size> in kB, MB or GB  (mandatory)\n"); \
+    printf("-W\t\t <thread_domain>:<size>[:<num_threads>[:<chunk size>:<stride>]]\n"); \
+    printf("\t\t <size> in kB, MB or GB (mandatory)\n"); \
+    printf("For dynamically loaded benchmarks\n"); \
+    printf("-f <PATH>\t Specify a folder for the temporary files. default: /tmp\n"); \
+    printf("-o <FILE>\t Save generated assembly to file\n"); \
+    printf("\n"); \
+    printf("Difference between -w and -W :\n"); \
+    printf("-w allocates the streams in the thread_domain with one thread and support placement of streams\n"); \
+    printf("-W allocates the streams chunk-wise by each thread in the thread_domain\n"); \
     printf("\n"); \
     printf("Usage: \n"); \
     printf("# Run the store benchmark on all CPUs of the system with a vector size of 1 GB\n"); \
@@ -81,6 +92,8 @@ extern void* getIterSingle(void* arg);
     printf("likwid-bench -t copy -w S0:100kB:1\n"); \
     printf("# Run the copy benchmark on one CPU at CPU socket 0 with a vector size of 100MB but place one stream on CPU socket 1\n"); \
     printf("likwid-bench -t copy -w S0:100MB:1-0:S0,1:S1\n"); \
+/*    printf("-c <COMP_LIST>\t Specify a list of compilers that should be searched for. default: gcc,icc,pgcc\n"); \*/
+/*    printf("-f <COMP_FLAGS>\t Specify compiler flags. Use \". default: \"-shared -fPIC\"\n"); \*/
 
 #define VERSION_MSG \
     printf("likwid-bench -- Version %d.%d.%d\n",VERSION,RELEASE,MINORVERSION); \
@@ -108,6 +121,13 @@ copyThreadData(ThreadUserData* src,ThreadUserData* dst)
 }
 
 
+void illhandler(int signum, siginfo_t *info, void *ptr)
+{
+    fprintf(stderr, "ERROR: Illegal instruction\n");
+    fprintf(stderr, "This happens if you want to run a kernel that uses instructions not available on your system.\n");
+    exit(EXIT_FAILURE);
+}
+
 
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
@@ -119,6 +139,8 @@ int main(int argc, char** argv)
     int globalNumberOfThreads = 0;
     int optPrintDomains = 0;
     int c;
+    int saveAsm = 0;
+    bstring asmFile = bfromcstr("");
     ThreadUserData myData;
     bstring testcase = bfromcstr("none");
     uint64_t numberOfWorkgroups = 0;
@@ -126,7 +148,7 @@ int main(int argc, char** argv)
     double time;
     double cycPerUp = 0.0;
     double cycPerCL = 0.0;
-    const TestCase* test = NULL;
+    TestCase* test = NULL;
     uint64_t realSize = 0;
     uint64_t realIter = 0;
     uint64_t maxCycles = 0;
@@ -141,8 +163,17 @@ int main(int argc, char** argv)
     binsertch(HLINE, 0, 80, '-');
     binsertch(HLINE, 80, 1, '\n');
     int (*ownprintf)(const char *format, ...);
+#ifdef _ARCH_PPC
+    int clsize = 128;
+#else
     int clsize = sysconf (_SC_LEVEL1_DCACHE_LINESIZE);
+#endif
+    char compilers[512] = "gcc,icc,pgcc";
+    char defcompilepath[512] = "/tmp";
+    char compilepath[513] = "";
+    char compileflags[512] = "-shared -fPIC";
     ownprintf = &printf;
+    struct sigaction sig;
 
     /* Handling of command line options */
     if (argc ==  1)
@@ -151,7 +182,28 @@ int main(int argc, char** argv)
         exit(EXIT_SUCCESS);
     }
 
-    while ((c = getopt (argc, argv, "w:t:s:l:aphvi:")) != -1) {
+    while ((c = getopt (argc, argv, "W:w:t:s:l:aphvi:f:o:")) != -1) {
+        switch (c)
+        {
+            case 'f':
+                tmp = snprintf(compilepath, 512, "%s", optarg);
+                if (tmp > 0)
+                {
+                    compilepath[tmp] = '\0';
+                }
+                break;
+            case 'o':
+                bdestroy(asmFile);
+                asmFile = bfromcstr(optarg);
+                saveAsm = 1;
+                break;
+            default:
+                break;
+        }
+    }
+    optind = 0;
+
+    while ((c = getopt (argc, argv, "W:w:t:s:l:aphvi:f:o:")) != -1) {
         switch (c)
         {
             case 'h':
@@ -162,8 +214,29 @@ int main(int argc, char** argv)
                 exit (EXIT_SUCCESS);
             case 'a':
                 ownprintf(TESTS"\n");
+
+                struct bstrList* l = dynbench_getall();
+                if (l)
+                {
+                    ownprintf("\nUser benchmarks:\n");
+                    for (i = 0; i < l->qty; i++)
+                    {
+                        if (dynbench_test(l->entry[i]))
+                        {
+                            TestCase* t = NULL;
+                            int err = dynbench_load(l->entry[i], &t, NULL, NULL, NULL);
+                            if (!err && t)
+                            {
+                                printf("%s - %s\n", t->name, t->desc);
+                                dynbench_close(t, NULL);
+                            }
+                        }
+                    }
+                    bstrListDestroy(l);
+                }
                 exit (EXIT_SUCCESS);
             case 'w':
+            case 'W':
                 numberOfWorkgroups++;
                 break;
             case 's':
@@ -180,15 +253,22 @@ int main(int argc, char** argv)
             case 'l':
                 bdestroy(testcase);
                 testcase = bfromcstr(optarg);
+                int builtin = 1;
                 for (i=0; i<NUMKERNELS; i++)
                 {
                     if (biseqcstr(testcase, kernels[i].name))
                     {
-                        test = kernels+i;
+                        test = (TestCase*)kernels+i;
                         break;
                     }
                 }
 
+                if (test == NULL && dynbench_test(testcase))
+                {
+                    dynbench_load(testcase, &test, NULL, NULL, NULL);
+                    builtin = 0;
+                }
+
                 if (test == NULL)
                 {
                     fprintf (stderr, "Error: Unknown test case %s\n",optarg);
@@ -255,6 +335,10 @@ int main(int argc, char** argv)
                     }
                 }
                 bdestroy(testcase);
+                if (!builtin)
+                {
+                    dynbench_close(test, NULL);
+                }
                 exit (EXIT_SUCCESS);
 
                 break;
@@ -275,11 +359,31 @@ int main(int argc, char** argv)
                 {
                     if (biseqcstr(testcase, kernels[i].name))
                     {
-                        test = kernels+i;
+                        test = (TestCase*)kernels+i;
                         break;
                     }
                 }
 
+                if (test == NULL && dynbench_test(testcase))
+                {
+                    int ret = 0;
+                    if (strlen(compilepath) == 0)
+                    {
+                        ret = snprintf(compilepath, 512, "%s", defcompilepath);
+                        if (ret > 0) compilepath[ret] = '\0';
+                    }
+                    ret = dynbench_load(testcase, &test, compilepath, compilers, compileflags);
+                    if (saveAsm)
+                    {
+                        dynbench_asm(testcase, compilepath, asmFile);
+                    }
+                    if (ret != 0)
+                    {
+                        dynbench_close(test, compilepath);
+                        return EXIT_FAILURE;
+                    }
+                }
+
                 if (test == NULL)
                 {
                     fprintf (stderr, "Error: Unknown test case %s\n",optarg);
@@ -287,6 +391,9 @@ int main(int argc, char** argv)
                 }
                 bdestroy(testcase);
                 break;
+            case 'o':
+            case 'f':
+                break;
             case '?':
                 if (isprint (optopt))
                     fprintf (stderr, "Unknown option `-%c'.\n", optopt);
@@ -322,6 +429,11 @@ int main(int argc, char** argv)
     affinity_init();
     timer_init();
 
+    memset(&sig, 0, sizeof(struct sigaction));
+    sig.sa_sigaction = illhandler;
+    sig.sa_flags = SA_SIGINFO;
+    sigaction(SIGILL, &sig, NULL);
+
     if (optPrintDomains)
     {
         bdestroy(testcase);
@@ -343,21 +455,28 @@ int main(int argc, char** argv)
 
     allocator_init(numberOfWorkgroups * MAX_STREAMS);
     groups = (Workgroup*) malloc(numberOfWorkgroups*sizeof(Workgroup));
+    memset(groups, 0, numberOfWorkgroups*sizeof(Workgroup));
     tmp = 0;
 
     optind = 0;
-    while ((c = getopt (argc, argv, "w:t:s:l:i:aphv")) != -1)
+    while ((c = getopt (argc, argv, "W:w:t:s:l:i:aphvf:o:")) != -1)
     {
         switch (c)
         {
             case 'w':
+            case 'W':
                 currentWorkgroup = groups+tmp;
                 bstring groupstr = bfromcstr(optarg);
+                if (c == 'W')
+                {
+                    currentWorkgroup->init_per_thread = 1;
+                }
                 i = bstr_to_workgroup(currentWorkgroup, groupstr, test->type, test->streams);
                 bdestroy(groupstr);
                 size_t newsize = 0;
                 size_t stride = test->stride;
                 int nrThreads = currentWorkgroup->numberOfThreads;
+                int clsize = 128;
                 size_t orig_size = currentWorkgroup->size;
                 if (i == 0)
                 {
@@ -375,7 +494,7 @@ int main(int argc, char** argv)
                             newsize = (((int)(floor(orig_size/nrThreads))/stride)*(stride))*nrThreads;
                             if (newsize > 0 && warn_once)
                             {
-                                fprintf (stderr, "Warning: Sanitizing vector length to a multiple of the loop stride %d and thread count %d from %d elements (%d bytes) to %d elements (%d bytes)\n",stride, nrThreads, orig_size, orig_size*typesize, newsize, newsize*typesize);
+                                fprintf (stdout, "Warning: Sanitizing vector length to a multiple of the loop stride %d and thread count %d from %d elements (%d bytes) to %d elements (%d bytes)\n",stride, nrThreads, orig_size, orig_size*typesize, newsize, newsize*typesize);
                                 warn_once = 0;
                             }
                             else if (newsize == 0)
@@ -404,7 +523,8 @@ int main(int argc, char** argv)
                                                     currentWorkgroup->streams[i].offset,
                                                     test->type,
                                                     test->stride,
-                                                    currentWorkgroup->streams[i].domain);
+                                                    currentWorkgroup->streams[i].domain,
+                                                    currentWorkgroup->init_per_thread && nrThreads > 1);
                     }
                     tmp++;
                 }
@@ -413,16 +533,48 @@ int main(int argc, char** argv)
                     exit(EXIT_FAILURE);
                 }
                 if (newsize != currentWorkgroup->size)
+                {
                     currentWorkgroup->size = newsize;
+                }
+                if (nrThreads > 1)
+                {
+                    if (currentWorkgroup->init_per_thread)
+                    {
+                        fprintf (stdout, "Initialization: Each thread in domain initializes its own stream chunks\n");
+                    }
+                    else
+                    {
+                        fprintf (stdout, "Initialization: First thread in domain initializes the whole stream\n");
+                    }
+                }
                 break;
             default:
                 continue;
                 break;
         }
     }
+    if (numberOfWorkgroups > 1)
+    {
+        int g0_numberOfThreads = groups[0].numberOfThreads;
+        int g0_size = groups[0].size;
+        for (i = 1; i < numberOfWorkgroups; i++)
+        {
+            if (g0_numberOfThreads != groups[i].numberOfThreads)
+            {
+                fprintf (stdout, "Warning: Multiple workgroups with different thread counts are not recommended. Use with case!\n");
+                break;
+            }
+        }
+        for (i = 1; i < numberOfWorkgroups; i++)
+        {
+            if (g0_size != groups[i].size)
+            {
+                fprintf (stdout, "Warning: Multiple workgroups with different sizes are not recommended. Use with case!\n");
+                break;
+            }
+        }
+    }
 
-    /* :WARNING:05/04/2010 08:58:05 AM:jt: At the moment the thread
-     * module only allows equally sized thread groups*/
     for (i=0; i<numberOfWorkgroups; i++)
     {
         globalNumberOfThreads += groups[i].numberOfThreads;
@@ -438,7 +590,7 @@ int main(int argc, char** argv)
 
 
     threads_init(globalNumberOfThreads);
-    threads_createGroups(numberOfWorkgroups);
+    threads_createGroups(numberOfWorkgroups, groups);
 
     /* we configure global barriers only */
     barrier_init(1);
@@ -468,6 +620,7 @@ int main(int argc, char** argv)
         myData.test = test;
         myData.cycles = 0;
         myData.numberOfThreads = groups[i].numberOfThreads;
+        myData.init_per_thread = groups[i].init_per_thread;
         myData.processors = (int*) malloc(myData.numberOfThreads * sizeof(int));
         myData.streams = (void**) malloc(test->streams * sizeof(void*));
 
@@ -529,6 +682,19 @@ int main(int argc, char** argv)
     {
         time = timer_print(&itertime);
     }
+
+/*#if defined(__ARM_ARCH_7A__) || defined(__ARM_ARCH_8A)*/
+/*    if (maxCycles > 0)*/
+/*    {*/
+/*        ownprintf("WARNING: The cycle based values are calculated using the current but fixed CPU frequency on ARMv7/8\n");*/
+/*    }*/
+/*    else*/
+/*    {*/
+/*        ownprintf(bdata(HLINE));*/
+/*        ownprintf("WARNING: All cycle based values will be zero on ARMv7!\n");*/
+/*        ownprintf("WARNING: The cycle count cannot be calculated because the clock frequency is not fixed.\n");*/
+/*    }*/
+/*#endif*/
     int datatypesize = allocator_dataTypeLength(test->type);
     uint64_t size_per_thread = threads_data[0].data.size;
     uint64_t iters_per_thread = threads_data[0].data.iter;
@@ -542,8 +708,8 @@ int main(int argc, char** argv)
     ownprintf("Iterations:\t\t%" PRIu64 "\n", realIter);
     ownprintf("Iterations per thread:\t%" PRIu64 "\n",iters_per_thread);
     ownprintf("Inner loop executions:\t%d\n", (int)(((double)realSize)/((double)test->stride*globalNumberOfThreads)));
-    ownprintf("Size (Byte):\t\t%" PRIu64 "\n",  realSize * test->bytes );
-    ownprintf("Size per thread:\t%" PRIu64 "\n", size_per_thread * test->bytes);
+    ownprintf("Size (Byte):\t\t%" PRIu64 "\n",  realSize * datatypesize * test->streams);
+    ownprintf("Size per thread:\t%" PRIu64 "\n", size_per_thread * datatypesize * test->streams);
     ownprintf("Number of Flops:\t%" PRIu64 "\n", (iters_per_thread * realSize *  test->flops));
     ownprintf("MFlops/s:\t\t%.2f\n",
             1.0E-06 * ((double) (iters_per_thread * realSize *  test->flops) /  time));
@@ -624,7 +790,12 @@ int main(int argc, char** argv)
     LIKWID_MARKER_CLOSE;
 #endif
 
+    if (test->dlhandle != NULL)
+    {
+        dynbench_close(test, compilepath);
+    }
+
     bdestroy(HLINE);
+    bdestroy(asmFile);
     return EXIT_SUCCESS;
 }
-
