@@ -5,13 +5,13 @@
  *
  *      Description:  Implementation of allocator module.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -96,13 +96,15 @@ allocator_allocateVector(
         int offset,
         DataType type,
         int stride,
-        bstring domainString)
+        bstring domainString,
+        int init_per_thread)
 {
     int i;
     size_t bytesize = 0;
     const AffinityDomain* domain = NULL;
     int errorCode;
     int elements = 0;
+    affinity_init();
 
     size_t typesize = allocator_dataTypeLength(type);
     bytesize = (size+offset) * typesize;
@@ -153,7 +155,7 @@ allocator_allocateVector(
     numberOfAllocatedVectors++;
 
     affinity_pinProcess(domain->processorList[0]);
-    printf("Allocate: Process running on core %d (Domain %s) - Vector length %llu/%llu Offset %d Alignment %llu\n",
+    printf("Allocate: Process running on hwthread %d (Domain %s) - Vector length %llu/%llu Offset %d Alignment %llu\n",
             affinity_processGetProcessorId(),
             bdata(domain->tag),
             LLU_CAST size,
@@ -161,48 +163,50 @@ allocator_allocateVector(
             offset,
             LLU_CAST elements);
 
-    switch ( type )
+    if (!init_per_thread)
     {
-        case INT:
-            {
-                int* sptr = (int*) (*ptr);
-                sptr += offset;
-
-                for ( uint64_t i=0; i < size; i++ )
+        switch ( type )
+        {
+            case INT:
                 {
-                    sptr[i] = 1;
+                    int* sptr = (int*) (*ptr);
+                    sptr += offset;
+
+                    for ( uint64_t i=0; i < size; i++ )
+                    {
+                        sptr[i] = 1;
+                    }
+                    *ptr = (void*) sptr;
+
                 }
-                *ptr = (void*) sptr;
+                break;
 
-            }
-            break;
-
-        case SINGLE:
-            {
-                float* sptr = (float*) (*ptr);
-                sptr += offset;
-
-                for ( uint64_t i=0; i < size; i++ )
+            case SINGLE:
                 {
-                    sptr[i] = 1.0;
+                    float* sptr = (float*) (*ptr);
+                    sptr += offset;
+
+                    for ( uint64_t i=0; i < size; i++ )
+                    {
+                        sptr[i] = 1.0;
+                    }
+                    *ptr = (void*) sptr;
+
                 }
-                *ptr = (void*) sptr;
+                break;
 
-            }
-            break;
-
-        case DOUBLE:
-            {
-                double* dptr = (double*) (*ptr);
-                dptr += offset;
-
-                for ( uint64_t i=0; i < size; i++ )
+            case DOUBLE:
                 {
-                    dptr[i] = 1.0;
+                    double* dptr = (double*) (*ptr);
+                    dptr += offset;
+
+                    for ( uint64_t i=0; i < size; i++ )
+                    {
+                        dptr[i] = 1.0;
+                    }
+                    *ptr = (void*) dptr;
                 }
-                *ptr = (void*) dptr;
-            }
-            break;
+                break;
+        }
     }
 }
-
