@@ -5,14 +5,14 @@
  *
  *      Description:  Benchmarking framework for likwid-bench
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
- *               Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *               Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -42,13 +42,15 @@
 #include <allocator.h>
 #include <threads.h>
 #include <barrier.h>
-#include <likwid.h>
+//#include <likwid.h>
+#include <likwid-marker.h>
 
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
 
 #define BARRIER   barrier_synchronize(&barr)
 
 #define EXECUTE(func)   \
+    LIKWID_MARKER_REGISTER("bench");  \
     BARRIER; \
     LIKWID_MARKER_START("bench");  \
     timer_start(&time); \
@@ -69,10 +71,11 @@ void*
 runTest(void* arg)
 {
     int threadId;
-    int offset;
+    size_t offset;
     size_t size;
     size_t vecsize;
     size_t i;
+    size_t j = 0;
     BarrierData barr;
     ThreadData* data;
     ThreadUserData* myData;
@@ -88,7 +91,7 @@ runTest(void* arg)
     /* Prepare ptrs for thread */
     vecsize = myData->size / data->numberOfThreads;
     size = myData->size / data->numberOfThreads;
-    
+
     size -= (size % myData->test->stride);
     myData->size = size;
     offset = data->threadId * size;
@@ -96,6 +99,17 @@ runTest(void* arg)
     if (size != vecsize && data->threadId == 0)
         printf("Sanitizing vector length to a multiple of the loop stride from %d elements (%d bytes) to %d elements (%d bytes)\n", vecsize, vecsize*myData->test->bytes, size, size*myData->test->bytes);
 
+    /* pin the thread */
+    likwid_pinThread(myData->processors[threadId]);
+    printf("Group: %d Thread %d Global Thread %d running on hwthread %d - Vector length %llu Offset %zd\n",
+            data->groupId,
+            threadId,
+            data->globalThreadId,
+            affinity_threadGetProcessorId(),
+            LLU_CAST size,
+            offset);
+    BARRIER;
+
     switch ( myData->test->type )
     {
         case SINGLE:
@@ -105,6 +119,13 @@ runTest(void* arg)
                 {
                     sptr = (float*) myData->streams[i];
                     sptr +=  offset;
+                    if (myData->init_per_thread)
+                    {
+                        for (j = 0; j < vecsize; j++)
+                        {
+                            sptr[j] = 1.0;
+                        }
+                    }
                     myData->streams[i] = (float*) sptr;
                 }
             }
@@ -116,6 +137,13 @@ runTest(void* arg)
                 {
                     sptr = (int*) myData->streams[i];
                     sptr +=  offset;
+                    if (myData->init_per_thread)
+                    {
+                        for (j = 0; j < vecsize; j++)
+                        {
+                            sptr[j] = 1;
+                        }
+                    }
                     myData->streams[i] = (int*) sptr;
                 }
             }
@@ -127,21 +155,19 @@ runTest(void* arg)
                 {
                     dptr = (double*) myData->streams[i];
                     dptr +=  offset;
+                    if (myData->init_per_thread)
+                    {
+                        for (j = 0; j < vecsize; j++)
+                        {
+                            dptr[j] = 1.0;
+                        }
+                    }
                     myData->streams[i] = (double*) dptr;
                 }
             }
             break;
     }
 
-    /* pin the thread */
-    likwid_pinThread(myData->processors[threadId]);
-    printf("Group: %d Thread %d Global Thread %d running on core %d - Vector length %llu Offset %d\n",
-            data->groupId,
-            threadId,
-            data->globalThreadId,
-            affinity_threadGetProcessorId(),
-            LLU_CAST size,
-            offset);
     BARRIER;
 
     /* Up to 10 streams the following registers are used for Array ptr:
@@ -456,9 +482,9 @@ void*
 getIterSingle(void* arg)
 {
     int threadId = 0;
-    int offset = 0;
+    size_t offset = 0;
     size_t size = 0, vecsize = 0;
-    size_t i;
+    size_t i, j;
     ThreadData* data;
     ThreadUserData* myData;
     TimerData time;
@@ -471,9 +497,9 @@ getIterSingle(void* arg)
     threadId = data->threadId;
 
     //size = myData->size - (myData->size % myData->test->stride);
-    vecsize = myData->size;
+    vecsize = myData->size / data->numberOfThreads;
     size = myData->size / data->numberOfThreads;
-    
+
     size -= (size % myData->test->stride);
     offset = data->threadId * size;
 
@@ -483,6 +509,64 @@ getIterSingle(void* arg)
     printf("Automatic iteration count detection:");
 #endif
 
+    switch ( myData->test->type )
+    {
+        case SINGLE:
+            {
+                float* sptr;
+                for (i=0; i <  myData->test->streams; i++)
+                {
+                    sptr = (float*) myData->streams[i];
+                    sptr +=  offset;
+                    if (myData->init_per_thread)
+                    {
+                        for (j = 0; j < vecsize; j++)
+                        {
+                            sptr[j] = 1.0;
+                        }
+                    }
+                    myData->streams[i] = (float*) sptr;
+                }
+            }
+            break;
+        case INT:
+            {
+                int* sptr;
+                for (i=0; i <  myData->test->streams; i++)
+                {
+                    sptr = (int*) myData->streams[i];
+                    sptr +=  offset;
+                    if (myData->init_per_thread)
+                    {
+                        for (j = 0; j < vecsize; j++)
+                        {
+                            sptr[j] = 1;
+                        }
+                    }
+                    myData->streams[i] = (int*) sptr;
+                }
+            }
+            break;
+        case DOUBLE:
+            {
+                double* dptr;
+                for (i=0; i <  myData->test->streams; i++)
+                {
+                    dptr = (double*) myData->streams[i];
+                    dptr +=  offset;
+                    if (myData->init_per_thread)
+                    {
+                        for (j = 0; j < vecsize; j++)
+                        {
+                            dptr[j] = 1.0;
+                        }
+                    }
+                    myData->streams[i] = (double*) dptr;
+                }
+            }
+            break;
+    }
+
     switch ( myData->test->streams ) {
         case STREAM_1:
             MEASURE(func(size,myData->streams[0]));
@@ -771,4 +855,3 @@ getIterSingle(void* arg)
 #endif
     return NULL;
 }
-
