@@ -12,13 +12,13 @@ my $lc = ' *';
 
 #my $VERSION   = '<VERSION>';
 #my $DATE   = '<DATE>';
-my $VERSION   = '4.3.4';
-my $DATE   = '05.04.2019';
-my $YEAR  = '2019';
-my $AUTHOR = 'RRZE, University Erlangen-Nuremberg';
+my $VERSION   = '5.2.1';
+my $DATE   = '03.12.2021';
+my $YEAR  = '2021';
+my $AUTHOR = 'NHR@FAU, University Erlangen-Nuremberg';
 my $LICENSE = 'gpl';
 
-my @SKIPLIST = ('ghash.c','ghash.h','loadData.S','bstrlib.c','bstrlib.h');
+my @SKIPLIST = ('ghash.c','ghash.h','loadData.S','loadDataARM.S','bstrlib.c','bstrlib.h', 'calculator_stack.h', 'calculator_stack.c');
 
 sub print_copyright
 {
@@ -85,7 +85,7 @@ sub wanted
         $filename = $_;
     }
 
-    if (($filename =~ /^\./) or (-d $filename)) {
+    if (($filename =~ /^\./) or (-d $filename) or (-l $filename)) {
         return;
     }
 
