@@ -5,13 +5,13 @@
  *
  *      Description:  Interface to the access daemon for the access module.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
- *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -38,6 +38,7 @@
 #include <unistd.h>
 #include <signal.h>
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -52,6 +53,10 @@
 #include <configuration.h>
 #include <affinity.h>
 
+#if defined(__x86_64__) || defined(__i386__)
+#include <access_x86_rdpmc.h>
+#endif
+
 #define gettid() syscall(SYS_gettid)
 
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
@@ -228,6 +233,9 @@ access_client_init(int cpu_id)
             pthread_mutex_init(&cpuLocks[i], NULL);
         }
     }
+#if defined(__x86_64__) || defined(__i386__)
+    access_x86_rdpmc_init(cpu_id);
+#endif
     if (masterPid != 0 && gettid() == masterPid)
     {
         return 0;
@@ -311,6 +319,12 @@ access_client_read(PciDeviceIndex dev, const int cpu_id, uint32_t reg, uint64_t
         record.cpu = affinity_thread2socket_lookup[cpu_id];
         record.device = dev;
     }
+    else
+    {
+#if defined(__x86_64__) || defined(__i386__)
+        access_x86_rdpmc_read(cpu_id, reg, data);
+#endif
+    }
     if (socket != -1)
     {
         record.reg = reg;
@@ -443,11 +457,22 @@ access_client_finalize(int cpu_id)
     {
         memset(&record, 0, sizeof(AccessDataRecord));
         record.type = DAEMON_EXIT;
+        record.cpu = cpu_id;
         CHECK_ERROR(write(cpuSockets[cpu_id], &record, sizeof(AccessDataRecord)),socket write failed);
+        if (cpuSockets[cpu_id] == globalSocket)
+        {
+            globalSocket = -1;
+        }
         CHECK_ERROR(close(cpuSockets[cpu_id]),socket close failed);
         cpuSockets[cpu_id] = -1;
-        daemon_pids[cpu_id] = 0;
-        nr_daemons--;
+        if (daemon_pids[cpu_id] != 0)
+        {
+            int status = 0;
+            waitpid(daemon_pids[cpu_id], &status, 0);
+            daemon_pids[cpu_id] = 0;
+            nr_daemons--;
+        }
+
         cpuSockets_open--;
     }
     if (cpuSockets_open == 0)
@@ -455,6 +480,9 @@ access_client_finalize(int cpu_id)
         globalSocket = -1;
     }
     masterPid = 0;
+#if defined(__x86_64__) || defined(__i386__)
+    access_x86_rdpmc_finalize(cpu_id);
+#endif
 }
 
 int
@@ -488,6 +516,10 @@ access_client_check(PciDeviceIndex dev, int cpu_id)
         {
             return 1;
         }
+        else
+        {
+            DEBUG_PRINT(DEBUGLEV_DEVELOP, Device check for dev %d on CPU %d with accessDaemon failed: %s\n, dev, cpu_id, access_client_strerror(record.errorcode));
+        }
     }
     return 0;
 }
@@ -496,12 +528,31 @@ void __attribute__((destructor (104))) close_access_client(void)
 {
     if (cpuSockets)
     {
+        for (int i = 0; i < cpuid_topology.numHWThreads; i++)
+        {
+            if (cpuSockets[i] > 0)
+            {
+                close(cpuSockets[i]);
+                cpuSockets[i] = -1;
+                cpuSockets_open--;
+            }
+        }
         free(cpuSockets);
         cpuSockets = NULL;
         cpuSockets_open = 0;
     }
     if (daemon_pids)
     {
+        for (int i = 0; i < cpuid_topology.numHWThreads; i++)
+        {
+            if (daemon_pids[i] != 0)
+            {
+                int status = 0;
+                waitpid(daemon_pids[i], &status, 0);
+                daemon_pids[i] = 0;
+                nr_daemons--;
+            }
+        }
         free(daemon_pids);
         daemon_pids = NULL;
         free(daemon_pinned);
@@ -510,8 +561,11 @@ void __attribute__((destructor (104))) close_access_client(void)
     }
     if (cpuLocks)
     {
+        for (int i = 0; i < cpuid_topology.numHWThreads; i++)
+        {
+            pthread_mutex_destroy(&cpuLocks[i]);
+        }
         free(cpuLocks);
         cpuLocks = NULL;
     }
 }
-
