@@ -5,13 +5,13 @@
  *
  *      Description:  Configuration file module.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
- *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -81,9 +81,9 @@ default_configuration(void)
         goto use_hardcoded;
     }
     ret = getline(&fptr, &len, fp);
+    pclose(fp);
     if (ret < 0)
     {
-        pclose(fp);
         if (fptr)
             free(fptr);
         goto use_hardcoded;
@@ -99,12 +99,10 @@ default_configuration(void)
     else
     {
         fprintf(stderr, "Found access daemon at %s but it is not executable, using compiled in daemon path.\n", fptr);
-        fclose(fp);
         if (fptr)
             free(fptr);
         goto use_hardcoded;
     }
-    fclose(fp);
 #else
     config.daemonMode = ACCESSMODE_PERF;
 #endif
@@ -361,4 +359,3 @@ config_setGroupPath(const char* path)
     printf("Given path is no directory\n");
     return -ENOTDIR;
 }
-
