@@ -9,13 +9,13 @@
  *                  Allows to turn on and off the Hardware prefetcher
  *                  available.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -92,6 +92,7 @@ cpuFeatures_update(int cpu)
     {
         fprintf(stderr, "Cannot read register 0x%X on cpu %d: err %d\n", MSR_IA32_MISC_ENABLE, cpu, ret);
     }
+
     /*cpuFeatureFlags.fastStrings = 0;
     cpuFeatureFlags.thermalControl = 0;
     cpuFeatureFlags.perfMonitoring = 0;
@@ -163,7 +164,10 @@ cpuFeatures_update(int cpu)
              (cpuid_info.model == SKYLAKE2) ||
              (cpuid_info.model == SKYLAKEX) ||
              (cpuid_info.model == KABYLAKE1) ||
-             (cpuid_info.model == KABYLAKE2))
+             (cpuid_info.model == KABYLAKE2) ||
+             (cpuid_info.model == CANNONLAKE) ||
+             (cpuid_info.model == COMETLAKE1) ||
+             (cpuid_info.model == COMETLAKE2))
     {
         TEST_FLAG_INV(FEAT_TURBO_MODE,38);
     }
@@ -192,7 +196,15 @@ cpuFeatures_update(int cpu)
             (cpuid_info.model == SKYLAKEX) ||
             (cpuid_info.model == KABYLAKE1) ||
             (cpuid_info.model == KABYLAKE2) ||
-            (cpuid_info.model == ATOM_SILVERMONT_GOLD))
+            (cpuid_info.model == ICELAKE1) ||
+            (cpuid_info.model == ICELAKE2) ||
+            (cpuid_info.model == ROCKETLAKE) ||
+            (cpuid_info.model == ICELAKEX1) ||
+            (cpuid_info.model == ICELAKEX2) ||
+            (cpuid_info.model == ATOM_SILVERMONT_GOLD) ||
+            (cpuid_info.model == CANNONLAKE) ||
+            (cpuid_info.model == COMETLAKE1) ||
+            (cpuid_info.model == COMETLAKE2))
     {
         ret = HPMread(cpu, MSR_DEV, MSR_PREFETCH_ENABLE, &flags);
         if (ret != 0)
@@ -366,6 +378,14 @@ cpuFeatures_enable(int cpu, CpuFeature type, int print)
             (cpuid_info.model == SKYLAKEX) ||
             (cpuid_info.model == KABYLAKE1) ||
             (cpuid_info.model == KABYLAKE2) ||
+            (cpuid_info.model == CANNONLAKE) ||
+            (cpuid_info.model == COMETLAKE1) ||
+            (cpuid_info.model == COMETLAKE2) ||
+            (cpuid_info.model == ICELAKE1) ||
+            (cpuid_info.model == ICELAKE2) ||
+            (cpuid_info.model == ROCKETLAKE) ||
+            (cpuid_info.model == ICELAKEX1) ||
+            (cpuid_info.model == ICELAKEX2) ||
             (cpuid_info.model == ATOM_SILVERMONT_GOLD))
     {
         reg = MSR_PREFETCH_ENABLE;
@@ -522,6 +542,14 @@ cpuFeatures_disable(int cpu, CpuFeature type, int print)
             (cpuid_info.model == SKYLAKEX) ||
             (cpuid_info.model == KABYLAKE1) ||
             (cpuid_info.model == KABYLAKE2) ||
+            (cpuid_info.model == CANNONLAKE) ||
+            (cpuid_info.model == COMETLAKE1) ||
+            (cpuid_info.model == COMETLAKE2) ||
+            (cpuid_info.model == ICELAKE1) ||
+            (cpuid_info.model == ICELAKE2) ||
+            (cpuid_info.model == ROCKETLAKE) ||
+            (cpuid_info.model == ICELAKEX1) ||
+            (cpuid_info.model == ICELAKEX2) ||
             (cpuid_info.model == ATOM_SILVERMONT_GOLD))
     {
         reg = MSR_PREFETCH_ENABLE;
