@@ -6,13 +6,13 @@
  *      Description:  Header File bitUtil Module.
  *                    Helper routines for dealing with bit manipulations
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -41,7 +41,7 @@ extern uint32_t getBitFieldWidth(uint32_t number);
 #define setBit(reg,bit)  (reg) |= (1ULL<<(bit))
 #define clearBit(reg,bit) (reg) &= ~(1ULL<<(bit))
 #define toggleBit(reg,bit) (reg) ^= (1ULL<<(bit))
-#define testBit(reg,bit) (reg) & (1ULL<<(bit))
+#define testBit(reg,bit) ((reg) & (1ULL<<(bit)))
 
 #define bitMask_init(_mask)  \
     (_mask).mask[0] = 0ULL; \
