@@ -5,13 +5,13 @@
  *
  *      Description:  Common macro definition for CPUID instruction
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
- *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -34,6 +34,8 @@
  * Kernel version 3.19
  * File: arch/x86/boot/cpuflags.c
 */
+#if !defined(__ARM_ARCH_7A__) && !defined(__ARM_ARCH_8A) && !defined(_ARCH_PPC)
+
 
 #if defined(__i386__) && defined(__PIC__)
 # define EBX_REG "=r"
@@ -54,4 +56,8 @@
     __asm__ volatile("cpuid" : "=a" (eax), "=c" (ecx), "=d" (edx), EBX_REG (ebx) : "a" (eax), "c" (ecx) );
 #endif
 
-#endif
+#else /* ARCH Filters */
+#define CPUID(eax,ebx,ecx,edx)
+#endif /* ARCH Filters */
+
+#endif /* LIKWID_CPUID_H */
