@@ -5,13 +5,13 @@
  *
  *      Description:  Header File Locking primitive Module
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -91,7 +91,7 @@ lock_check(void)
         }
     }
 
-    if (lock_handle)
+    if (lock_handle > 0)
     {
         close(lock_handle);
     }
