@@ -5,13 +5,13 @@
  *
  *      Description:  Header File procfs/sysfs NUMA backend
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
- *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -33,5 +33,6 @@
 extern int proc_numa_init(void);
 extern void proc_numa_membind(void* ptr, size_t size, int domainId);
 extern void proc_numa_setInterleaved(const int* processorList, int numberOfProcessors);
+extern void proc_numa_setMembind(const int* processorList, int numberOfProcessors);
 
 #endif
