@@ -5,14 +5,14 @@
  *
  *      Description:  Types file for pci module.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
- *                Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -55,8 +55,14 @@ typedef enum {
     PCI_IMC_DEVICE_0_CH_1,
     PCI_IMC_DEVICE_0_CH_2,
     PCI_IMC_DEVICE_0_CH_3,
+    PCI_IMC_DEVICE_1_CH_0,
+    PCI_IMC_DEVICE_1_CH_1,
+    PCI_IMC_DEVICE_1_CH_2,
+    PCI_IMC_DEVICE_1_CH_3,
     PCI_HA_DEVICE_0,
     PCI_HA_DEVICE_1,
+    PCI_HA_DEVICE_2,
+    PCI_HA_DEVICE_3,
     PCI_QPI_DEVICE_PORT_0,
     PCI_QPI_DEVICE_PORT_1,
     PCI_QPI_DEVICE_PORT_2,
@@ -66,10 +72,6 @@ typedef enum {
     PCI_QPI_MISC_DEVICE_PORT_0,
     PCI_QPI_MISC_DEVICE_PORT_1,
     PCI_QPI_MISC_DEVICE_PORT_2,
-    PCI_IMC_DEVICE_1_CH_0,
-    PCI_IMC_DEVICE_1_CH_1,
-    PCI_IMC_DEVICE_1_CH_2,
-    PCI_IMC_DEVICE_1_CH_3,
     PCI_IRP_DEVICE,
     PCI_EDC0_UCLK_DEVICE,
     PCI_EDC0_DCLK_DEVICE,
@@ -87,6 +89,18 @@ typedef enum {
     PCI_EDC6_DCLK_DEVICE,
     PCI_EDC7_UCLK_DEVICE,
     PCI_EDC7_DCLK_DEVICE,
+    MMIO_IMC_DEVICE_0_CH_0,
+    MMIO_IMC_DEVICE_0_CH_1,
+    MMIO_IMC_DEVICE_1_CH_0,
+    MMIO_IMC_DEVICE_1_CH_1,
+    MMIO_IMC_DEVICE_2_CH_0,
+    MMIO_IMC_DEVICE_2_CH_1,
+    MMIO_IMC_DEVICE_3_CH_0,
+    MMIO_IMC_DEVICE_3_CH_1,
+    MMIO_IMC_DEVICE_0_FREERUN,
+    MMIO_IMC_DEVICE_1_FREERUN,
+    MMIO_IMC_DEVICE_2_FREERUN,
+    MMIO_IMC_DEVICE_3_FREERUN,
     MAX_NUM_PCI_DEVICES
 } PciDeviceIndex;
 
