@@ -5,13 +5,13 @@
  *
  *      Description:  Header File of perfmon module for Intel Broadwell.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
- *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -39,6 +39,7 @@
 #include <limits.h>
 #include <topology.h>
 #include <access.h>
+#include <voltage.h>
 
 
 static int perfmon_numCountersBroadwell = NUM_COUNTERS_BROADWELL;
@@ -1092,6 +1093,7 @@ int perfmon_setupCounterThread_broadwell(
 
             case POWER:
             case THERMAL:
+            case VOLTAGE:
                 break;
 
             case UBOX:
@@ -1137,6 +1139,11 @@ int perfmon_setupCounterThread_broadwell(
                 break;
 
             case MBOX0:
+                if (!cpuid_info.supportClientmem)
+                {
+                    bdw_mbox_setup(cpu_id, index, event);
+                }
+                break;
             case MBOX1:
             case MBOX2:
             case MBOX3:
@@ -1256,7 +1263,22 @@ int perfmon_startCountersThread_broadwell(int thread_id, PerfmonEventSet* eventS
                         eventSet->events[i].threadCounter[thread_id].startData = field64(tmp, 0, box_map[type].regWidth);
                     }
                     break;
-
+                case MBOX0:
+                    if (haveLock)
+                    {
+                        if (!cpuid_info.supportClientmem)
+                        {
+                            VERBOSEPRINTREG(cpu_id, counter1, 0x0ULL, CLEAR_BOX)
+                            CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, dev, counter1, 0x0ULL));
+                        }
+                        else
+                        {
+                            CHECK_MSR_READ_ERROR(HPMread(cpu_id, dev, counter1, &tmp));
+                            eventSet->events[i].threadCounter[thread_id].startData = field64(tmp, 0, box_map[type].regWidth);
+                            VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST tmp, READ_MBOX)
+                        }
+                    }
+                    break;
                 case WBOX0FIX:
                     if (haveLock)
                     {
@@ -1286,7 +1308,7 @@ int perfmon_startCountersThread_broadwell(int thread_id, PerfmonEventSet* eventS
         }
     }
 
-    BDW_UNFREEZE_UNCORE_AND_RESET_CTR;
+    BDW_UNFREEZE_UNCORE;
 
     if (MEASURE_CORE(eventSet))
     {
@@ -1475,6 +1497,10 @@ int perfmon_stopCountersThread_broadwell(int thread_id, PerfmonEventSet* eventSe
                 case THERMAL:
                     CHECK_TEMP_READ_ERROR(thermal_read(cpu_id,(uint32_t*)&counter_result));
                     break;
+                
+                case VOLTAGE:
+                    CHECK_TEMP_READ_ERROR(voltage_read(cpu_id, &counter_result));
+                    break;
 
                 case BBOX0:
                 case BBOX1:
@@ -1483,6 +1509,27 @@ int perfmon_stopCountersThread_broadwell(int thread_id, PerfmonEventSet* eventSe
                     break;
 
                 case MBOX0:
+                case MBOX0TMP:
+                    if (haveLock)
+                    {
+                        if (!cpuid_info.supportClientmem)
+                        {
+                            bdw_uncore_read(cpu_id, index, event, current, overflows,
+                                            FREEZE_FLAG_CLEAR_CTR, ovf_offset, getCounterTypeOffset(index)+1);
+                            counter_result = *current;
+                        }
+                        else
+                        {
+                            CHECK_MSR_READ_ERROR(HPMread(cpu_id, PCI_IMC_DEVICE_0_CH_0, counter1, &counter_result));
+                            if (counter_result < eventSet->events[i].threadCounter[thread_id].counterData)
+                            {
+                                VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, OVERFLOW_CLIENTMEM)
+                                eventSet->events[i].threadCounter[thread_id].overflows++;
+                            }
+                        }
+                        VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, READ_MBOX)
+                    }
+                    break;
                 case MBOX1:
                 case MBOX2:
                 case MBOX3:
@@ -1688,8 +1735,32 @@ int perfmon_readCountersThread_broadwell(int thread_id, PerfmonEventSet* eventSe
                     CHECK_TEMP_READ_ERROR(thermal_read(cpu_id,(uint32_t*)&counter_result));
                     *current = field64(counter_result, 0, box_map[type].regWidth);
                     break;
+                
+                case VOLTAGE:
+                    CHECK_TEMP_READ_ERROR(voltage_read(cpu_id, &counter_result));
+                    *current = field64(counter_result, 0, box_map[type].regWidth);
+                    break;
 
                 case MBOX0:
+                case MBOX0TMP:
+                    if (!cpuid_info.supportClientmem)
+                    {
+                        bdw_uncore_read(cpu_id, index, event, current, overflows,
+                                0, ovf_offset, getCounterTypeOffset(index)+1);
+                    }
+                    else if (haveLock)
+                    {
+                        CHECK_MSR_READ_ERROR(HPMread(cpu_id, dev, counter1, &counter_result));
+                        if (counter_result < eventSet->events[i].threadCounter[thread_id].counterData)
+                        {
+                            VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, OVERFLOW_CLIENTMEM)
+                            eventSet->events[i].threadCounter[thread_id].overflows++;
+                        }
+                        *current = counter_result;
+                        VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, READ_MBOX)
+                    }
+                    break;
+
                 case MBOX1:
                 case MBOX2:
                 case MBOX3:
