@@ -5,14 +5,14 @@
  *
  *      Description:  Header File of perfmon module for Intel Ivy Bridge.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
- *                Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -66,7 +66,7 @@ int perfmon_init_ivybridge(int cpu_id)
     lock_acquire((int*) &socket_lock[affinity_thread2socket_lookup[cpu_id]], cpu_id);
     lock_acquire((int*) &tile_lock[affinity_thread2core_lookup[cpu_id]], cpu_id);
     HPMwrite(cpu_id, MSR_DEV, MSR_PEBS_ENABLE, 0x0ULL);
-    if ((cpuid_info.model == IVYBRIDGE_EP))
+    if (cpuid_info.model == IVYBRIDGE_EP)
     {
         ivy_cbox_setup = ivbep_cbox_setup;
         ivb_did_cbox_test = 1;
@@ -791,6 +791,11 @@ int perfmon_setupCounterThread_ivybridge(
                 break;
 
             case MBOX0:
+                if (!cpuid_info.supportClientmem)
+                {
+                    ivb_pci_box_setup(cpu_id, index, event);
+                }
+                break;
             case MBOX1:
             case MBOX2:
             case MBOX3:
@@ -1205,6 +1210,23 @@ int perfmon_stopCountersThread_ivybridge(int thread_id, PerfmonEventSet* eventSe
                     break;
 
                 case MBOX0:
+                    if (haveLock)
+                    {
+                        if (!cpuid_info.supportClientmem)
+                        {
+                            counter_result = ivb_uncore_read(cpu_id, index, event, FREEZE_FLAG_CLEAR_CTR);
+                            ivb_uncore_overflow(cpu_id, index, event, overflows, counter_result,
+                                                *current, box_map[type].ovflOffset, getCounterTypeOffset(index)+1);
+                        }
+                        else
+                        {
+                            uint64_t tmp = 0x0;
+                            CHECK_MSR_READ_ERROR(HPMread(cpu_id, dev, counter1, &tmp));
+                            eventSet->events[i].threadCounter[thread_id].startData = field64(tmp, 0, box_map[type].regWidth);
+                            VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST tmp, READ_MBOX)
+                        }
+                    }
+                    break;
                 case MBOX1:
                 case MBOX2:
                 case MBOX3:
@@ -1212,9 +1234,12 @@ int perfmon_stopCountersThread_ivybridge(int thread_id, PerfmonEventSet* eventSe
                 case MBOX5:
                 case MBOX6:
                 case MBOX7:
-                    counter_result = ivb_uncore_read(cpu_id, index, event, FREEZE_FLAG_CLEAR_CTR);
-                    ivb_uncore_overflow(cpu_id, index, event, overflows, counter_result,
-                                        *current, box_map[type].ovflOffset, getCounterTypeOffset(index)+1);
+                    if (haveLock)
+                    {
+                        counter_result = ivb_uncore_read(cpu_id, index, event, FREEZE_FLAG_CLEAR_CTR);
+                        ivb_uncore_overflow(cpu_id, index, event, overflows, counter_result,
+                                            *current, box_map[type].ovflOffset, getCounterTypeOffset(index)+1);
+                    }
                     break;
 
 
@@ -1226,9 +1251,12 @@ int perfmon_stopCountersThread_ivybridge(int thread_id, PerfmonEventSet* eventSe
                 case MBOX5FIX:
                 case MBOX6FIX:
                 case MBOX7FIX:
-                    counter_result = ivb_uncore_read(cpu_id, index, event, FREEZE_FLAG_CLEAR_CTR);
-                    ivb_uncore_overflow(cpu_id, index, event, overflows, counter_result,
-                                        *current, box_map[type].ovflOffset, 0);
+                    if (haveLock)
+                    {
+                        counter_result = ivb_uncore_read(cpu_id, index, event, FREEZE_FLAG_CLEAR_CTR);
+                        ivb_uncore_overflow(cpu_id, index, event, overflows, counter_result,
+                                            *current, box_map[type].ovflOffset, 0);
+                    }
                     break;
                 case WBOX0FIX:
                 case WBOX1FIX:
@@ -1276,9 +1304,12 @@ int perfmon_stopCountersThread_ivybridge(int thread_id, PerfmonEventSet* eventSe
                 case RBOX1:
                 case RBOX2:
                 case IBOX0:
-                    counter_result = ivb_uncore_read(cpu_id, index, event, FREEZE_FLAG_CLEAR_CTR);
-                    ivb_uncore_overflow(cpu_id, index, event, overflows, counter_result,
-                                        *current, box_map[type].ovflOffset, getCounterTypeOffset(index));
+                    if (haveLock)
+                    {
+                        counter_result = ivb_uncore_read(cpu_id, index, event, FREEZE_FLAG_CLEAR_CTR);
+                        ivb_uncore_overflow(cpu_id, index, event, overflows, counter_result,
+                                            *current, box_map[type].ovflOffset, getCounterTypeOffset(index));
+                    }
                     break;
 
                 default:
@@ -1420,6 +1451,27 @@ int perfmon_readCountersThread_ivybridge(int thread_id, PerfmonEventSet* eventSe
                     break;
 
                 case MBOX0:
+                    if (haveLock)
+                    {
+                        if (!cpuid_info.supportClientmem)
+                        {
+                            counter_result = ivb_uncore_read(cpu_id, index, event, FREEZE_FLAG_ONLYFREEZE);
+                            ivb_uncore_overflow(cpu_id, index, event, overflows, counter_result,
+                                                *current, box_map[type].ovflOffset, getCounterTypeOffset(index)+1);
+                        }
+                        else
+                        {
+                            CHECK_MSR_READ_ERROR(HPMread(cpu_id, dev, counter1, &counter_result));
+                            if (counter_result < eventSet->events[i].threadCounter[thread_id].counterData)
+                            {
+                                VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, OVERFLOW_CLIENTMEM)
+                                eventSet->events[i].threadCounter[thread_id].overflows++;
+                            }
+                            *current = counter_result;
+                            VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, READ_MBOX)
+                        }
+                    }
+                    break;
                 case MBOX1:
                 case MBOX2:
                 case MBOX3:
@@ -1427,9 +1479,12 @@ int perfmon_readCountersThread_ivybridge(int thread_id, PerfmonEventSet* eventSe
                 case MBOX5:
                 case MBOX6:
                 case MBOX7:
-                    counter_result = ivb_uncore_read(cpu_id, index, event, FREEZE_FLAG_ONLYFREEZE);
-                    ivb_uncore_overflow(cpu_id, index, event, overflows, counter_result,
-                                        *current, box_map[type].ovflOffset, getCounterTypeOffset(index)+1);
+                    if (haveLock)
+                    {
+                        counter_result = ivb_uncore_read(cpu_id, index, event, FREEZE_FLAG_ONLYFREEZE);
+                        ivb_uncore_overflow(cpu_id, index, event, overflows, counter_result,
+                                            *current, box_map[type].ovflOffset, getCounterTypeOffset(index)+1);
+                    }
                     break;
 
 
@@ -1441,9 +1496,12 @@ int perfmon_readCountersThread_ivybridge(int thread_id, PerfmonEventSet* eventSe
                 case MBOX5FIX:
                 case MBOX6FIX:
                 case MBOX7FIX:
-                    counter_result = ivb_uncore_read(cpu_id, index, event, FREEZE_FLAG_ONLYFREEZE);
-                    ivb_uncore_overflow(cpu_id, index, event, overflows, counter_result,
-                                        *current, box_map[type].ovflOffset, 0);
+                    if (haveLock)
+                    {
+                        counter_result = ivb_uncore_read(cpu_id, index, event, FREEZE_FLAG_ONLYFREEZE);
+                        ivb_uncore_overflow(cpu_id, index, event, overflows, counter_result,
+                                            *current, box_map[type].ovflOffset, 0);
+                    }
                     break;
                 case WBOX0FIX:
                 case WBOX1FIX:
@@ -1458,9 +1516,12 @@ int perfmon_readCountersThread_ivybridge(int thread_id, PerfmonEventSet* eventSe
                     break;
 
                 case IBOX1:
-                    counter_result = ivb_uncore_read(cpu_id, index, event, FREEZE_FLAG_ONLYFREEZE);
-                    ivb_uncore_overflow(cpu_id, index, event, overflows, counter_result,
-                                        *current, -1, getCounterTypeOffset(index)+2);
+                    if (haveLock)
+                    {
+                        counter_result = ivb_uncore_read(cpu_id, index, event, FREEZE_FLAG_ONLYFREEZE);
+                        ivb_uncore_overflow(cpu_id, index, event, overflows, counter_result,
+                                            *current, -1, getCounterTypeOffset(index)+2);
+                    }
                     break;
 
                 case SBOX0:
@@ -1491,9 +1552,12 @@ int perfmon_readCountersThread_ivybridge(int thread_id, PerfmonEventSet* eventSe
                 case RBOX1:
                 case RBOX2:
                 case IBOX0:
-                    counter_result = ivb_uncore_read(cpu_id, index, event, FREEZE_FLAG_ONLYFREEZE);
-                    ivb_uncore_overflow(cpu_id, index, event, overflows, counter_result,
-                                        *current, box_map[type].ovflOffset, getCounterTypeOffset(index));
+                    if (haveLock)
+                    {
+                        counter_result = ivb_uncore_read(cpu_id, index, event, FREEZE_FLAG_ONLYFREEZE);
+                        ivb_uncore_overflow(cpu_id, index, event, overflows, counter_result,
+                                            *current, box_map[type].ovflOffset, getCounterTypeOffset(index));
+                    }
                     break;
 
                 default:
@@ -1587,10 +1651,20 @@ int perfmon_finalizeCountersThread_ivybridge(int thread_id, PerfmonEventSet* eve
     }
     if (haveLock && MEASURE_UNCORE(eventSet))
     {
-        VERBOSEPRINTREG(cpu_id, MSR_UNC_U_PMON_GLOBAL_STATUS, LLU_CAST 0x0ULL, CLEAR_UNCORE_OVF)
-        CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, MSR_DEV, MSR_UNC_U_PMON_GLOBAL_STATUS, 0x0ULL));
-        VERBOSEPRINTREG(cpu_id, MSR_UNC_U_PMON_GLOBAL_CTL, LLU_CAST 0x0ULL, CLEAR_UNCORE_CTRL)
-        CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, MSR_DEV, MSR_UNC_U_PMON_GLOBAL_CTL, 0x0ULL));
+        if (cpuid_info.model == IVYBRIDGE_EP)
+        {
+            VERBOSEPRINTREG(cpu_id, MSR_UNC_U_PMON_GLOBAL_STATUS, LLU_CAST 0x0ULL, CLEAR_UNCORE_OVF)
+            CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, MSR_DEV, MSR_UNC_U_PMON_GLOBAL_STATUS, 0x0ULL));
+            VERBOSEPRINTREG(cpu_id, MSR_UNC_U_PMON_GLOBAL_CTL, LLU_CAST 0x0ULL, CLEAR_UNCORE_CTRL)
+            CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, MSR_DEV, MSR_UNC_U_PMON_GLOBAL_CTL, 0x0ULL));
+        }
+        else if (cpuid_info.model == IVYBRIDGE)
+        {
+            VERBOSEPRINTREG(cpu_id, MSR_UNC_PERF_GLOBAL_OVF_CTRL, LLU_CAST 0x0ULL, CLEAR_UNCORE_OVF)
+            CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, MSR_DEV, MSR_UNC_PERF_GLOBAL_OVF_CTRL, 0x0ULL));
+            VERBOSEPRINTREG(cpu_id, MSR_UNC_PERF_GLOBAL_CTRL, LLU_CAST 0x0ULL, CLEAR_UNCORE_CTRL)
+            CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, MSR_DEV, MSR_UNC_PERF_GLOBAL_CTRL, 0x0ULL));
+        }
         for (int i=UNCORE;i<NUM_UNITS;i++)
         {
             if (TESTTYPE(eventSet, i) && box_map[i].ctrlRegister != 0x0)
