@@ -5,13 +5,13 @@
  *
  *      Description:  Header File of perfmon module for perf_event kernel interface.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
- *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -43,9 +43,12 @@
 extern char** translate_types;
 static int** cpu_event_fds = NULL;
 static int active_cpus = 0;
-static int paranoid_level = -1;
-static int informed_paranoid = 0;
+static int perf_event_initialized = 0;
+/*static int informed_paranoid = 0;*/
 static int running_group = -1;
+static int perf_event_num_cpus = 0;
+static int perf_disable_uncore = 0;
+static int perf_event_paranoid = -1;
 
 static long
 perf_event_open(struct perf_event_attr *hw_event, pid_t pid,
@@ -58,40 +61,42 @@ perf_event_open(struct perf_event_attr *hw_event, pid_t pid,
     return ret;
 }
 
+int perfevent_paranoid_value()
+{
+    FILE* fd;
+    int paranoid = 3;
+    char buff[100];
+    fd = fopen("/proc/sys/kernel/perf_event_paranoid", "r");
+    if (fd == NULL)
+    {
+        errno = EPERM;
+        ERROR_PRINT(Linux kernel has no perf_event support. Cannot access /proc/sys/kernel/perf_event_paranoid);
+        return paranoid;
+    }
+    size_t read = fread(buff, sizeof(char), 100, fd);
+    if (read > 0)
+    {
+        paranoid = atoi(buff);
+    }
+    fclose(fd);
+    return paranoid;
+}
+
 int perfmon_init_perfevent(int cpu_id)
 {
-    size_t read;
-    int paranoid = -1;
-    char buff[100];
-    FILE* fd;
-    if (!informed_paranoid)
+    perf_event_paranoid = perfevent_paranoid_value();
+    if (getuid() != 0 && perf_event_paranoid > 2)
     {
-        fd = fopen("/proc/sys/kernel/perf_event_paranoid", "r");
-        if (fd == NULL)
-        {
-            fprintf(stderr, "ERROR: Linux kernel has no perf_event support\n");
-            fprintf(stderr, "ERROR: Cannot open file /proc/sys/kernel/perf_event_paranoid\n");
-            fclose(fd);
-            exit(EXIT_FAILURE);
-        }
-        read = fread(buff, sizeof(char), 100, fd);
-        if (read > 0)
-        {
-            paranoid_level = atoi(buff);
-        }
-        fclose(fd);
-#if defined(__x86_64__) || defined(__i386__)
-        if (paranoid_level > 0 && getuid() != 0)
-        {
-            fprintf(stderr, "WARN: Linux kernel configured with paranoid level %d\n", paranoid_level);
-            fprintf(stderr, "WARN: Paranoid level 0 or root access is required to measure Uncore counters\n");
-        }
-#endif
-        }
-        informed_paranoid = 1;
+        errno = EPERM;
+        ERROR_PRINT(Cannot use performance monitoring with perf_event_paranoid = %d, perf_event_paranoid);
+        return -(cpu_id+1);
     }
+
     lock_acquire((int*) &tile_lock[affinity_thread2core_lookup[cpu_id]], cpu_id);
     lock_acquire((int*) &socket_lock[affinity_thread2socket_lookup[cpu_id]], cpu_id);
+    lock_acquire((int*) &numa_lock[affinity_thread2numa_lookup[cpu_id]], cpu_id);
+    lock_acquire((int*) &sharedl3_lock[affinity_thread2sharedl3_lookup[cpu_id]], cpu_id);
+    lock_acquire((int*) &die_lock[affinity_thread2die_lookup[cpu_id]], cpu_id);
     if (cpu_event_fds == NULL)
     {
         cpu_event_fds = malloc(cpuid_topology.numHWThreads * sizeof(int*));
@@ -108,38 +113,12 @@ int perfmon_init_perfevent(int cpu_id)
         memset(cpu_event_fds[cpu_id], -1, perfmon_numCounters * sizeof(int));
         active_cpus += 1;
     }
+    perf_event_num_cpus = cpuid_topology.numHWThreads;
+    perf_event_initialized = 1;
     return 0;
 }
 
-int perf_fixed_setup(struct perf_event_attr *attr, PerfmonEvent *event)
-{
-    int ret = -1;
-    attr->type = PERF_TYPE_HARDWARE;
-    attr->exclude_kernel = 1;
-    attr->exclude_hv = 1;
-    attr->disabled = 1;
-    attr->inherit = 1;
-    //attr->exclusive = 1;
-    if (strcmp(event->name, "INSTR_RETIRED_ANY") == 0)
-    {
-        attr->config = PERF_COUNT_HW_INSTRUCTIONS;
-        ret = 0;
-    }
-    if (strcmp(event->name, "CPU_CLK_UNHALTED_CORE") == 0)
-    {
-        attr->config = PERF_COUNT_HW_CPU_CYCLES;
-        ret = 0;
-    }
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,3,0)
-    if (strcmp(event->name, "CPU_CLK_UNHALTED_REF") == 0)
-    {
-        attr->config = PERF_COUNT_HW_REF_CPU_CYCLES;
-        ret = 0;
-    }
-#endif
 
-    return ret;
-}
 
 typedef enum {
     PERF_EVENT_INVAL_REG = 0,
@@ -158,6 +137,8 @@ static char* perfEventOptionNames[] = {
     [EVENT_OPTION_MATCH0] = "offcore_rsp",
     [EVENT_OPTION_MATCH1] = "offcore_rsp",
     [EVENT_OPTION_TID] = "tid_en",
+    [EVENT_OPTION_CID] = "cid",
+    [EVENT_OPTION_SLICE] = "slice",
     [EVENT_OPTION_STATE] = "filter_state",
     [EVENT_OPTION_NID] = "filter_nid",
     [EVENT_OPTION_OPCODE] = "filter_opc",
@@ -165,6 +146,17 @@ static char* perfEventOptionNames[] = {
     [EVENT_OPTION_OCCUPANCY_FILTER] = "occ_band0",
     [EVENT_OPTION_OCCUPANCY_EDGE] = "occ_edge",
     [EVENT_OPTION_OCCUPANCY_INVERT] = "occ_inv",
+#ifdef _ARCH_PPC
+    [EVENT_OPTION_GENERIC_CONFIG] = "pmcxsel",
+#else
+    [EVENT_OPTION_GENERIC_CONFIG] = "event",
+#endif
+    [EVENT_OPTION_GENERIC_UMASK] = "umask",
+#ifdef _ARCH_PPC
+    [EVENT_OPTION_PMC] = "pmc",
+    [EVENT_OPTION_PMCXSEL] = "pmcxsel",
+    [EVENT_OPTION_UNCORE_CONFIG] = "event",
+#endif
 };
 
 int getEventOptionConfig(char* base, EventOptionType type, PERF_EVENT_PMC_OPT_REGS *reg, int* start, int* end)
@@ -247,22 +239,156 @@ uint64_t create_mask(uint32_t value, int start, int end)
     return 0x0ULL;
 }
 
-int perf_pmc_setup(struct perf_event_attr *attr, PerfmonEvent *event)
+int perf_fixed_setup(struct perf_event_attr *attr, RegisterIndex index, PerfmonEvent *event)
+{
+    int ret = -1;
+    attr->type = PERF_TYPE_HARDWARE;
+    attr->disabled = 1;
+    attr->inherit = 1;
+    if (translate_types[FIXED] != NULL &&
+        strcmp(translate_types[PMC], translate_types[FIXED]) == 0)
+    {
+        attr->exclude_kernel = 1;
+        attr->exclude_hv = 1;
+        if (strcmp(event->name, "INSTR_RETIRED_ANY") == 0)
+        {
+            attr->config = PERF_COUNT_HW_INSTRUCTIONS;
+            ret = 0;
+        }
+        if (strcmp(event->name, "CPU_CLK_UNHALTED_CORE") == 0 ||
+            strcmp(event->name, "ACTUAL_CPU_CLOCK") == 0 ||
+            strcmp(event->name, "APERF") == 0)
+        {
+            attr->config = PERF_COUNT_HW_CPU_CYCLES;
+            ret = 0;
+        }
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,3,0)
+        if (strcmp(event->name, "CPU_CLK_UNHALTED_REF") == 0)
+        {
+            attr->config = PERF_COUNT_HW_REF_CPU_CYCLES;
+            ret = 0;
+        }
+#endif
+    }
+    else
+    {
+        char checkfolder[1024];
+        int perf_type = PERF_TYPE_HARDWARE;
+        int start = 0, end = 0;
+        PERF_EVENT_PMC_OPT_REGS reg = PERF_EVENT_INVAL_REG;
+        int err = snprintf(checkfolder, 1023, "%s/type", translate_types[FIXED]);
+        if (err > 0)
+        {
+            checkfolder[err] = '\0';
+        }
+        FILE* fp = fopen(checkfolder, "r");
+        if (fp != NULL)
+        {
+            ret = fread(checkfolder, sizeof(char), 1024, fp);
+            fclose(fp);
+            perf_type = atoi(checkfolder);
+            if (perf_type >= 0)
+            {
+                attr->type = perf_type;
+                getEventOptionConfig(translate_types[FIXED], EVENT_OPTION_GENERIC_CONFIG, &reg, &start, &end);
+                switch(reg)
+                {
+                    case PERF_EVENT_CONFIG_REG:
+                        attr->config |= create_mask(event->eventId, start, end);
+                        ret = 0;
+                        break;
+                    case PERF_EVENT_CONFIG1_REG:
+                        attr->config1 |= create_mask(event->eventId, start, end);
+                        ret = 0;
+                        break;
+                    case PERF_EVENT_CONFIG2_REG:
+                        attr->config2 |= create_mask(event->eventId, start, end);
+                        ret = 0;
+                        break;
+                    default:
+                        ret = -1;
+                        break;
+                }
+            }
+        }
+    }
+
+    return ret;
+}
+
+int perf_perf_setup(struct perf_event_attr *attr, RegisterIndex index, PerfmonEvent *event)
+{
+    attr->type = PERF_TYPE_HARDWARE;
+    attr->exclude_kernel = 1;
+    attr->exclude_hv = 1;
+    attr->disabled = 1;
+    attr->inherit = 1;
+    attr->config = event->eventId;
+    return 0;
+}
+
+int perf_pmc_setup(struct perf_event_attr *attr, RegisterIndex index, PerfmonEvent *event)
 {
     uint64_t offcore_flags = 0x0ULL;
     PERF_EVENT_PMC_OPT_REGS reg = PERF_EVENT_INVAL_REG;
     int start = 0, end = -1;
     attr->type = PERF_TYPE_RAW;
-    attr->config = (event->umask<<8) + event->eventId;
+    //attr->config = (event->umask<<8) + event->eventId;
     attr->exclude_kernel = 1;
     attr->exclude_hv = 1;
     attr->disabled = 1;
     attr->inherit = 1;
     //attr->exclusive = 1;
+#if defined(__ARM_ARCH_8A) || defined(__ARM_ARCH_7A__)
+    if (cpuid_info.vendor == FUJITSU_ARM && cpuid_info.part == FUJITSU_A64FX)
+    {
+        reg = PERF_EVENT_CONFIG_REG;
+        start = 0;
+        end = 31;
+    }
+    else
+    {
+#endif
+        getEventOptionConfig(translate_types[PMC], EVENT_OPTION_GENERIC_CONFIG, &reg, &start, &end);
+#if defined(__ARM_ARCH_8A) || defined(__ARM_ARCH_7A__)
+    }
+#endif
+    switch(reg)
+    {
+        case PERF_EVENT_CONFIG_REG:
+            attr->config |= create_mask(event->eventId, start, end);
+            break;
+        case PERF_EVENT_CONFIG1_REG:
+            attr->config1 |= create_mask(event->eventId, start, end);
+            break;
+        case PERF_EVENT_CONFIG2_REG:
+            attr->config2 |= create_mask(event->eventId, start, end);
+            break;
+    }
+#ifdef _ARCH_PPC
+    reg = PERF_EVENT_CONFIG_REG;
+    start = 8;
+    end = 15;
+#else
+    getEventOptionConfig(translate_types[PMC], EVENT_OPTION_GENERIC_UMASK, &reg, &start, &end);
+#endif
+    switch(reg)
+    {
+        case PERF_EVENT_CONFIG_REG:
+            attr->config |= create_mask(event->umask, start, end);
+            break;
+        case PERF_EVENT_CONFIG1_REG:
+            attr->config1 |= create_mask(event->umask, start, end);
+            break;
+        case PERF_EVENT_CONFIG2_REG:
+            attr->config2 |= create_mask(event->umask, start, end);
+            break;
+    }
     if (event->numberOfOptions > 0)
     {
         for(int j = 0; j < event->numberOfOptions; j++)
         {
+
             switch (event->options[j].type)
             {
                 case EVENT_OPTION_COUNT_KERNEL:
@@ -274,7 +400,7 @@ int perf_pmc_setup(struct perf_event_attr *attr, PerfmonEvent *event)
                 case EVENT_OPTION_INVERT:
                 case EVENT_OPTION_IN_TRANS:
                 case EVENT_OPTION_IN_TRANS_ABORT:
-                    getEventOptionConfig("/sys/devices/cpu", event->options[j].type, &reg, &start, &end);
+                    getEventOptionConfig(translate_types[PMC], event->options[j].type, &reg, &start, &end);
                     switch(reg)
                     {
                         case PERF_EVENT_CONFIG_REG:
@@ -311,7 +437,7 @@ int perf_pmc_setup(struct perf_event_attr *attr, PerfmonEvent *event)
         {
             offcore_flags = (1ULL<<event->cfgBits)|(1ULL<<event->cmask);
         }
-        getEventOptionConfig("/sys/devices/cpu", EVENT_OPTION_MATCH0, &reg, &start, &end);
+        getEventOptionConfig(translate_types[PMC], EVENT_OPTION_MATCH0, &reg, &start, &end);
         switch(reg)
         {
             case PERF_EVENT_CONFIG_REG:
@@ -325,7 +451,23 @@ int perf_pmc_setup(struct perf_event_attr *attr, PerfmonEvent *event)
                 break;
         }
     }
-
+#ifdef _ARCH_PPC
+    getEventOptionConfig(translate_types[PMC], EVENT_OPTION_PMC, &reg, &start, &end);
+    switch(reg)
+    {
+        case PERF_EVENT_CONFIG_REG:
+            attr->config |= create_mask(getCounterTypeOffset(index)+1,start, end);
+            break;
+        case PERF_EVENT_CONFIG1_REG:
+            attr->config1 |= create_mask(getCounterTypeOffset(index)+1,start, end);
+            break;
+        case PERF_EVENT_CONFIG2_REG:
+            attr->config2 |= create_mask(getCounterTypeOffset(index)+1,start, end);
+            break;
+        default:
+            break;
+    }
+#endif
     return 0;
 }
 
@@ -338,9 +480,10 @@ int perf_uncore_setup(struct perf_event_attr *attr, RegisterType type, PerfmonEv
     int perf_type = 0;
     PERF_EVENT_PMC_OPT_REGS reg = PERF_EVENT_INVAL_REG;
     int start = 0, end = -1;
-    if (paranoid_level > 0)
+    uint64_t eventConfig = 0x0;
+    if (perf_event_paranoid > 0 && getuid() != 0)
     {
-        return 1;
+        return EPERM;
     }
     attr->type = 0;
     ret = sprintf(checkfolder, "%s", translate_types[type]);
@@ -359,6 +502,7 @@ int perf_uncore_setup(struct perf_event_attr *attr, RegisterType type, PerfmonEv
             return 1;
         }
     }
+    DEBUG_PRINT(DEBUGLEV_DEVELOP, Get information for uncore counters from folder %s, checkfolder);
     ret = sprintf(&(checkfolder[ret]), "/type");
     fp = fopen(checkfolder, "r");
     if (fp == NULL)
@@ -369,9 +513,61 @@ int perf_uncore_setup(struct perf_event_attr *attr, RegisterType type, PerfmonEv
     perf_type = atoi(checkfolder);
     fclose(fp);
     attr->type = perf_type;
-    attr->config = (event->umask<<8) + event->eventId;
     attr->disabled = 1;
     attr->inherit = 1;
+#ifdef _ARCH_PPC
+    eventConfig = (event->umask<<8)|event->eventId;
+#else
+    eventConfig = event->eventId;
+#endif
+    getEventOptionConfig(translate_types[type], EVENT_OPTION_GENERIC_CONFIG, &reg, &start, &end);
+    switch(reg)
+    {
+        case PERF_EVENT_CONFIG_REG:
+            attr->config |= create_mask(eventConfig, start, end);
+            break;
+        case PERF_EVENT_CONFIG1_REG:
+            attr->config1 |= create_mask(eventConfig, start, end);
+            break;
+        case PERF_EVENT_CONFIG2_REG:
+            attr->config2 |= create_mask(eventConfig, start, end);
+            break;
+    }
+#ifdef _ARCH_PPC
+    if (reg == PERF_EVENT_INVAL_REG)
+    {
+        getEventOptionConfig(translate_types[type], EVENT_OPTION_UNCORE_CONFIG, &reg, &start, &end);
+        switch(reg)
+        {
+            case PERF_EVENT_CONFIG_REG:
+                attr->config |= create_mask(eventConfig, start, end);
+                break;
+            case PERF_EVENT_CONFIG1_REG:
+                attr->config1 |= create_mask(eventConfig, start, end);
+                break;
+            case PERF_EVENT_CONFIG2_REG:
+                attr->config2 |= create_mask(eventConfig, start, end);
+                break;
+        }
+    }
+#else
+    if (event->umask != 0x0)
+    {
+        getEventOptionConfig(translate_types[type], EVENT_OPTION_GENERIC_UMASK, &reg, &start, &end);
+        switch(reg)
+        {
+            case PERF_EVENT_CONFIG_REG:
+                attr->config |= create_mask(event->umask, start, end);
+                break;
+            case PERF_EVENT_CONFIG1_REG:
+                attr->config1 |= create_mask(event->umask, start, end);
+                break;
+            case PERF_EVENT_CONFIG2_REG:
+                attr->config2 |= create_mask(event->umask, start, end);
+                break;
+        }
+    }
+#endif
 
     //attr->exclusive = 1;
     if (event->numberOfOptions > 0)
@@ -416,8 +612,6 @@ int perf_uncore_setup(struct perf_event_attr *attr, RegisterType type, PerfmonEv
 }
 
 
-
-
 int perfmon_setupCountersThread_perfevent(
         int thread_id,
         PerfmonEventSet* eventSet)
@@ -429,24 +623,47 @@ int perfmon_setupCountersThread_perfevent(
     int is_uncore = 0;
     pid_t allpid = -1;
     unsigned long allflags = 0;
+    char* env_perf_pid = getenv("LIKWID_PERF_PID");
 
-    if (getenv("LIKWID_PERF_PID") != NULL)
+    if (!perf_event_initialized)
     {
-        allpid = (pid_t)atoi(getenv("LIKWID_PERF_PID"));
+        return -(thread_id+1);
     }
-    else if (paranoid_level > 0)
+    if (env_perf_pid != NULL)
     {
-        fprintf(stderr, "Cannot setup events without PID of executed application because perf_event_paranoid > 0\n");
-        fprintf(stderr, "You can use either --execpid to track the started application or --perfpid <pid> to monitor another application\n");
-        return -((int)thread_id+1);
+        allpid = (pid_t)atoi(env_perf_pid);
     }
+    if (perf_event_paranoid > 0 && getuid() != 0)
+    {
+        if (allpid == -1)
+        {
+            DEBUG_PRINT(DEBUGLEV_INFO, PID of application required. Use LIKWID_PERF_PID env variable or likwid-perfctr options);
+            return -EPERM;
+        }
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, Using PID %d for perf_event measurements, allpid);
+    }
+
     if (getenv("LIKWID_PERF_FLAGS") != NULL)
     {
         allflags = strtoul(getenv("LIKWID_PERF_FLAGS"), NULL, 16);
     }
+    if (groupSet->activeGroup >= 0)
+    {
+        for (int j = 0; j < perfmon_numCounters; j++)
+        {
+            if (cpu_event_fds[cpu_id][j] != -1)
+            {
+                close(cpu_event_fds[cpu_id][j]);
+                cpu_event_fds[cpu_id][j] = -1;
+            }
+        }
+    }
     for (int i=0;i < eventSet->numberOfEvents;i++)
     {
+        int has_lock = 0;
+        int pmc_lock = 0;
         is_uncore = 0;
+        ret = 1;
         RegisterIndex index = eventSet->events[i].index;
         if (cpu_event_fds[cpu_id][index] != -1)
         {
@@ -463,21 +680,53 @@ int perfmon_setupCountersThread_perfevent(
         switch (type)
         {
             case FIXED:
-                ret = perf_fixed_setup(&attr, event);
+                ret = perf_fixed_setup(&attr, index, event);
                 if (ret < 0)
                 {
                     continue;
                 }
                 VERBOSEPRINTREG(cpu_id, index, attr.config, SETUP_FIXED);
                 break;
+            case PERF:
+                ret = perf_perf_setup(&attr, index, event);
+                if (ret < 0)
+                {
+                    continue;
+                }
+                VERBOSEPRINTREG(cpu_id, index, attr.config, SETUP_PERF);
+                break;
             case PMC:
-                ret = perf_pmc_setup(&attr, event);
-                VERBOSEPRINTREG(cpu_id, index, attr.config, SETUP_PMC);
+                pmc_lock = 1;
+#if defined(__ARM_ARCH_8A)
+                if (cpuid_info.vendor == FUJITSU_ARM && cpuid_info.part == FUJITSU_A64FX)
+                {
+                    if (event->eventId == 0x3E8 ||
+                        event->eventId == 0x3E0 ||
+                        event->eventId == 0x308 ||
+                        event->eventId == 0x309 ||
+                        (event->eventId >= 0x314 &&  event->eventId <= 0x31E))
+                    {
+                        if (numa_lock[affinity_thread2numa_lookup[cpu_id]] != cpu_id)
+                        {
+                            pmc_lock = 0;
+                        }
+                    }
+                }
+#endif
+                if (pmc_lock)
+                {
+                    ret = perf_pmc_setup(&attr, index, event);
+                    VERBOSEPRINTREG(cpu_id, index, attr.config, SETUP_PMC);
+                }
                 break;
             case POWER:
-                ret = perf_uncore_setup(&attr, type, event);
+                if (socket_lock[affinity_thread2socket_lookup[cpu_id]] == cpu_id)
+                {
+                    has_lock = 1;
+                    VERBOSEPRINTREG(cpu_id, index, attr.config, SETUP_POWER);
+                    ret = perf_uncore_setup(&attr, type, event);
+                }
                 is_uncore = 1;
-                VERBOSEPRINTREG(cpu_id, index, attr.config, SETUP_POWER);
                 break;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3,5,0)
             case MBOX0:
@@ -517,6 +766,7 @@ int perfmon_setupCountersThread_perfevent(
             case CBOX26:
             case CBOX27:
             case UBOX:
+            case UBOXFIX:
             case SBOX0:
             case SBOX1:
             case SBOX2:
@@ -545,9 +795,43 @@ int perfmon_setupCountersThread_perfevent(
             case EUBOX5:
             case EUBOX6:
             case EUBOX7:
-                ret = perf_uncore_setup(&attr, type, event);
-                is_uncore = 1;
-                VERBOSEPRINTREG(cpu_id, index, attr.config, SETUP_UNCORE);
+                if (cpuid_info.family == ZEN_FAMILY && type == MBOX0)
+                {
+                    if (numa_lock[affinity_thread2numa_lookup[cpu_id]] == cpu_id)
+                    {
+                        has_lock = 1;
+                    }
+                }
+                else if (cpuid_info.family == ZEN_FAMILY && type == CBOX0)
+                {
+                    if (sharedl3_lock[affinity_thread2sharedl3_lookup[cpu_id]] == cpu_id)
+                    {
+                        has_lock = 1;
+                    }
+                }
+                else if (cpuid_info.family == P6_FAMILY &&
+                         cpuid_info.model == SKYLAKEX &&
+                         cpuid_info.stepping >= 5 &&
+                         cpuid_topology.numDies > cpuid_topology.numSockets)
+                {
+                    if (die_lock[affinity_thread2die_lookup[cpu_id]] == cpu_id)
+                    {
+                        has_lock = 1;
+                    }
+                }
+                else
+                {
+                    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] == cpu_id)
+                    {
+                        has_lock = 1;
+                    }
+                }
+                if (has_lock)
+                {
+                    ret = perf_uncore_setup(&attr, type, event);
+                    is_uncore = 1;
+                    VERBOSEPRINTREG(cpu_id, index, attr.config, SETUP_UNCORE);
+                }
                 break;
 #endif
             default:
@@ -555,34 +839,49 @@ int perfmon_setupCountersThread_perfevent(
         }
         if (ret == 0)
         {
+            pid_t curpid = allpid;
+            if (is_uncore && curpid >= 0)
+            {
+                curpid = -1;
+            }
 
-            if (!is_uncore || socket_lock[affinity_thread2socket_lookup[cpu_id]] == cpu_id)
+            if (!is_uncore)
             {
-                pid_t curpid = allpid;
-                if (is_uncore && curpid >= 0)
-                    curpid = -1;
                 DEBUG_PRINT(DEBUGLEV_DEVELOP, perf_event_open: cpu_id=%d pid=%d flags=%d, cpu_id, curpid, allflags);
                 cpu_event_fds[cpu_id][index] = perf_event_open(&attr, curpid, cpu_id, -1, allflags);
-                if (cpu_event_fds[cpu_id][index] < 0)
+            }
+            else if ((perf_disable_uncore == 0) && (has_lock))
+            {
+                if (perf_event_paranoid > 0 && getuid() != 0)
                 {
-                    fprintf(stderr, "Setup of event %s on CPU %d failed: %s\n", event->name, cpu_id, strerror(errno));
-                    DEBUG_PRINT(DEBUGLEV_DEVELOP, perf_event_open: cpu_id=%d pid=%d flags=%d, cpu_id, curpid, allflags);
-                    DEBUG_PRINT(DEBUGLEV_DEVELOP, type %d, attr.type);
-                    DEBUG_PRINT(DEBUGLEV_DEVELOP, size %d, attr.size);
-                    DEBUG_PRINT(DEBUGLEV_DEVELOP, config 0x%llX, attr.config);
-                    DEBUG_PRINT(DEBUGLEV_DEVELOP, read_format %d, attr.read_format);
-                    DEBUG_PRINT(DEBUGLEV_DEVELOP, disabled %d, attr.disabled);
-                    DEBUG_PRINT(DEBUGLEV_DEVELOP, inherit %d, attr.inherit);
-                    DEBUG_PRINT(DEBUGLEV_DEVELOP, pinned %d, attr.pinned);
-                    DEBUG_PRINT(DEBUGLEV_DEVELOP, exclusive %d, attr.exclusive);
-                    continue;
+                    DEBUG_PRINT(DEBUGLEV_INFO, Cannot measure Uncore with perf_event_paranoid value = %d, perf_event_paranoid);
+                    perf_disable_uncore = 1;
                 }
-                if (group_fd < 0)
-                {
-                    group_fd = cpu_event_fds[cpu_id][index];
-                    running_group = group_fd;
-                }
-                eventSet->events[i].threadCounter[thread_id].init = TRUE;
+                DEBUG_PRINT(DEBUGLEV_DEVELOP, perf_event_open: cpu_id=%d pid=%d flags=%d, cpu_id, curpid, allflags);
+                cpu_event_fds[cpu_id][index] = perf_event_open(&attr, curpid, cpu_id, -1, allflags);
+            }
+            else
+            {
+                DEBUG_PRINT(DEBUGLEV_INFO, Unknown perf_event_paranoid value = %d, perf_event_paranoid);
+            }
+            if (cpu_event_fds[cpu_id][index] < 0)
+            {
+                ERROR_PRINT(Setup of event %s on CPU %d failed: %s, event->name, cpu_id, strerror(errno));
+                DEBUG_PRINT(DEBUGLEV_DEVELOP, open error: cpu_id=%d pid=%d flags=%d type=%d config=0x%llX disabled=%d inherit=%d exclusive=%d config2=0x%llX, cpu_id, curpid, allflags, attr.type, attr.config, attr.disabled, attr.inherit, attr.exclusive);
+            }
+            if (group_fd < 0)
+            {
+                group_fd = cpu_event_fds[cpu_id][index];
+                running_group = group_fd;
+            }
+            eventSet->events[i].threadCounter[thread_id].init = TRUE;
+        }
+        else if (ret == EPERM)
+        {
+            if (is_uncore && perf_disable_uncore == 0 && perf_event_paranoid > 0 && getuid() != 0)
+            {
+                DEBUG_PRINT(DEBUGLEV_INFO, Cannot measure Uncore with perf_event_paranoid value = %d, perf_event_paranoid);
+                perf_disable_uncore = 1;
             }
         }
     }
@@ -593,6 +892,10 @@ int perfmon_startCountersThread_perfevent(int thread_id, PerfmonEventSet* eventS
 {
     int ret = 0;
     int cpu_id = groupSet->threads[thread_id].processorId;
+    if (!perf_event_initialized)
+    {
+        return -(thread_id+1);
+    }
     for (int i=0;i < eventSet->numberOfEvents;i++)
     {
         if (eventSet->events[i].threadCounter[thread_id].init == TRUE)
@@ -602,7 +905,9 @@ int perfmon_startCountersThread_perfevent(int thread_id, PerfmonEventSet* eventS
                 continue;
             VERBOSEPRINTREG(cpu_id, 0x0, 0x0, RESET_COUNTER);
             ioctl(cpu_event_fds[cpu_id][index], PERF_EVENT_IOC_RESET, 0);
-            eventSet->events[i].threadCounter[thread_id].startData = 0x0ULL;
+            PerfmonCounter *c = &eventSet->events[i].threadCounter[thread_id];
+            c->startData = 0x0ULL;
+            c->counterData = 0x0ULL;
             if (eventSet->events[i].type == POWER)
             {
                 ret = read(cpu_event_fds[cpu_id][index],
@@ -610,7 +915,7 @@ int perfmon_startCountersThread_perfevent(int thread_id, PerfmonEventSet* eventS
                         sizeof(long long));
             }
             VERBOSEPRINTREG(cpu_id, 0x0,
-                            eventSet->events[i].threadCounter[thread_id].startData,
+                            c->startData,
                             START_COUNTER);
             ioctl(cpu_event_fds[cpu_id][index], PERF_EVENT_IOC_ENABLE, 0);
         }
@@ -623,6 +928,10 @@ int perfmon_stopCountersThread_perfevent(int thread_id, PerfmonEventSet* eventSe
     int ret;
     int cpu_id = groupSet->threads[thread_id].processorId;
     long long tmp = 0;
+    if (!perf_event_initialized)
+    {
+        return -(thread_id+1);
+    }
     for (int i=0;i < eventSet->numberOfEvents;i++)
     {
         if (eventSet->events[i].threadCounter[thread_id].init == TRUE)
@@ -651,6 +960,10 @@ int perfmon_readCountersThread_perfevent(int thread_id, PerfmonEventSet* eventSe
     int ret;
     int cpu_id = groupSet->threads[thread_id].processorId;
     long long tmp = 0;
+    if (!perf_event_initialized)
+    {
+        return -(thread_id+1);
+    }
     for (int i=0;i < eventSet->numberOfEvents;i++)
     {
         if (eventSet->events[i].threadCounter[thread_id].init == TRUE)
@@ -677,29 +990,46 @@ int perfmon_readCountersThread_perfevent(int thread_id, PerfmonEventSet* eventSe
 int perfmon_finalizeCountersThread_perfevent(int thread_id, PerfmonEventSet* eventSet)
 {
     int cpu_id = groupSet->threads[thread_id].processorId;
-    for (int i=0;i < eventSet->numberOfEvents;i++)
+    if (cpu_event_fds != NULL)
     {
-        if (eventSet->events[i].threadCounter[thread_id].init == TRUE)
+        if (cpu_event_fds[cpu_id] != NULL)
         {
-            RegisterIndex index = eventSet->events[i].index;
-            ioctl(cpu_event_fds[cpu_id][index], PERF_EVENT_IOC_DISABLE, 0);
-            ioctl(cpu_event_fds[cpu_id][index], PERF_EVENT_IOC_RESET, 0);
-            eventSet->events[i].threadCounter[thread_id].init = FALSE;
-            close(cpu_event_fds[cpu_id][index]);
-            cpu_event_fds[cpu_id][index] = -1;
+            for (int j = 0; j < perfmon_numCounters; j++)
+            {
+                if (cpu_event_fds[cpu_id][j] > 0)
+                {
+                    ioctl(cpu_event_fds[cpu_id][j], PERF_EVENT_IOC_DISABLE, 0);
+                    ioctl(cpu_event_fds[cpu_id][j], PERF_EVENT_IOC_RESET, 0);
+                    close(cpu_event_fds[cpu_id][j]);
+                    cpu_event_fds[cpu_id][j] = -1;
+                    if (j < eventSet->numberOfEvents && eventSet->events[j].threadCounter[thread_id].init == TRUE)
+                    {
+                        eventSet->events[j].threadCounter[thread_id].init = FALSE;
+                    }
+                }
+            }
+            free(cpu_event_fds[cpu_id]);
+            cpu_event_fds[cpu_id] = NULL;
+            active_cpus--;
         }
     }
-    if (cpu_event_fds[cpu_id] != NULL)
-    {
-        free(cpu_event_fds[cpu_id]);
-        cpu_event_fds[cpu_id] = NULL;
-        active_cpus--;
-    }
-    if (active_cpus == 0)
-    {
-        free(cpu_event_fds);
-        cpu_event_fds = NULL;
-    }
     return 0;
 }
 
+void __attribute__((destructor (101))) close_perfmon_perfevent(void)
+{
+    if (cpu_event_fds != NULL)
+    {
+        for (int i = 0; i < perf_event_num_cpus; i++)
+        {
+            if (cpu_event_fds[i] != NULL)
+            {
+                free(cpu_event_fds[i]);
+                cpu_event_fds[i] = NULL;
+                active_cpus--;
+            }
+        }
+        free(cpu_event_fds);
+        cpu_event_fds = NULL;
+    }
+}
