@@ -5,14 +5,14 @@
  *
  *      Description:  Header File of perfmon module for Intel Skylake.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
- *                Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -38,6 +38,16 @@
 #include <limits.h>
 #include <topology.h>
 #include <access.h>
+#include <voltage.h>
+#include <linux/version.h>
+
+#define IS_SKYLAKE_CLIENT(model) ((model) == SKYLAKE1 || \
+                                  (model) == SKYLAKE2 || \
+                                  (model) == KABYLAKE1 || \
+                                  (model) == KABYLAKE2 || \
+                                  (model) == CANNONLAKE || \
+                                  (model) == COMETLAKE1 || \
+                                  (model) == COMETLAKE2)
 
 static int perfmon_numCountersSkylake = NUM_COUNTERS_SKYLAKE;
 static int perfmon_numCoreCountersSkylake = NUM_COUNTERS_CORE_SKYLAKE;
@@ -59,40 +69,89 @@ int skl_cbox_nosetup(int cpu_id, RegisterIndex index, PerfmonEvent *event)
     event++;
     return 0;
 }
+
+int has_uncore_lock(int cpu_id)
+{
+    if (IS_SKYLAKE_CLIENT(cpuid_info.model) &&
+        socket_lock[affinity_thread2socket_lookup[cpu_id]] == cpu_id)
+    {
+        return 1;
+    }
+    else if (cpuid_info.model == SKYLAKEX)
+    {
+        if (cpuid_topology.numSockets == cpuid_topology.numDies)
+        {
+            if (socket_lock[affinity_thread2socket_lookup[cpu_id]] == cpu_id)
+            {
+                return 1;
+            }
+        }
+        else
+        {
+            if (die_lock[affinity_thread2die_lookup[cpu_id]] == cpu_id)
+            {
+                return 1;
+            }
+        }
+    }
+    return 0;
+}
 int perfmon_init_skylake(int cpu_id)
 {
     int ret = 0;
+
     lock_acquire((int*) &tile_lock[affinity_thread2core_lookup[cpu_id]], cpu_id);
     lock_acquire((int*) &socket_lock[affinity_thread2socket_lookup[cpu_id]], cpu_id);
-    ret = HPMwrite(cpu_id, MSR_DEV, MSR_PEBS_ENABLE, 0x0ULL);
-    if (ret != 0)
+    lock_acquire((int*) &die_lock[affinity_thread2die_lookup[cpu_id]], cpu_id);
+
+    uint64_t misc_enable = 0x0;
+    ret = HPMread(cpu_id, MSR_DEV, MSR_IA32_MISC_ENABLE, &misc_enable);
+    if (ret == 0 && (testBit(misc_enable, 7) == 1) && (testBit(misc_enable, 12) == 0))
     {
-        ERROR_PRINT(Cannot zero MSR_PEBS_ENABLE (0x%X), MSR_PEBS_ENABLE);
+        ret = HPMwrite(cpu_id, MSR_DEV, MSR_PEBS_ENABLE, 0x0ULL);
+        if (ret != 0)
+        {
+            ERROR_PRINT(Cannot zero %s (0x%X), str(MSR_PEBS_ENABLE), MSR_PEBS_ENABLE);
+        }
+        ret = HPMwrite(cpu_id, MSR_DEV, MSR_PEBS_FRONTEND, 0x0ULL);
+        if (ret != 0)
+        {
+            ERROR_PRINT(Cannot zero %s (0x%X), str(MSR_PEBS_FRONTEND), MSR_PEBS_FRONTEND);
+        }
     }
-    ret = HPMwrite(cpu_id, MSR_DEV, MSR_PEBS_FRONTEND, 0x0ULL);
-    if (ret != 0)
+    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] == cpu_id)
     {
-        ERROR_PRINT(Cannot zero MSR_PEBS_FRONTEND (0x%X), MSR_PEBS_FRONTEND);
-    }
-    if (cpuid_info.model == SKYLAKEX)
-    {
-        skylake_cbox_setup = skx_cbox_setup;
-        skl_did_cbox_check = 1;
-    }
-    else if ((cpuid_info.model == SKYLAKE1 || cpuid_info.model == SKYLAKE2) &&
-             socket_lock[affinity_thread2socket_lookup[cpu_id]] == cpu_id &&
-             skl_did_cbox_check == 0)
-    {
-        uint64_t data = 0x0ULL;
-        ret = HPMwrite(cpu_id, MSR_DEV, MSR_V4_C0_PERF_CTRL0, 0x0ULL);
-        ret += HPMread(cpu_id, MSR_DEV, MSR_V4_UNC_PERF_GLOBAL_CTRL, &data);
-        ret += HPMwrite(cpu_id, MSR_DEV, MSR_V4_UNC_PERF_GLOBAL_CTRL, 0x0ULL);
-        ret += HPMread(cpu_id, MSR_DEV, MSR_V4_C0_PERF_CTRL0, &data);
-        if ((ret == 0) && (data == 0x0ULL))
-            skylake_cbox_setup = skl_cbox_setup;
-        else
-            skylake_cbox_setup = skl_cbox_nosetup;
-        skl_did_cbox_check = 1;
+        if (skl_did_cbox_check == 0)
+        {
+            if (IS_SKYLAKE_CLIENT(cpuid_info.model))
+            {
+                uint64_t data = 0x0ULL;
+                ret = 0;
+                ret += HPMread(cpu_id, MSR_DEV, MSR_UNC_CBO_CONFIG, &data);
+                skylake_cbox_setup = NULL;
+                if (ret == 0 && (data & 0xF) > 0)
+                {
+                    ret = 0;
+                    ret += HPMread(cpu_id, MSR_DEV, MSR_V4_UNC_PERF_GLOBAL_CTRL, &data);
+                    ret += HPMwrite(cpu_id, MSR_DEV, MSR_V4_UNC_PERF_GLOBAL_CTRL, data);
+                    if (ret == 0)
+                        skylake_cbox_setup = skl_cbox_setup;
+                }
+                if (!skylake_cbox_setup)
+                {
+                    skylake_cbox_setup = skl_cbox_nosetup;
+                }
+            }
+            else if (cpuid_info.model == SKYLAKEX)
+            {
+                skylake_cbox_setup = skx_cbox_setup;
+            }
+            else
+            {
+                skylake_cbox_setup = skl_cbox_nosetup;
+            }
+            skl_did_cbox_check = 1;
+        }
     }
     return 0;
 }
@@ -222,7 +281,7 @@ int skl_ubox_setup(int cpu_id, RegisterIndex index, PerfmonEvent *event)
     int j;
     uint64_t flags = 0x0ULL;
 
-    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] != cpu_id)
+    if (!has_uncore_lock(cpu_id))
     {
         return 0;
     }
@@ -262,7 +321,7 @@ int skl_uboxfix_setup(int cpu_id, RegisterIndex index, PerfmonEvent *event)
 {
     uint64_t flags = 0x0ULL;
     event++;
-    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] != cpu_id)
+    if (!has_uncore_lock(cpu_id))
     {
         return 0;
     }
@@ -280,7 +339,7 @@ int skl_cbox_setup(int cpu_id, RegisterIndex index, PerfmonEvent *event)
 {
     int j;
     uint64_t flags = 0x0ULL;
-    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] != cpu_id)
+    if (!has_uncore_lock(cpu_id))
     {
         return 0;
     }
@@ -327,7 +386,7 @@ int skx_cbox_setup(int cpu_id, RegisterIndex index, PerfmonEvent *event)
     int opc_match = 0;
     int match1 = 0;
 
-    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] != cpu_id)
+    if (!has_uncore_lock(cpu_id))
     {
         return 0;
     }
@@ -369,7 +428,7 @@ int skx_cbox_setup(int cpu_id, RegisterIndex index, PerfmonEvent *event)
                     set_state_all = 0;
                     break;
                 case EVENT_OPTION_TID:
-                    filter_flags0 |= (extractBitField(event->options[j].value,8,0));
+                    filter_flags0 |= (extractBitField(event->options[j].value,9,0));
                     flags |= (1ULL<<19);
                     break;
                 case EVENT_OPTION_MATCH0:
@@ -432,7 +491,7 @@ int skx_mbox_setup(int cpu_id, RegisterIndex index, PerfmonEvent *event)
     uint64_t flags = 0x0ULL;
     PciDeviceIndex dev = counter_map[index].device;
 
-    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] != cpu_id)
+    if (!has_uncore_lock(cpu_id))
     {
         return 0;
     }
@@ -478,7 +537,7 @@ int skx_mboxfix_setup(int cpu_id, RegisterIndex index, PerfmonEvent *event)
     uint64_t flags = 0x0ULL;
     PciDeviceIndex dev = counter_map[index].device;
     event++;
-    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] != cpu_id)
+    if (!has_uncore_lock(cpu_id))
     {
         return 0;
     }
@@ -504,7 +563,7 @@ int skx_wbox_setup(int cpu_id, RegisterIndex index, PerfmonEvent *event)
     uint64_t filter = box_map[counter_map[index].type].filterRegister1;
     int clean_filter = 1;
 
-    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] != cpu_id)
+    if (!has_uncore_lock(cpu_id))
     {
         return 0;
     }
@@ -574,7 +633,7 @@ int skx_sbox_setup(int cpu_id, RegisterIndex index, PerfmonEvent *event)
     uint64_t flags = 0x0ULL;
     PciDeviceIndex dev = counter_map[index].device;
 
-    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] != cpu_id)
+    if (!has_uncore_lock(cpu_id))
     {
         return 0;
     }
@@ -629,7 +688,7 @@ int skx_uncorebox_setup(int cpu_id, RegisterIndex index, PerfmonEvent *event)
     uint64_t flags = 0x0ULL;
     PciDeviceIndex dev = counter_map[index].device;
 
-    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] != cpu_id)
+    if (!has_uncore_lock(cpu_id))
     {
         return 0;
     }
@@ -675,7 +734,7 @@ int skx_ibox_setup(int cpu_id, RegisterIndex index, PerfmonEvent *event)
     uint64_t flags = 0x0ULL;
     PciDeviceIndex dev = counter_map[index].device;
 
-    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] != cpu_id)
+    if (!has_uncore_lock(cpu_id))
     {
         return 0;
     }
@@ -725,7 +784,7 @@ int skx_ibox_setup(int cpu_id, RegisterIndex index, PerfmonEvent *event)
 #define SKL_UNCORE_FREEZE \
     if (haveLock && MEASURE_UNCORE(eventSet)) \
     { \
-        if (cpuid_info.model == SKYLAKE1 || cpuid_info.model == SKYLAKE2) \
+        if (IS_SKYLAKE_CLIENT(cpuid_info.model)) \
         { \
             VERBOSEPRINTREG(cpu_id, MSR_V4_UNC_PERF_GLOBAL_CTRL, 0x0ULL, FREEZE_UNCORE) \
             CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, MSR_DEV, MSR_V4_UNC_PERF_GLOBAL_CTRL, 0x0ULL)); \
@@ -741,7 +800,7 @@ int skx_ibox_setup(int cpu_id, RegisterIndex index, PerfmonEvent *event)
 #define SKL_UNCORE_UNFREEZE \
     if (haveLock && MEASURE_UNCORE(eventSet)) \
     { \
-        if (cpuid_info.model == SKYLAKE1 || cpuid_info.model == SKYLAKE2) \
+        if (IS_SKYLAKE_CLIENT(cpuid_info.model)) \
         { \
             VERBOSEPRINTREG(cpu_id, MSR_V4_UNC_PERF_GLOBAL_CTRL, uflags|(1ULL<<29), UNFREEZE_UNCORE) \
             CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, MSR_DEV, MSR_V4_UNC_PERF_GLOBAL_CTRL, uflags|(1ULL<<29))); \
@@ -761,10 +820,7 @@ int perfmon_setupCounterThread_skylake(
     uint64_t fixed_flags = 0x0ULL;
     int cpu_id = groupSet->threads[thread_id].processorId;
 
-    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] == cpu_id)
-    {
-        haveLock = 1;
-    }
+    haveLock = has_uncore_lock(cpu_id);
 
     if (MEASURE_CORE(eventSet))
     {
@@ -789,7 +845,35 @@ int perfmon_setupCounterThread_skylake(
         switch (type)
         {
             case PMC:
-                skl_pmc_setup(cpu_id, index, event);
+                if ((getCounterTypeOffset(index) == 3) &&
+                    (cpuid_info.featureFlags & (1ULL<<RTM)) &&
+                    (cpuid_info.model == SKYLAKEX) &&
+                    (cpuid_info.stepping < 5))
+                {
+                    uint64_t flags = 0x0;
+                    HPMread(cpu_id, MSR_DEV, TSX_FORCE_ABORT, &flags);
+                    if ((flags & 0x1) == 0)
+                    {
+                        fprintf(stderr, "Warning: Counter PMC3 cannot be used if Restricted Transactional Memory feature is enabled and\n");
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,2,0)
+                        fprintf(stderr, "         bit 0 of register TSX_FORCE_ABORT is 0. As workaround enable\n");
+                        fprintf(stderr, "         allow_tsx_force_abort in /sys/devices/cpu/\n");
+#else
+                        fprintf(stderr, "         bit 0 of register TSX_FORCE_ABORT is 0. As workaround write 0x1 to TSX_FORCE_ABORT:\n");
+                        fprintf(stderr, "         sudo wrmsr 0x10f 0x1\n");
+#endif
+                        eventSet->events[i].type = NOTYPE;
+                        continue;
+                    }
+                    else
+                    {
+                        skl_pmc_setup(cpu_id, index, event);
+                    }
+                }
+                else
+                {
+                    skl_pmc_setup(cpu_id, index, event);
+                }
                 break;
 
             case FIXED:
@@ -850,12 +934,20 @@ int perfmon_setupCounterThread_skylake(
                 skylake_cbox_setup(cpu_id, index, event);
                 break;
             case MBOX0:
+                if (haveLock && !cpuid_info.supportClientmem)
+                {
+                    skx_mbox_setup(cpu_id, index, event);
+                }
+                break;
             case MBOX1:
             case MBOX2:
             case MBOX3:
             case MBOX4:
             case MBOX5:
-                skx_mbox_setup(cpu_id, index, event);
+                if (haveLock)
+                {
+                    skx_mbox_setup(cpu_id, index, event);
+                }
                 break;
             case MBOX0FIX:
             case MBOX1FIX:
@@ -911,10 +1003,7 @@ int perfmon_startCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet
     uint64_t tmp = 0x0ULL;
     int cpu_id = groupSet->threads[thread_id].processorId;
 
-    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] == cpu_id)
-    {
-        haveLock = 1;
-    }
+    haveLock = has_uncore_lock(cpu_id);
 
     for (int i=0;i < eventSet->numberOfEvents;i++)
     {
@@ -944,6 +1033,12 @@ int perfmon_startCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet
                     flags |= (1ULL<<(index+32));  /* enable fixed counter */
                     break;
 
+                case PERF:
+                    tmp = 0x0ULL;
+                    CHECK_MSR_READ_ERROR(HPMread(cpu_id, dev, counter1, &tmp));
+                    eventSet->events[i].threadCounter[thread_id].startData = field64(tmp, 0, box_map[type].regWidth);
+                    VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST tmp, READ_PERF)
+                    break;
                 case POWER:
                     if (haveLock)
                     {
@@ -995,7 +1090,7 @@ int perfmon_startCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet
                 case CBOX25:
                 case CBOX26:
                 case CBOX27:
-                    if (haveLock && (cpuid_info.model == SKYLAKE1 || cpuid_info.model == SKYLAKE2))
+                    if (haveLock && IS_SKYLAKE_CLIENT(cpuid_info.model))
                     {
                         uflags |= (1ULL<<(type-CBOX0));
                     }
@@ -1005,8 +1100,23 @@ int perfmon_startCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet
                         CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, dev, counter1, 0x0ULL));
                     }
                     break;
-                case WBOX:
                 case MBOX0:
+                    if (haveLock)
+                    {
+                        if (!cpuid_info.supportClientmem)
+                        {
+                            VERBOSEPRINTREG(cpu_id, counter1, 0x0ULL, CLEAR_BOX)
+                            CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, dev, counter1, 0x0ULL));
+                        }
+                        else
+                        {
+                            CHECK_MSR_READ_ERROR(HPMread(cpu_id, dev, counter1, &tmp));
+                            eventSet->events[i].threadCounter[thread_id].startData = field64(tmp, 0, box_map[type].regWidth);
+                            VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST tmp, READ_MBOX)
+                        }
+                    }
+                    break;
+                case WBOX:
                 case MBOX1:
                 case MBOX2:
                 case MBOX3:
@@ -1038,7 +1148,6 @@ int perfmon_startCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet
                 case EUBOX3:
                 case EUBOX4:
                 case EUBOX5:
-
                 case IBOX0FIX:
                 case IBOX1FIX:
                 case IBOX2FIX:
@@ -1127,10 +1236,7 @@ int skl_uncore_read(int cpu_id, RegisterIndex index, PerfmonEvent *event,
     PciDeviceIndex dev = counter_map[index].device;
     uint64_t counter1 = counter_map[index].counterRegister;
     event++;
-    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] != cpu_id)
-    {
-        return 0;
-    }
+    int haveLock = has_uncore_lock(cpu_id);
 
     CHECK_PCI_READ_ERROR(HPMread(cpu_id, dev, counter1, &result));
     VERBOSEPRINTPCIREG(cpu_id, dev, counter1, LLU_CAST result, READ_REG_1);
@@ -1203,10 +1309,7 @@ int perfmon_stopCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
     uint64_t counter_result = 0x0ULL;
     int cpu_id = groupSet->threads[thread_id].processorId;
 
-    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] == cpu_id)
-    {
-        haveLock = 1;
-    }
+    haveLock = has_uncore_lock(cpu_id);
 
     if (MEASURE_CORE(eventSet))
     {
@@ -1246,6 +1349,10 @@ int perfmon_stopCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
                     VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, READ_FIXED)
                     break;
 
+                case PERF:
+                    CHECK_MSR_READ_ERROR(HPMread(cpu_id, MSR_DEV, counter1, &counter_result));
+                    VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, READ_PERF)
+                    break;
                 case POWER:
                     if (haveLock)
                     {
@@ -1263,6 +1370,10 @@ int perfmon_stopCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
                     CHECK_TEMP_READ_ERROR(thermal_read(cpu_id,(uint32_t*)&counter_result));
                     break;
 
+                case VOLTAGE:
+                    CHECK_TEMP_READ_ERROR(voltage_read(cpu_id, &counter_result));
+                    break;
+
                 case UBOXFIX:
                     if (haveLock)
                     {
@@ -1319,7 +1430,7 @@ int perfmon_stopCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
                 case CBOX26:
                 case CBOX27:
                 case WBOX:
-                    if (haveLock && (cpuid_info.model == SKYLAKE1 || cpuid_info.model == SKYLAKE2))
+                    if (haveLock && IS_SKYLAKE_CLIENT(cpuid_info.model))
                     {
                         CHECK_MSR_READ_ERROR(HPMread(cpu_id, MSR_DEV, counter1, &counter_result));
                         SKL_CHECK_UNCORE_OVERFLOW(box_map[type].ovflOffset);
@@ -1334,15 +1445,39 @@ int perfmon_stopCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
                     }
                     break;
                 case MBOX0:
+                case MBOX0TMP:
+                    if (haveLock)
+                    {
+                        if (!cpuid_info.supportClientmem)
+                        {
+                            skl_uncore_read(cpu_id, index, event, current, overflows,
+                                            0, ovf_offset, getCounterTypeOffset(index)+1);
+                            counter_result = *current;
+                        }
+                        else
+                        {
+                            CHECK_MSR_READ_ERROR(HPMread(cpu_id, PCI_IMC_DEVICE_0_CH_0, counter1, &counter_result));
+                            if (counter_result < eventSet->events[i].threadCounter[thread_id].counterData)
+                            {
+                                VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, OVERFLOW_CLIENTMEM)
+                                eventSet->events[i].threadCounter[thread_id].overflows++;
+                            }
+                        }
+                        VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, READ_MBOX)
+                    }
+                    break;
                 case MBOX1:
                 case MBOX2:
                 case MBOX3:
                 case MBOX4:
                 case MBOX5:
-                    skl_uncore_read(cpu_id, index, event, current, overflows,
-                                    FREEZE_FLAG_CLEAR_CTR, ovf_offset, getCounterTypeOffset(index)+1);
-                    counter_result = *current;
-                    VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, READ_MBOX)
+                    if (haveLock)
+                    {
+                        skl_uncore_read(cpu_id, index, event, current, overflows,
+                                        0, ovf_offset, getCounterTypeOffset(index)+1);
+                        counter_result = *current;
+                        VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, READ_MBOX)
+                    }
                     break;
                 case MBOX0FIX:
                 case MBOX1FIX:
@@ -1352,9 +1487,12 @@ int perfmon_stopCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
                 case MBOX5FIX:
                 case MBOX6FIX:
                 case MBOX7FIX:
-                    bdw_uncore_read(cpu_id, index, event, current, overflows,
-                                    FREEZE_FLAG_CLEAR_CTR, ovf_offset, 0);
-                    counter_result = *current;
+                    if (haveLock)
+                    {
+                        skl_uncore_read(cpu_id, index, event, current, overflows,
+                                        FREEZE_FLAG_CLEAR_CTR, ovf_offset, 0);
+                        counter_result = *current;
+                    }
                     break;
                 case BBOX0:
                 case BBOX1:
@@ -1376,18 +1514,19 @@ int perfmon_stopCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
                 case EUBOX3:
                 case EUBOX4:
                 case EUBOX5:
-                    skl_uncore_read(cpu_id, index, event, current, overflows,
-                                    FREEZE_FLAG_CLEAR_CTR, ovf_offset, getCounterTypeOffset(index));
-                    counter_result = *current;
-                    VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, READ_BBOX)
-
+                    if (haveLock)
+                    {
+                        skl_uncore_read(cpu_id, index, event, current, overflows,
+                                        FREEZE_FLAG_CLEAR_CTR, ovf_offset, getCounterTypeOffset(index));
+                        counter_result = *current;
+                        VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, READ_BBOX)
+                    }
                     break;
                 default:
                     break;
             }
             *current = field64(counter_result, 0, box_map[type].regWidth);
         }
-        eventSet->events[i].threadCounter[thread_id].init = FALSE;
     }
     if ((haveLock) && MEASURE_UNCORE(eventSet))
     {
@@ -1411,10 +1550,7 @@ int perfmon_readCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
     uint64_t counter_result = 0x0ULL;
     int cpu_id = groupSet->threads[thread_id].processorId;
 
-    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] == cpu_id)
-    {
-        haveLock = 1;
-    }
+    haveLock = has_uncore_lock(cpu_id);
 
     if (MEASURE_CORE(eventSet))
     {
@@ -1457,6 +1593,11 @@ int perfmon_readCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
                     VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, READ_FIXED)
                     break;
 
+                case PERF:
+                    CHECK_MSR_READ_ERROR(HPMread(cpu_id, MSR_DEV, counter1, &counter_result));
+                    VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, READ_PERF)
+                    break;
+
                 case POWER:
                     if (haveLock)
                     {
@@ -1475,6 +1616,11 @@ int perfmon_readCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
                     eventSet->events[i].threadCounter[thread_id].counterData = field64(counter_result, 0, box_map[type].regWidth);
                     break;
 
+                case VOLTAGE:
+                    CHECK_TEMP_READ_ERROR(voltage_read(cpu_id, &counter_result));
+                    eventSet->events[i].threadCounter[thread_id].counterData = field64(counter_result, 0, box_map[type].regWidth);
+                    break;
+
                 case UBOXFIX:
                 case WBOX0FIX:
                 case IBOX0FIX:
@@ -1524,7 +1670,7 @@ int perfmon_readCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
                 case CBOX25:
                 case CBOX26:
                 case CBOX27:
-                    if (haveLock && (cpuid_info.model == SKYLAKE1 || cpuid_info.model == SKYLAKE2))
+                    if (haveLock && IS_SKYLAKE_CLIENT(cpuid_info.model))
                     {
                         CHECK_MSR_READ_ERROR(HPMread(cpu_id, MSR_DEV, counter1, &counter_result));
                         SKL_CHECK_UNCORE_OVERFLOW(box_map[type].ovflOffset);
@@ -1539,14 +1685,38 @@ int perfmon_readCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
                     }
                     break;
                 case MBOX0:
+                case MBOX0TMP:
+                    if (haveLock)
+                    {
+                        if (!cpuid_info.supportClientmem)
+                        {
+                            skl_uncore_read(cpu_id, index, event, current, overflows,
+                                            0, ovf_offset, getCounterTypeOffset(index)+1);
+                            counter_result = *current;
+                        }
+                        else
+                        {
+                            CHECK_MSR_READ_ERROR(HPMread(cpu_id, dev, counter1, &counter_result));
+                            if (counter_result < eventSet->events[i].threadCounter[thread_id].counterData)
+                            {
+                                VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, OVERFLOW_CLIENTMEM)
+                                eventSet->events[i].threadCounter[thread_id].overflows++;
+                            }
+                        }
+                        VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, READ_MBOX)
+                    }
+                    break;
                 case MBOX1:
                 case MBOX2:
                 case MBOX3:
                 case MBOX4:
                 case MBOX5:
-                    skl_uncore_read(cpu_id, index, event, current, overflows,
-                                    0, ovf_offset, getCounterTypeOffset(index)+1);
-                    counter_result = *current;
+                    if (haveLock)
+                    {
+                        skl_uncore_read(cpu_id, index, event, current, overflows,
+                                        0, ovf_offset, getCounterTypeOffset(index)+1);
+                        counter_result = *current;
+                    }
                     break;
                 case MBOX0FIX:
                 case MBOX1FIX:
@@ -1554,9 +1724,12 @@ int perfmon_readCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
                 case MBOX3FIX:
                 case MBOX4FIX:
                 case MBOX5FIX:
-                    skl_uncore_read(cpu_id, index, event, current, overflows,
-                                    0, ovf_offset, 0);
-                    counter_result = *current;
+                    if (haveLock)
+                    {
+                        skl_uncore_read(cpu_id, index, event, current, overflows,
+                                        0, ovf_offset, 0);
+                        counter_result = *current;
+                    }
                     break;
                 case BBOX0:
                 case BBOX1:
@@ -1579,10 +1752,13 @@ int perfmon_readCountersThread_skylake(int thread_id, PerfmonEventSet* eventSet)
                 case EUBOX4:
                 case EUBOX5:
                 case WBOX:
-                    skl_uncore_read(cpu_id, index, event, current, overflows,
-                                    0, ovf_offset, getCounterTypeOffset(index));
-                    counter_result = *current;
-                    VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, READ_BBOX)
+                    if (haveLock)
+                    {
+                        skl_uncore_read(cpu_id, index, event, current, overflows,
+                                        0, ovf_offset, getCounterTypeOffset(index));
+                        counter_result = *current;
+                        VERBOSEPRINTREG(cpu_id, counter1, LLU_CAST counter_result, READ_BBOX)
+                    }
                     break;
                 default:
                     break;
@@ -1611,10 +1787,7 @@ int perfmon_finalizeCountersThread_skylake(int thread_id, PerfmonEventSet* event
     uint64_t ovf_values_uncore = 0x0ULL;
     int cpu_id = groupSet->threads[thread_id].processorId;
 
-    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] == cpu_id)
-    {
-        haveLock = 1;
-    }
+    haveLock = has_uncore_lock(cpu_id);
     if (tile_lock[affinity_thread2core_lookup[cpu_id]] == cpu_id)
     {
         haveTileLock = 1;
@@ -1635,12 +1808,12 @@ int perfmon_finalizeCountersThread_skylake(int thread_id, PerfmonEventSet* event
                 ovf_values_core |= (1ULL<<(index-cpuid_info.perf_num_fixed_ctr));
                 if ((haveTileLock) && (eventSet->events[i].event.eventId == 0xB7))
                 {
-                    VERBOSEPRINTREG(cpu_id, MSR_OFFCORE_RESP0, 0x0ULL, CLEAR_OFFCORE_RESP0);
+                    VERBOSEPRINTREG(cpu_id, MSR_OFFCORE_RESP0, 0x0ULL, CLEAR_PMC_OFFCORE0);
                     CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, MSR_DEV, MSR_OFFCORE_RESP0, 0x0ULL));
                 }
                 else if ((haveTileLock) && (eventSet->events[i].event.eventId == 0xBB))
                 {
-                    VERBOSEPRINTREG(cpu_id, MSR_OFFCORE_RESP1, 0x0ULL, CLEAR_OFFCORE_RESP1);
+                    VERBOSEPRINTREG(cpu_id, MSR_OFFCORE_RESP1, 0x0ULL, CLEAR_PMC_OFFCORE1);
                     CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, MSR_DEV, MSR_OFFCORE_RESP1, 0x0ULL));
                 }
                 else if (eventSet->events[i].event.eventId == 0xCD)
