@@ -7,14 +7,14 @@
  *                    Configures and reads out performance counters
  *                    on x86 based architectures. Supports multi threading.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
- *                Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -65,8 +65,10 @@ typedef enum {
     EVENT_OPTION_MASK1, /*!< \brief Mask1 register */
     EVENT_OPTION_MASK2, /*!< \brief Mask2 register */
     EVENT_OPTION_MASK3, /*!< \brief Mask3 register */
-    EVENT_OPTION_NID, /*!< \brief Set NUMA node ID */
-    EVENT_OPTION_TID, /*!< \brief Set Thread ID */
+    EVENT_OPTION_NID, /*!< \brief Set NUMA node IDs */
+    EVENT_OPTION_TID, /*!< \brief Set Thread IDs */
+    EVENT_OPTION_CID, /*!< \brief Set Core IDs */
+    EVENT_OPTION_SLICE, /*!< \brief Set Slice IDs, often when the L3 is assembled with different sections */
     EVENT_OPTION_STATE, /*!< \brief Match for state */
     EVENT_OPTION_EDGE, /*!< \brief Increment counter at each edge */
     EVENT_OPTION_THRESHOLD, /*!< \brief Increment only if exceeding threshold */
@@ -79,9 +81,16 @@ typedef enum {
     EVENT_OPTION_OCCUPANCY_INVERT, /*!< \brief Invert filter for occupancy counting */
     EVENT_OPTION_IN_TRANS, /*!< \brief Count events during transactions */
     EVENT_OPTION_IN_TRANS_ABORT, /*!< \brief Count events that aborted during transactions */
+    EVENT_OPTION_GENERIC_CONFIG, /*!< \brief Configuration bitmask for generic event */
+    EVENT_OPTION_GENERIC_UMASK, /*!< \brief Umask bitmask for generic event */
 #ifdef LIKWID_USE_PERFEVENT
     EVENT_OPTION_PERF_PID, /*!< \brief PID parameter to use in the perf_event_open call */
     EVENT_OPTION_PERF_FLAGS, /*!< \brief FLAGS parameters to use in the perf_event_open call */
+#endif
+#ifdef _ARCH_PPC
+    EVENT_OPTION_PMC,
+    EVENT_OPTION_PMCXSEL,
+    EVENT_OPTION_UNCORE_CONFIG, /*!< \brief Configuration bitmask for NEST (uncore) counters */
 #endif
     NUM_EVENT_OPTIONS /*!< \brief Amount of defined options */
 } EventOptionType;
@@ -123,6 +132,8 @@ extern char* eventOptionTypeName[NUM_EVENT_OPTIONS];
 #define EVENT_OPTION_MASK3_MASK (1ULL<<EVENT_OPTION_MASK3)
 #define EVENT_OPTION_NID_MASK (1ULL<<EVENT_OPTION_NID)
 #define EVENT_OPTION_TID_MASK (1ULL<<EVENT_OPTION_TID)
+#define EVENT_OPTION_CID_MASK (1ULL<<EVENT_OPTION_CID)
+#define EVENT_OPTION_SLICE_MASK (1ULL<<EVENT_OPTION_SLICE)
 #define EVENT_OPTION_STATE_MASK (1ULL<<EVENT_OPTION_STATE)
 #define EVENT_OPTION_EDGE_MASK (1ULL<<EVENT_OPTION_EDGE)
 #define EVENT_OPTION_THRESHOLD_MASK (1ULL<<EVENT_OPTION_THRESHOLD)
@@ -135,6 +146,8 @@ extern char* eventOptionTypeName[NUM_EVENT_OPTIONS];
 #define EVENT_OPTION_OCCUPANCY_INVERT_MASK (1ULL<<EVENT_OPTION_OCCUPANCY_INVERT)
 #define EVENT_OPTION_IN_TRANS_MASK (1ULL<<EVENT_OPTION_IN_TRANS)
 #define EVENT_OPTION_IN_TRANS_ABORT_MASK (1ULL<<EVENT_OPTION_IN_TRANS_ABORT)
+#define EVENT_OPTION_GENERIC_CONFIG_MASK (1ULL<<EVENT_OPTION_GENERIC_CONFIG)
+#define EVENT_OPTION_GENERIC_UMASK_MASK (1ULL<<EVENT_OPTION_GENERIC_UMASK)
 /** @endcond */
 
 /*! \brief Structure specifying thread to CPU relation
@@ -168,12 +181,12 @@ the event options are hold here.
 */
 typedef struct {
     const char*     name; /*!< \brief Name of the event */
-    const char*     limit; /*!< \brief Valid counters for the event */
-    uint16_t        eventId; /*!< \brief ID of the event */
-    uint8_t         umask; /*!< \brief Most events need to specify a mask to limit counting */
-    uint8_t         cfgBits; /*!< \brief Misc configuration bits */
+    char*           limit; /*!< \brief Valid counters for the event */
+    uint64_t        eventId; /*!< \brief ID of the event */
+    uint64_t        umask; /*!< \brief Most events need to specify a mask to limit counting */
+    uint64_t        cfgBits; /*!< \brief Misc configuration bits */
     uint64_t        cmask; /*!< \brief Misc mask bits */
-    uint8_t         numberOfOptions; /*!< \brief Number of options for the event */
+    uint64_t         numberOfOptions; /*!< \brief Number of options for the event */
     uint64_t        optionMask; /*!< \brief Bitmask for fast check of set options */
     PerfmonEventOption options[NUM_EVENT_OPTIONS]; /*!< \brief List of options */
 } PerfmonEvent;
@@ -192,6 +205,12 @@ typedef struct {
     uint64_t    counterData; /*!< \brief Intermediate data from the counters */
     double      lastResult; /*!< \brief Last measurement result*/
     double      fullResult; /*!< \brief Aggregated measurement result */
+#if defined(__x86_64__) || defined(__i386__) || defined(__ARM_ARCH_8A) || defined(__ARM_ARCH_7A__)
+    uint64_t    _padding[2]; /*!< \brief Padding to one  64B cache line */
+#endif
+#if defined(_ARCH_PPC)
+    uint64_t    _padding[10]; /*!< \brief Padding to one 128B cache line */
+#endif
 } PerfmonCounter;
 
 
