@@ -5,13 +5,13 @@
  *
  *      Description:  Header file of perfmon module for AMD Family 17 (ZEN)
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
- *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -41,6 +41,7 @@ int perfmon_init_zen(int cpu_id)
     lock_acquire((int*) &socket_lock[affinity_thread2socket_lookup[cpu_id]], cpu_id);
     lock_acquire((int*) &core_lock[affinity_thread2core_lookup[cpu_id]], cpu_id);
     lock_acquire((int*) &sharedl3_lock[affinity_thread2sharedl3_lookup[cpu_id]], cpu_id);
+    lock_acquire((int*) &numa_lock[affinity_thread2numa_lookup[cpu_id]], cpu_id);
     return 0;
 }
 
@@ -156,7 +157,7 @@ int k17_uncore_setup(int cpu_id, RegisterIndex index, PerfmonEvent* event)
 {
     uint64_t flags = 0x0ULL;
 
-    if (socket_lock[affinity_thread2socket_lookup[cpu_id]] != cpu_id)
+    if (numa_lock[affinity_thread2numa_lookup[cpu_id]] != cpu_id)
     {
         return 0;
     }
@@ -164,7 +165,7 @@ int k17_uncore_setup(int cpu_id, RegisterIndex index, PerfmonEvent* event)
     flags |= ((uint64_t)(event->eventId>>8)<<32) + (event->umask<<8) + (event->eventId & ~(0xF00U));
     if (flags != currentConfig[cpu_id][index])
     {
-        VERBOSEPRINTREG(cpu_id, counter_map[index].configRegister, LLU_CAST flags, SETUP_UNCORE);
+        VERBOSEPRINTREG(cpu_id, counter_map[index].configRegister, LLU_CAST flags, SETUP_MBOX0);
         CHECK_MSR_WRITE_ERROR(HPMwrite(cpu_id, MSR_DEV, counter_map[index].configRegister, flags));
         currentConfig[cpu_id][index] = flags;
     }
@@ -198,7 +199,7 @@ int perfmon_setupCounterThread_zen(int thread_id, PerfmonEventSet* eventSet)
             case FIXED:
                 fixed_flags |= k17_fixed_setup(cpu_id, index, event);
                 break;
-            case UNCORE:
+            case MBOX0:
                 k17_uncore_setup(cpu_id, index, event);
                 break;
             default:
@@ -224,6 +225,7 @@ int perfmon_startCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
     int haveSLock = 0;
     int haveL3Lock = 0;
     int haveCLock = 0;
+    int haveMLock = 0;
     uint64_t flags = 0x0ULL;
     int cpu_id = groupSet->threads[thread_id].processorId;
 
@@ -231,6 +233,10 @@ int perfmon_startCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
     {
         haveSLock = 1;
     }
+    if (numa_lock[affinity_thread2numa_lookup[cpu_id]] == cpu_id)
+    {
+        haveMLock = 1;
+    }
     if (sharedl3_lock[affinity_thread2sharedl3_lookup[cpu_id]] == cpu_id)
     {
         haveL3Lock = 1;
@@ -256,7 +262,7 @@ int perfmon_startCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
             eventSet->events[i].threadCounter[thread_id].startData = 0;
             eventSet->events[i].threadCounter[thread_id].counterData = 0;
             if ((type == PMC) ||
-                ((type == UNCORE) && (haveSLock)) ||
+                ((type == MBOX0) && (haveMLock)) ||
                 ((type == CBOX0) && (haveL3Lock)))
             {
                 VERBOSEPRINTREG(cpu_id, counter, LLU_CAST 0x0ULL, RESET_CTR);
@@ -295,6 +301,7 @@ int perfmon_stopCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
     int haveSLock = 0;
     int haveL3Lock = 0;
     int haveCLock = 0;
+    int haveMLock = 0;
     uint64_t counter_result = 0x0ULL;
     int cpu_id = groupSet->threads[thread_id].processorId;
 
@@ -302,6 +309,10 @@ int perfmon_stopCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
     {
         haveSLock = 1;
     }
+    if (numa_lock[affinity_thread2numa_lookup[cpu_id]] == cpu_id)
+    {
+        haveMLock = 1;
+    }
     if (sharedl3_lock[affinity_thread2sharedl3_lookup[cpu_id]] == cpu_id)
     {
         haveL3Lock = 1;
@@ -325,7 +336,7 @@ int perfmon_stopCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
             uint32_t reg = counter_map[index].configRegister;
             uint32_t counter = counter_map[index].counterRegister;
             if ((type == PMC) ||
-                ((type == UNCORE) && (haveSLock)) ||
+                ((type == MBOX0) && (haveMLock)) ||
                 ((type == CBOX0) && (haveL3Lock)))
             {
                 CHECK_MSR_READ_ERROR(HPMread(cpu_id, MSR_DEV, reg, &flags));
@@ -379,6 +390,7 @@ int perfmon_readCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
     int haveSLock = 0;
     int haveL3Lock = 0;
     int haveCLock = 0;
+    int haveMLock = 0;
     uint64_t counter_result = 0x0ULL;
     int cpu_id = groupSet->threads[thread_id].processorId;
 
@@ -386,6 +398,10 @@ int perfmon_readCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
     {
         haveSLock = 1;
     }
+    if (numa_lock[affinity_thread2numa_lookup[cpu_id]] == cpu_id)
+    {
+        haveMLock = 1;
+    }
     if (sharedl3_lock[affinity_thread2sharedl3_lookup[cpu_id]] == cpu_id)
     {
         haveL3Lock = 1;
@@ -409,7 +425,7 @@ int perfmon_readCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
             uint32_t counter = counter_map[index].counterRegister;
             uint64_t* current = &(eventSet->events[i].threadCounter[thread_id].counterData);
             if ((type == PMC) ||
-                ((type == UNCORE) && (haveSLock)) ||
+                ((type == MBOX0) && (haveMLock)) ||
                 ((type == CBOX0) && (haveL3Lock)))
             {
                 CHECK_MSR_READ_ERROR(HPMread(cpu_id, MSR_DEV, counter, &counter_result));
@@ -456,12 +472,17 @@ int perfmon_finalizeCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
 {
     int haveSLock = 0;
     int haveL3Lock = 0;
+    int haveMLock = 0;
     int cpu_id = groupSet->threads[thread_id].processorId;
 
     if (socket_lock[affinity_thread2socket_lookup[cpu_id]] == cpu_id)
     {
         haveSLock = 1;
     }
+    if (numa_lock[affinity_thread2numa_lookup[cpu_id]] == cpu_id)
+    {
+        haveMLock = 1;
+    }
     if (sharedl3_lock[affinity_thread2sharedl3_lookup[cpu_id]] == cpu_id)
     {
         haveL3Lock = 1;
@@ -476,7 +497,7 @@ int perfmon_finalizeCountersThread_zen(int thread_id, PerfmonEventSet* eventSet)
         }
         RegisterIndex index = eventSet->events[i].index;
         if ((type == PMC) ||
-            ((type == UNCORE) && (haveSLock)) ||
+            ((type == MBOX0) && (haveMLock)) ||
             ((type == CBOX0) && (haveL3Lock)))
         {
             if (counter_map[index].configRegister != 0x0)
