@@ -5,13 +5,13 @@
  *
  *      Description:  Counter Header File of perfmon module for AMD Family 17
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
- *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -84,24 +84,24 @@ static RegisterMap zen_counter_map[NUM_COUNTERS_ZEN] = {
     {"PWR0", PMC13, POWER, 0, MSR_AMD17_RAPL_CORE_STATUS, 0, 0},
     {"PWR1", PMC14, POWER, 0, MSR_AMD17_RAPL_PKG_STATUS, 0, 0},
     /* Northbridge counters */
-    {"UPMC0",PMC15, UNCORE, MSR_AMD16_NB_PERFEVTSEL0, MSR_AMD16_NB_PMC0, 0, 0},
-    {"UPMC1",PMC16, UNCORE, MSR_AMD16_NB_PERFEVTSEL1, MSR_AMD16_NB_PMC1, 0, 0},
-    {"UPMC2",PMC17, UNCORE, MSR_AMD16_NB_PERFEVTSEL2, MSR_AMD16_NB_PMC2, 0, 0},
-    {"UPMC3",PMC18, UNCORE, MSR_AMD16_NB_PERFEVTSEL3, MSR_AMD16_NB_PMC3, 0, 0}
+    {"DFC0",PMC15, MBOX0, MSR_AMD16_NB_PERFEVTSEL0, MSR_AMD16_NB_PMC0, 0, 0},
+    {"DFC1",PMC16, MBOX0, MSR_AMD16_NB_PERFEVTSEL1, MSR_AMD16_NB_PMC1, 0, 0},
+    {"DFC2",PMC17, MBOX0, MSR_AMD16_NB_PERFEVTSEL2, MSR_AMD16_NB_PMC2, 0, 0},
+    {"DFC3",PMC18, MBOX0, MSR_AMD16_NB_PERFEVTSEL3, MSR_AMD16_NB_PMC3, 0, 0}
 };
 
 static BoxMap zen_box_map[NUM_UNITS] = {
     [FIXED] = {0, 0, 0, 0, 0, 0, 64},
     [PMC] = {0, 0, 0, 0, 0, 0, 48},
     [CBOX0] = {0, 0, 0, 0, 0, 0, 48},
-    [UNCORE] = {0, 0, 0, 0, 0, 0, 48},
+    [MBOX0] = {0, 0, 0, 0, 0, 0, 48},
     [POWER] = {0, 0, 0, 0, 0, 0, 32},
 };
 
 static char* zen_translate_types[NUM_UNITS] = {
-    [FIXED] = "/sys/bus/event_source/devices/cpu",
+    [FIXED] = "/sys/bus/event_source/devices/msr",
     [PMC] = "/sys/bus/event_source/devices/cpu",
     [POWER] = "/sys/bus/event_source/devices/power",
-    [CBOX0] = "/sys/bus/event_source/devices/amd_l2",
-    [UNCORE] = "/sys/bus/event_source/devices/amd_nb",
+    [CBOX0] = "/sys/bus/event_source/devices/amd_l3",
+    [MBOX0] = "/sys/bus/event_source/devices/amd_df",
 };
