@@ -5,14 +5,14 @@
  *
  *      Description:  Header File of registers.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
- *                Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -93,21 +93,34 @@ typedef enum {
     PMC331, PMC332, PMC333, PMC334, PMC335, PMC336,
     PMC337, PMC338, PMC339, PMC340, PMC341, PMC342,
     PMC343, PMC344, PMC345, PMC346, PMC347, PMC348,
+    PMC349, PMC350, PMC351, PMC352, PMC353, PMC354,
+    PMC355, PMC356, PMC357, PMC358, PMC359,
+    PMC360, PMC361, PMC362, PMC363, PMC364, PMC365, PMC366, PMC367, PMC368, PMC369,
+    PMC370, PMC371, PMC372, PMC373, PMC374, PMC375, PMC376, PMC377, PMC378, PMC379,
+    PMC380, PMC381, PMC382, PMC383, PMC384, PMC385, PMC386, PMC387, PMC388, PMC389,
+    PMC390, PMC391, PMC392, PMC393, PMC394, PMC395, PMC396, PMC397, PMC398, PMC399, 
+    PMC400, PMC401, PMC402, PMC403, PMC404, PMC405, PMC406, PMC407, PMC408, PMC409, 
     NUM_PMC
 } RegisterIndex;
 
 typedef enum {
-    PMC = 0, FIXED, THERMAL,
+    PMC = 0, FIXED, PERF, THERMAL, VOLTAGE, METRICS,
     POWER, UNCORE, MBOX0,
     MBOX1, MBOX2, MBOX3,
     MBOX4, MBOX5, MBOX6, MBOX7,
+    MBOX8, MBOX9, MBOX10, MBOX11,
+    MBOX12,
     MBOX0FIX, MBOX1FIX, MBOX2FIX,
     MBOX3FIX, MBOX4FIX, MBOX5FIX,
-    MBOX6FIX, MBOX7FIX,
-    BBOX0, BBOX1,
+    MBOX6FIX, MBOX7FIX, MBOX8FIX,
+    MBOX9FIX, MBOX10FIX, MBOX11FIX,
+    MBOX12FIX,
+    MDEV0, MDEV1, MDEV2, MDEV3,
+    MBOX0TMP,
+    BBOX0, BBOX1, BBOX2, BBOX3,
     RBOX0, RBOX1, RBOX2,
     WBOX,
-    WBOX0FIX, WBOX1FIX,
+    WBOX0FIX, WBOX1FIX, WBOX2FIX, WBOX3FIX,
     SBOX0, SBOX1, SBOX2, SBOX3,
     SBOX0FIX, SBOX1FIX, SBOX2FIX, SBOX3FIX,
     CBOX0, CBOX1, CBOX2,
@@ -122,8 +135,8 @@ typedef enum {
     CBOX27, CBOX28, CBOX29,
     CBOX30, CBOX31, CBOX32,
     CBOX33, CBOX34, CBOX35,
-    CBOX36, CBOX37,
-    PBOX, PBOX1, PBOX2, PBOX3,
+    CBOX36, CBOX37, CBOX38, CBOX39,
+    PBOX, PBOX1, PBOX2, PBOX3, PBOX4, PBOX5,
     UBOX,
     UBOXFIX,
     IBOX0, IBOX1, IBOX2, IBOX3, IBOX4, IBOX5,
@@ -144,7 +157,10 @@ typedef enum {
 static char* RegisterTypeNames[MAX_UNITS] = {
     [PMC] = "Core-local general purpose counters",
     [FIXED] = "Fixed counters",
+    [PERF] = "Perf counters",
     [THERMAL] = "Thermal",
+    [VOLTAGE] = "Voltage of hardware thread",
+    [METRICS] = "Performance metrics provided by Intel systems starting with Intel Icelake",
     [POWER] = "Energy/Power counters (RAPL)",
     [UNCORE] = "Socket-local general/fixed purpose counters",
     [MBOX0] = "Memory Controller 0 Channel 0",
@@ -163,15 +179,27 @@ static char* RegisterTypeNames[MAX_UNITS] = {
     [MBOX5FIX] = "Memory Controller 1 Channel 1 Fixed Counter",
     [MBOX6FIX] = "Memory Controller 1 Channel 2 Fixed Counter",
     [MBOX7FIX] = "Memory Controller 1 Channel 3 Fixed Counter",
+    [MBOX0TMP] = "PP0/PP1 Temperature Sensor",
+#ifdef _ARCH_PPC
+    [BBOX0] = "Memory controller synchronous (port 0 & 1)",
+    [BBOX1] = "Memory controller synchronous (port 2 & 3)",
+#else
     [BBOX0] = "Home Agent box 0",
     [BBOX1] = "Home Agent box 1",
+#endif
     [RBOX0] = "Routing box 0",
     [RBOX1] = "Routing box 1",
     [RBOX2] = "Routing box 2",
     [WBOX] = "Power control box",
     [WBOX0FIX] = "Power control box fixed counter 0",
     [WBOX1FIX] = "Power control box fixed counter 1",
+    [WBOX2FIX] = "Power control box fixed counter 2",
+    [WBOX3FIX] = "Power control box fixed counter 3",
+#ifdef _ARCH_PPC
+    [SBOX0] = "PowerBus",
+#else
     [SBOX0] = "QPI Link Layer box 0",
+#endif
     [SBOX1] = "QPI Link Layer box 1",
     [SBOX2] = "QPI Link Layer box 2",
     [SBOX3] = "QPI Link Layer box 3",
@@ -217,10 +245,14 @@ static char* RegisterTypeNames[MAX_UNITS] = {
     [CBOX35] = "Caching Agent box 35",
     [CBOX36] = "Caching Agent box 36",
     [CBOX37] = "Caching Agent box 37",
-    [PBOX] = "Physical Layer box",
-    [PBOX1] = "Physical Layer box",
-    [PBOX2] = "Physical Layer box",
-    [PBOX3] = "Physical Layer box",
+    [CBOX38] = "Caching Agent box 38",
+    [CBOX39] = "Caching Agent box 39",
+    [PBOX] = "Physical Layer box 0",
+    [PBOX1] = "Physical Layer box 1",
+    [PBOX2] = "Physical Layer box 2",
+    [PBOX3] = "Physical Layer box 3",
+    [PBOX4] = "Physical Layer box 4",
+    [PBOX5] = "Physical Layer box 5",
     [UBOX] = "System Configuration box",
     [UBOXFIX] = "System Configuration box fixed counter",
     [IBOX0] = "Coherency Maintainer for IIO traffic",
@@ -229,8 +261,15 @@ static char* RegisterTypeNames[MAX_UNITS] = {
     [IBOX3] = "Coherency Maintainer for IIO traffic",
     [IBOX4] = "Coherency Maintainer for IIO traffic",
     [IBOX5] = "Coherency Maintainer for IIO traffic",
+#ifdef _ARCH_PPC
+    [QBOX0] = "Xlink 0",
+    [QBOX1] = "Xlink 1",
+    [QBOX2] = "Xlink 2",
+#else
     [QBOX0] = "QPI Link Layer 0",
     [QBOX1] = "QPI Link Layer 1",
+    [QBOX2] = "QPI Link Layer 2",
+#endif
     [QBOX0FIX] = "QPI Link Layer rate status 0",
     [QBOX1FIX] = "QPI Link Layer rate status 1",
     [EUBOX0] = "Embedded DRAM controller 0",
@@ -299,10 +338,12 @@ static char* RegisterTypeNames[MAX_UNITS] = {
             ERROR_PRINT(Cannot set out-of-bounds type %d, (type)); \
         }
 #define MEASURE_CORE(eventset) \
-        (eventset->regTypeMask1 & (REG_TYPE_MASK(PMC)|REG_TYPE_MASK(FIXED)))
+        (eventset->regTypeMask1 & (REG_TYPE_MASK(PMC)|REG_TYPE_MASK(FIXED)|REG_TYPE_MASK(METRICS)))
+
+#define MEASURE_METRICS(eventset) ((eventset)->regTypeMask1 & (REG_TYPE_MASK(METRICS))
 
 #define MEASURE_UNCORE(eventset) \
-        (eventset->regTypeMask1 & ~(REG_TYPE_MASK(PMC)|REG_TYPE_MASK(FIXED)|REG_TYPE_MASK(THERMAL)|REG_TYPE_MASK(POWER)) || eventset->regTypeMask2 || eventset->regTypeMask3 || eventset->regTypeMask4)
+        (eventset->regTypeMask1 & ~(REG_TYPE_MASK(PMC)|REG_TYPE_MASK(FIXED)|REG_TYPE_MASK(THERMAL)|REG_TYPE_MASK(VOLTAGE)|REG_TYPE_MASK(PERF)|REG_TYPE_MASK(POWER)|REG_TYPE_MASK(METRICS)) || eventset->regTypeMask2 || eventset->regTypeMask3 || eventset->regTypeMask4)
 
 
 typedef struct {
