@@ -5,14 +5,14 @@
  *
  *      Description:  Header File of topology module.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
- *                Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -34,7 +34,9 @@
 #include <stdlib.h>
 #include <stdio.h>
 
+#if !defined(__ARM_ARCH_7A__) && !defined(__ARM_ARCH_8A__)
 #include <topology_cpuid.h>
+#endif
 #include <topology_proc.h>
 #ifdef LIKWID_USE_HWLOC
 #include <topology_hwloc.h>
@@ -42,7 +44,7 @@
 #include <types.h>
 #include <tree.h>
 
-#define MAX_FEATURE_STRING_LENGTH 512
+#define MAX_FEATURE_STRING_LENGTH 512 /* only used in deprecated topology_cpuid.c */
 #define MAX_MODEL_STRING_LENGTH 512
 
 struct topology_functions {
@@ -101,6 +103,16 @@ struct topology_functions {
 #define KABYLAKE1            0x8EU
 #define KABYLAKE2            0x9EU
 #define CANNONLAKE           0x66U
+#define ICELAKE1             0x7EU
+#define ICELAKE2             0x7DU
+#define ROCKETLAKE           0xA7U
+#define COMETLAKE1           0xA5U
+#define COMETLAKE2           0xA6U
+#define ICELAKEX1            0x6AU
+#define ICELAKEX2            0x6CU
+#define SNOWRIDGEX           0x86U
+#define TIGERLAKE1           0x8CU
+#define TIGERLAKE2           0x8DU
 
 /* Intel MIC */
 #define XEON_PHI           0x01U
@@ -126,18 +138,69 @@ struct topology_functions {
 #define ATHLON64_G1     0x6FU
 #define ATHLON64_G2     0x7FU
 #define ZEN_RYZEN       0x01
+#define ZENPLUS_RYZEN   0x08
+#define ZENPLUS_RYZEN2  0x18
+#define ZEN2_RYZEN      0x31
+#define ZEN2_RYZEN2     0x71
+#define ZEN2_RYZEN3     0x60
+#define ZEN3_RYZEN      0x01
+#define ZEN3_RYZEN2     0x21
+#define ZEN3_RYZEN3     0x50
 
+/* ARM */
+#define  ARM7L          0x3U
+#define  ARMV7L         0xC07U
+#define  ARM_CORTEX_A35     0xD04U
+#define  ARM_CORTEX_A53     0xD03U
+#define  ARM_CORTEX_A57     0xD07U
+#define  ARM_CORTEX_A72     0xD08U
+#define  ARM_CORTEX_A73     0xD09U
+#define  CAV_THUNDERX	0x0A0U
+#define  CAV_THUNDERX88	0x0A1U
+#define  CAV_THUNDERX81	0x0A2U
+#define  CAV_THUNDERX82	0x0A3U
+#define  CAV_THUNDERX2T99  0x0AFU
+#define  CAV_THUNDERX2T99P1 0x516U
+#define  NV_DENVER1	0x00U
+#define  NV_DENVER2	0x03U
+#define  APP_XGENE1	0x00U
+#define  ARM_NEOVERSE_N1 0xD0CU
+#define  FUJITSU_A64FX 0x001U
+
+/* ARM vendors */
+#define DEFAULT_ARM	0x41U
+#define CAVIUM1		0x42U
+#define CAVIUM2		0x43U
+#define NVIDIA_ARM	0x4EU
+#define APPLIEDMICRO	0x50U
+#define QUALCOMM	0x51U
+#define SAMSUNG		0x53U
+#define APPLE		0x67U
+#define MARVELL		0x56U
+#define INTEL_ARM	0x69U
+#define FUJITSU_ARM 0x46U
+
+/* POWER */
+#define POWER7          0x7U
+#define POWER8          0x8U
+#define POWER9          0x9U
 
 #define  P6_FAMILY        0x6U
 #define  MIC_FAMILY       0xBU
 #define  NETBURST_FAMILY  0xFFU
-#define  ZEN_FAMILY       0x17
+#define  ZEN_FAMILY       0x17U
+#define  ZEN3_FAMILY      0x19U
 #define  K15_FAMILY       0x15U
 #define  K16_FAMILY       0x16U
 #define  K10_FAMILY       0x10U
 #define  K8_FAMILY        0xFU
+#define  ARMV7_FAMILY     0x7U
+#define  ARMV8_FAMILY     0x8U
+#define  PPC_FAMILY       0x42U
 
 extern int cpu_count(cpu_set_t* set);
+extern int likwid_cpu_online(int cpu_id);
+extern int likwid_sysfs_list_len(char* sysfsfile);
 
 static inline int cpuid_hasFeature(FeatureBit bit)
 {
