@@ -5,13 +5,13 @@
  *
  *      Description:  Header File of topology backend using the hwloc library
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
- *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -33,6 +33,15 @@
 #include <hwloc.h>
 #include <sched.h>
 
+#ifdef USE_INTERNAL_HWLOC
+#define HWLOC_PREFIX likwid_
+#else
+#define HWLOC_PREFIX
+#endif
+#define LIKWID_HWLOC_MUNGE_NAME(a, b) LIKWID_HWLOC_MUNGE_NAME2(a, b)
+#define LIKWID_HWLOC_MUNGE_NAME2(a, b) a ## b
+#define LIKWID_HWLOC_NAME(name) LIKWID_HWLOC_MUNGE_NAME(HWLOC_PREFIX, hwloc_ ## name)
+
 extern hwloc_topology_t hwloc_topology;
 
 int likwid_hwloc_record_objs_of_type_below_obj(hwloc_topology_t t, hwloc_obj_t obj, hwloc_obj_type_t type, int* index, uint32_t **list);
