@@ -5,14 +5,14 @@
  *
  *      Description:  Marker API interface of module perfmon
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com
- *                Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -53,6 +53,8 @@
 #include <access.h>
 #include <affinity.h>
 #include <perfmon.h>
+#include <bstrlib.h>
+#include <voltage.h>
 
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
@@ -128,10 +130,15 @@ calculateMarkerResult(RegisterIndex index, uint64_t start, uint64_t stop, int ov
     {
         result *= power_getEnergyUnit(getCounterTypeOffset(index));
     }
-    else if (counter_map[index].type == THERMAL)
+    else if ((counter_map[index].type == THERMAL) ||
+             (counter_map[index].type == MBOX0TMP))
     {
         result = (double)stop;
     }
+    else if (counter_map[index].type == VOLTAGE)
+    {
+        result = voltage_value(stop);
+    }
     return result;
 }
 
@@ -152,6 +159,8 @@ likwid_markerInit(void)
     char* cThreadStr = getenv("LIKWID_THREADS");
     char* filepath = getenv("LIKWID_FILEPATH");
     char* perfpid = getenv("LIKWID_PERF_EXECPID");
+    char* debugStr = getenv("LIKWID_DEBUG");
+    char* pinStr = getenv("LIKWID_PIN");
     char execpid[20];
     /* Dirty hack to avoid nonnull warnings */
     int (*ownatoi)(const char*);
@@ -182,12 +191,12 @@ likwid_markerInit(void)
     affinity_init();
     hashTable_init();
 
-//#ifndef LIKWID_USE_PERFEVENT
+#ifndef LIKWID_USE_PERFEVENT
     HPMmode(atoi(modeStr));
-//#endif
-    if (getenv("LIKWID_DEBUG") != NULL)
+#endif
+    if (debugStr != NULL)
     {
-        perfmon_verbosity = atoi(getenv("LIKWID_DEBUG"));
+        perfmon_verbosity = atoi(debugStr);
         verbosity = perfmon_verbosity;
     }
 
@@ -201,7 +210,7 @@ likwid_markerInit(void)
     bdestroy(bThreadStr);
     bstrListDestroy(threadTokens);
 
-    if (getenv("LIKWID_PIN") != NULL)
+    if (pinStr != NULL)
     {
         likwid_pinThread(threads2Cpu[0]);
         if (getenv("OMP_NUM_THREADS") != NULL)
@@ -287,6 +296,7 @@ likwid_markerThreadInit(void)
     {
         return;
     }
+    char* pinStr = getenv("LIKWID_PIN");
 
     pthread_mutex_lock(&globalLock);
     t = pthread_self();
@@ -304,7 +314,7 @@ likwid_markerThreadInit(void)
     }
     pthread_mutex_unlock(&globalLock);
 
-    if (getenv("LIKWID_PIN") != NULL)
+    if (pinStr != NULL)
     {
         cpu_set_t cpuset;
         CPU_ZERO(&cpuset);
@@ -352,8 +362,6 @@ likwid_markerClose(void)
     int numberOfThreads = 0;
     int numberOfRegions = 0;
     char* markerfile = NULL;
-    int lineidx = 0;
-    char line[1024];
     int* validRegions = NULL;
 
     if ( ! likwid_init )
@@ -409,15 +417,19 @@ likwid_markerClose(void)
         DEBUG_PRINT(DEBUGLEV_DEVELOP,
                 Creating Marker file %s with %d regions %d groups and %d threads,
                 markerfile, newNumberOfRegions, numberOfGroups, numberOfThreads);
-        fprintf(file,"%d %d %d\n",numberOfThreads, newNumberOfRegions, numberOfGroups);
-        DEBUG_PRINT(DEBUGLEV_DEVELOP, %d %d %d, numberOfThreads, newNumberOfRegions, numberOfGroups);
+        bstring thread_regs_grps = bformat("%d %d %d", numberOfThreads, newNumberOfRegions, numberOfGroups);
+        fprintf(file,"%s\n", bdata(thread_regs_grps));
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, %s, bdata(thread_regs_grps));
+        bdestroy(thread_regs_grps);
 
         for (int i=0; i<numberOfRegions; i++)
         {
             if (validRegions[i] == 0)
                 continue;
-            fprintf(file,"%d:%s\n",newRegionID,bdata(results[i].tag));
-            DEBUG_PRINT(DEBUGLEV_DEVELOP, %d:%s, newRegionID,bdata(results[i].tag));
+            bstring tmp = bformat("%d:%s", newRegionID, bdata(results[i].tag));
+            fprintf(file,"%s\n", bdata(tmp));
+            DEBUG_PRINT(DEBUGLEV_DEVELOP, %s, bdata(tmp));
+            bdestroy(tmp);
             newRegionID++;
         }
         newRegionID = 0;
@@ -425,25 +437,25 @@ likwid_markerClose(void)
         {
             if (validRegions[i] == 0)
                 continue;
+            int nevents = groupSet->groups[results[i].groupID].numberOfEvents;
             for (int j=0; j<numberOfThreads; j++)
             {
-                fprintf(file,"%d ",newRegionID);
-                fprintf(file,"%d ",results[i].groupID);
-                fprintf(file,"%d ",results[i].cpulist[j]);
-                fprintf(file,"%u ",results[i].count[j]);
-                fprintf(file,"%e ",results[i].time[j]);
-                fprintf(file,"%d ",groupSet->groups[results[i].groupID].numberOfEvents);
-                lineidx = sprintf(&(line[0]), "%d %d %d %u %e %d ",
-                        newRegionID, results[i].groupID,results[i].cpulist[j],results[i].count[j],
-                        results[i].time[j],groupSet->groups[results[i].groupID].numberOfEvents);
+                bstring l = bformat("%d %d %d %u %e %d ", newRegionID,
+                                                          results[i].groupID,
+                                                          results[i].cpulist[j],
+                                                          results[i].count[j],
+                                                          results[i].time[j],
+                                                          nevents);
 
-                for (int k=0; k<groupSet->groups[results[i].groupID].numberOfEvents; k++)
+                for (int k=0; k < MIN(nevents, NUM_PMC); k++)
                 {
-                    fprintf(file,"%e ",results[i].counters[j][k]);
-                    lineidx += sprintf(&(line[lineidx]), "%e ", results[i].counters[j][k]);
+                    bstring tmp = bformat("%e ", results[i].counters[j][k]);
+                    bconcat(l, tmp);
+                    bdestroy(tmp);
                 }
-                fprintf(file,"\n");
-                DEBUG_PRINT(DEBUGLEV_DEVELOP, %s,line);
+                fprintf(file,"%s\n", bdata(l));
+                DEBUG_PRINT(DEBUGLEV_DEVELOP, %s, bdata(l));
+                bdestroy(l);
             }
             newRegionID++;
         }
@@ -454,17 +466,10 @@ likwid_markerClose(void)
         fprintf(stderr, "Cannot open file %s\n", markerfile);
         fprintf(stderr, "%s", strerror(errno));
     }
-    free(validRegions);
-}
-
-void __attribute__((destructor (101))) likwid_markerCloseDestruct(void)
-{
-    LikwidResults* results = NULL;
-    int numberOfThreads = 0;
-    int numberOfRegions = 0;
-    if (!likwid_init)
-        return;
-    hashTable_finalize(&numberOfThreads, &numberOfRegions, &results);
+    if (validRegions)
+    {
+        free(validRegions);
+    }
     if ((numberOfThreads == 0)||(numberOfThreads == 0))
     {
         return;
@@ -481,7 +486,6 @@ void __attribute__((destructor (101))) likwid_markerCloseDestruct(void)
         free(results[i].cpulist);
         free(results[i].counters);
     }
-
     if (results != NULL)
     {
         free(results);
@@ -604,6 +608,10 @@ likwid_markerStopRegion(const char* regionTag)
     if (results->state != MARKER_STATE_START)
     {
         fprintf(stderr, "WARN: Stopping an unknown/not-started region %s\n", regionTag);
+        if (use_locks == 1)
+        {
+            pthread_mutex_lock(&threadLocks[myCPU]);
+        }
         return -EFAULT;
     }
     results->groupID = groupSet->activeGroup;
@@ -624,7 +632,9 @@ likwid_markerStopRegion(const char* regionTag)
                                             groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData,
                                             groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].overflows -
                                             results->StartOverflows[i]);
-            if (counter_map[groupSet->groups[groupSet->activeGroup].events[i].index].type != THERMAL)
+            if ((counter_map[groupSet->groups[groupSet->activeGroup].events[i].index].type != THERMAL) &&
+                (counter_map[groupSet->groups[groupSet->activeGroup].events[i].index].type != VOLTAGE) &&
+                (counter_map[groupSet->groups[groupSet->activeGroup].events[i].index].type != MBOX0TMP))
             {
                 results->PMcounters[i] += result;
             }
