@@ -5,13 +5,13 @@
  *
  *      Description:  C part of the Likwid Lua interface
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
- *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -51,6 +51,7 @@
 #include <tree.h>
 #include <access.h>
 #include <bstrlib.h>
+#include <perfmon.h>
 
 #ifdef COLOR
 #include <textcolor.h>
@@ -61,6 +62,7 @@
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
 static int topology_isInitialized = 0;
+static int gputopology_isInitialized = 0;
 static int numa_isInitialized = 0;
 static int affinity_isInitialized = 0;
 static int perfmon_isInitialized = 0;
@@ -69,6 +71,8 @@ static int power_isInitialized = 0;
 static int power_hasRAPL = 0;
 static int config_isInitialized = 0;
 
+static int nvmon_initialized = 0;
+
 /* #####   VARIABLES  -  EXPORTED VARIABLES   ############################# */
 
 CpuInfo_t cpuinfo = NULL;
@@ -732,6 +736,9 @@ lua_likwid_getCpuInfo(lua_State* L)
     lua_pushstring(L,"features");
     lua_pushstring(L,cpuinfo->features);
     lua_settable(L,-3);
+    lua_pushstring(L,"architecture");
+    lua_pushstring(L,cpuinfo->architecture);
+    lua_settable(L,-3);
     lua_pushstring(L,"isIntel");
     lua_pushinteger(L,cpuinfo->isIntel);
     lua_settable(L,-3);
@@ -830,6 +837,9 @@ lua_likwid_getCpuTopology(lua_State* L)
         lua_pushstring(L,"apicId");
         lua_pushinteger(L, (lua_Integer)(cputopo->threadPool[i].apicId));
         lua_settable(L,-3);
+        lua_pushstring(L,"dieId");
+        lua_pushinteger(L, (lua_Integer)(cputopo->threadPool[i].dieId));
+        lua_settable(L,-3);
         lua_pushstring(L,"inCpuSet");
         lua_pushinteger(L, (lua_Integer)(cputopo->threadPool[i].inCpuSet));
         lua_settable(L,-3);
@@ -979,6 +989,7 @@ lua_likwid_getEventsAndCounters(lua_State* L)
     }
     perfmon_init_maps();
     perfmon_check_counter_map(0);
+    char** archTypeNames = getArchRegisterTypeNames();
     lua_newtable(L);
     lua_pushstring(L,"Counters");
     lua_newtable(L);
@@ -1009,7 +1020,14 @@ lua_likwid_getEventsAndCounters(lua_State* L)
         lua_pushinteger(L, (lua_Integer)( counter_map[i-1].type));
         lua_settable(L,-3);
         lua_pushstring(L,"TypeName");
-        lua_pushstring(L, RegisterTypeNames[counter_map[i-1].type]);
+        if (archTypeNames && archTypeNames[counter_map[i-1].type] != NULL)
+        {
+            lua_pushstring(L, archTypeNames[counter_map[i-1].type]);
+        }
+        else
+        {
+            lua_pushstring(L, RegisterTypeNames[counter_map[i-1].type]);
+        }
         lua_settable(L,-3);
         lua_pushstring(L,"Index");
         lua_pushinteger(L, (lua_Integer)(counter_map[i-1].index));
@@ -1242,6 +1260,31 @@ lua_likwid_setMemInterleaved(lua_State* L)
     return 0;
 }
 
+static int
+lua_likwid_setMembind(lua_State* L)
+{
+    int ret;
+    int nrThreads = luaL_checknumber(L,1);
+    luaL_argcheck(L, nrThreads > 0, 1, "Thread count must be greater than 0");
+    int cpus[nrThreads];
+    if (!lua_istable(L, -1)) {
+      lua_pushstring(L,"No table given as second argument");
+      lua_error(L);
+    }
+    for (ret = 1; ret<=nrThreads; ret++)
+    {
+        lua_rawgeti(L,-1,ret);
+#if LUA_VERSION_NUM == 501
+        cpus[ret-1] = ((lua_Integer)lua_tointeger(L,-1));
+#else
+        cpus[ret-1] = ((lua_Unsigned)lua_tointegerx(L,-1, NULL));
+#endif
+        lua_pop(L,1);
+    }
+    numa_setMembind(cpus, nrThreads);
+    return 0;
+}
+
 static int
 lua_likwid_getAffinityInfo(lua_State* L)
 {
@@ -1356,14 +1399,15 @@ lua_likwid_cpustr_to_cpulist(lua_State* L)
     int* cpulist = (int*) malloc(cputopo->numHWThreads * sizeof(int));
     if (cpulist == NULL)
     {
-        lua_pushstring(L,"Cannot allocate data for the CPU list");
-        lua_error(L);
+        lua_pushnumber(L, 0);
+        return 1;
     }
     ret = cpustr_to_cpulist(cpustr, cpulist, cputopo->numHWThreads);
     if (ret <= 0)
     {
-        lua_pushstring(L,"Cannot parse cpustring");
-        lua_error(L);
+        free(cpulist);
+        lua_pushnumber(L, 0);
+        return 1;
     }
     lua_pushnumber(L, ret);
     lua_newtable(L);
@@ -1449,6 +1493,7 @@ lua_likwid_sockstr_to_socklist(lua_State* L)
     return 2;
 }
 
+
 static int
 lua_likwid_putAffinityInfo(lua_State* L)
 {
@@ -1495,11 +1540,11 @@ lua_likwid_getPowerInfo(lua_State* L)
         power_hasRAPL = power_init(0);
         if (power_hasRAPL > 0)
         {
-            for(i=0;i<affinity->numberOfAffinityDomains;i++)
+            for (i = 0; i < cputopo->numHWThreads; i++)
             {
-                if (bstrchrp(affinity->domains[i].tag, 'S', 0) != BSTR_ERR)
+                if (cputopo->threadPool[i].inCpuSet)
                 {
-                    HPMaddThread(affinity->domains[i].processorList[0]);
+                    HPMaddThread(cputopo->threadPool[i].apicId);
                 }
             }
             power_isInitialized = 1;
@@ -1909,18 +1954,23 @@ lua_likwid_send_signal(lua_State* L)
 
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
-void
-parse(char *line, char **argv)
+int
+parse(char *line, char **argv, int maxlen)
 {
-     while (*line != '\0') {       /* if not the end of line ....... */
-          while (*line == ' ' || *line == '\t' || *line == '\n')
+     int len = 0;
+     while (*line != '\0' && len < maxlen) {       /* if not the end of line ....... */
+          if (*line == ' ' || *line == '\t' || *line == '\n')
                *line++ = '\0';     /* replace white spaces with 0    */
           *argv++ = line;          /* save the argument position     */
+          len++;
           while (*line != '\0' && *line != ' ' &&
                  *line != '\t' && *line != '\n')
                line++;             /* skip the argument until ...    */
      }
-     *argv = '\0';                 /* mark the end of argument list  */
+     *argv = (char *)'\0';                 /* mark the end of argument list  */
+     if (len < maxlen || *line == '\0')
+         return len;
+     return -1;
 }
 
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
@@ -1937,7 +1987,7 @@ lua_likwid_startProgram(lua_State* L)
     pid_t pid, ppid;
     int status;
     char *exec;
-    char *argv[4096];
+    char *argv[MAX_NUM_CLIARGS];
     exec = (char *)luaL_checkstring(L, 1);
     int nrThreads = luaL_checknumber(L,2);
     CpuTopology_t cputopo = get_cpuTopology();
@@ -1969,13 +2019,19 @@ lua_likwid_startProgram(lua_State* L)
             lua_pop(L,1);
         }
     }
-    else
+    /*else
     {
         for (nrThreads = 0; nrThreads < cpuid_topology.numHWThreads; nrThreads++)
             cpus[nrThreads] = cpuid_topology.threadPool[nrThreads].apicId;
         nrThreads = cpuid_topology.numHWThreads;
+    }*/
+    int args = parse(exec, argv, MAX_NUM_CLIARGS);
+    if (args < 0)
+    {
+        lua_pushstring(L,"Number of CLI args greater than configured");
+        lua_error(L);
+        return 0;
     }
-    parse(exec, argv);
     ppid = getpid();
     pid = fork();
     if (pid < 0)
@@ -2010,26 +2066,34 @@ static int
 lua_likwid_checkProgram(lua_State* L)
 {
     int ret = -1;
+    int exited = 0;
     if (lua_gettop(L) == 1)
     {
         int status = 0;
         pid_t retpid = 0;
         pid_t pid = lua_tonumber(L, 1);
-        retpid = waitpid(pid, &status, WNOHANG);
+        retpid = waitpid(pid, &status, WNOHANG|WUNTRACED|WCONTINUED);
         if (retpid == pid)
         {
             if (WIFEXITED(status))
             {
                 ret = WEXITSTATUS(status);
+                exited = 1;
             }
             else if (WIFSIGNALED(status))
             {
                 ret = 128 + WTERMSIG(status);
+                exited = 1;
+            }
+            else
+            {
+                ret = 0;
             }
         }
     }
     lua_pushinteger(L, (lua_Integer)ret);
-    return 1;
+    lua_pushboolean(L, exited);
+    return 2;
 }
 
 static int
@@ -2114,7 +2178,7 @@ lua_likwid_pinProcess(lua_State* L)
 #ifdef COLOR
             color_on(BRIGHT, COLOR);
 #endif
-            printf("[likwid-pin] Main PID -> core %d - OK",  cpuID);
+            printf("[likwid-pin] Main PID -> hwthread %d - OK",  cpuID);
 #ifdef COLOR
             color_reset();
 #endif
@@ -2142,7 +2206,7 @@ lua_likwid_pinThread(lua_State* L)
 #ifdef COLOR
             color_on(BRIGHT, COLOR);
 #endif
-            printf("[likwid-pin] PID %lu -> core %d - OK", gettid(), cpuID);
+            printf("[likwid-pin] PID %lu -> hwthread %d - OK", gettid(), cpuID);
 #ifdef COLOR
             color_reset();
 #endif
@@ -2184,7 +2248,10 @@ lua_likwid_setVerbosity(lua_State* L)
     int verbosity = lua_tointeger(L,-1);
     luaL_argcheck(L, (verbosity >= 0 && verbosity <= DEBUGLEV_DEVELOP), -1,
                 "Verbosity must be between 0 (only errors) and 3 (developer)");
-    perfmon_verbosity = verbosity;
+    perfmon_setVerbosity(verbosity);
+#ifdef LIKWID_WITH_NVMON
+    nvmon_setVerbosity(verbosity);
+#endif /* LIKWID_WITH_NVMON */
     return 0;
 }
 
@@ -2285,7 +2352,7 @@ static int
 lua_likwid_getRegion(lua_State* L)
 {
     int i = 0;
-    const char* tag = (const char*)luaL_checkstring(L, -2);
+    const char* tag = (const char*)luaL_checkstring(L, -1);
     int nr_events = perfmon_getNumberOfEvents(perfmon_getIdOfActiveGroup());
     double* events = NULL;
     double time = 0.0;
@@ -2315,6 +2382,14 @@ lua_likwid_getRegion(lua_State* L)
     return 4;
 }
 
+static int
+lua_likwid_resetRegion(lua_State* L)
+{
+    const char* tag = (const char*)luaL_checkstring(L, -1);
+    lua_pushinteger(L, likwid_markerResetRegion(tag));
+    return 1;
+}
+
 static int
 lua_likwid_cpuFeatures_init(lua_State* L)
 {
@@ -2512,6 +2587,28 @@ lua_likwid_markerRegionMetric(lua_State* L)
     return 1;
 }
 
+static int
+lua_likwid_initFreq(lua_State* L)
+{
+    lua_pushnumber(L, freq_init());
+    return 1;
+}
+
+static int
+lua_likwid_finalizeFreq(lua_State* L)
+{
+    freq_finalize();
+    return 0;
+}
+
+static int
+lua_likwid_getCpuClockBase(lua_State* L)
+{
+    const int cpu_id = lua_tointeger(L,-1);
+    lua_pushnumber(L, freq_getCpuClockBase(cpu_id));
+    return 1;
+}
+
 static int
 lua_likwid_getCpuClockCurrent(lua_State* L)
 {
@@ -2528,6 +2625,14 @@ lua_likwid_getCpuClockMin(lua_State* L)
     return 1;
 }
 
+static int
+lua_likwid_getConfCpuClockMin(lua_State* L)
+{
+    const int cpu_id = lua_tointeger(L,-1);
+    lua_pushnumber(L, freq_getConfCpuClockMin(cpu_id));
+    return 1;
+}
+
 static int
 lua_likwid_setCpuClockMin(lua_State* L)
 {
@@ -2545,6 +2650,14 @@ lua_likwid_getCpuClockMax(lua_State* L)
     return 1;
 }
 
+static int
+lua_likwid_getConfCpuClockMax(lua_State* L)
+{
+    const int cpu_id = lua_tointeger(L,-1);
+    lua_pushnumber(L, freq_getConfCpuClockMax(cpu_id));
+    return 1;
+}
+
 static int
 lua_likwid_setCpuClockMax(lua_State* L)
 {
@@ -2780,6 +2893,605 @@ lua_likwid_setresuser(lua_State* L)
     return 1;
 }
 
+#ifdef LIKWID_WITH_NVMON
+
+GpuTopology_t gputopo = NULL;
+
+static int
+lua_likwid_getGpuTopology(lua_State* L)
+{
+    if (!gputopology_isInitialized)
+    {
+        if (topology_gpu_init() == EXIT_SUCCESS)
+        {
+            gputopo = get_gpuTopology();
+            gputopology_isInitialized = 1;
+        }
+        else
+        {
+            lua_pushnil(L);
+            return 1;
+        }
+    }
+    lua_newtable(L);
+    lua_pushstring(L,"numDevices");
+    lua_pushinteger(L, (lua_Integer)(gputopo->numDevices));
+    lua_settable(L,-3);
+
+    lua_pushstring(L,"devices");
+    lua_newtable(L);
+    for (int i = 0; i < gputopo->numDevices; i++)
+    {
+        GpuDevice* gpu = &gputopo->devices[i];
+        lua_pushinteger(L, i+1);
+        lua_newtable(L);
+        lua_pushstring(L,"id");
+        lua_pushinteger(L, (lua_Integer)(gpu->devid));
+        lua_settable(L,-3);
+        lua_pushstring(L,"numaNode");
+        lua_pushinteger(L, (lua_Integer)(gpu->numaNode));
+        lua_settable(L,-3);
+        lua_pushstring(L,"name");
+        lua_pushstring(L, gpu->name);
+        lua_settable(L,-3);
+        lua_pushstring(L,"short");
+        lua_pushstring(L, gpu->short_name);
+        lua_settable(L,-3);
+        lua_pushstring(L,"memory");
+        lua_pushinteger(L, (lua_Integer)(gpu->mem));
+        lua_settable(L,-3);
+        lua_pushstring(L,"ccapMajor");
+        lua_pushinteger(L, (lua_Integer)(gpu->ccapMajor));
+        lua_settable(L,-3);
+        lua_pushstring(L,"ccapMinor");
+        lua_pushinteger(L, (lua_Integer)(gpu->ccapMinor));
+        lua_settable(L,-3);
+        lua_pushstring(L,"simdWidth");
+        lua_pushinteger(L, (lua_Integer)(gpu->simdWidth));
+        lua_settable(L,-3);
+        lua_pushstring(L,"l2Size");
+        lua_pushinteger(L, (lua_Integer)(gpu->l2Size));
+        lua_settable(L,-3);
+        lua_pushstring(L,"maxThreadsPerBlock");
+        lua_pushinteger(L, (lua_Integer)(gpu->maxThreadsPerBlock));
+        lua_settable(L,-3);
+        lua_pushstring(L,"sharedMemPerBlock");
+        lua_pushinteger(L, (lua_Integer)(gpu->sharedMemPerBlock));
+        lua_settable(L,-3);
+        lua_pushstring(L,"totalConstantMemory");
+        lua_pushinteger(L, (lua_Integer)(gpu->totalConstantMemory));
+        lua_settable(L,-3);
+        lua_pushstring(L,"memPitch");
+        lua_pushinteger(L, (lua_Integer)(gpu->memPitch));
+        lua_settable(L,-3);
+        lua_pushstring(L,"regsPerBlock");
+        lua_pushinteger(L, (lua_Integer)(gpu->regsPerBlock));
+        lua_settable(L,-3);
+        lua_pushstring(L,"clockRatekHz");
+        lua_pushinteger(L, (lua_Integer)(gpu->clockRatekHz));
+        lua_settable(L,-3);
+        lua_pushstring(L,"textureAlign");
+        lua_pushinteger(L, (lua_Integer)(gpu->textureAlign));
+        lua_settable(L,-3);
+        lua_pushstring(L,"surfaceAlign");
+        lua_pushinteger(L, (lua_Integer)(gpu->surfaceAlign));
+        lua_settable(L,-3);
+        lua_pushstring(L,"memClockRatekHz");
+        lua_pushinteger(L, (lua_Integer)(gpu->memClockRatekHz));
+        lua_settable(L,-3);
+        lua_pushstring(L,"pciBus");
+        lua_pushinteger(L, (lua_Integer)(gpu->pciBus));
+        lua_settable(L,-3);
+        lua_pushstring(L,"pciDev");
+        lua_pushinteger(L, (lua_Integer)(gpu->pciDev));
+        lua_settable(L,-3);
+        lua_pushstring(L,"pciDom");
+        lua_pushinteger(L, (lua_Integer)(gpu->pciDom));
+        lua_settable(L,-3);
+        lua_pushstring(L,"maxBlockRegs");
+        lua_pushinteger(L, (lua_Integer)(gpu->maxBlockRegs));
+        lua_settable(L,-3);
+        lua_pushstring(L,"numMultiProcs");
+        lua_pushinteger(L, (lua_Integer)(gpu->numMultiProcs));
+        lua_settable(L,-3);
+        lua_pushstring(L,"maxThreadPerMultiProc");
+        lua_pushinteger(L, (lua_Integer)(gpu->maxThreadPerMultiProc));
+        lua_settable(L,-3);
+        lua_pushstring(L,"memBusWidth");
+        lua_pushinteger(L, (lua_Integer)(gpu->memBusWidth));
+        lua_settable(L,-3);
+        lua_pushstring(L,"unifiedAddrSpace");
+        lua_pushinteger(L, (lua_Integer)(gpu->unifiedAddrSpace));
+        lua_settable(L,-3);
+        lua_pushstring(L,"ecc");
+        lua_pushinteger(L, (lua_Integer)(gpu->ecc));
+        lua_settable(L,-3);
+        lua_pushstring(L,"asyncEngines");
+        lua_pushinteger(L, (lua_Integer)(gpu->asyncEngines));
+        lua_settable(L,-3);
+        lua_pushstring(L,"mapHostMem");
+        lua_pushinteger(L, (lua_Integer)(gpu->mapHostMem));
+        lua_settable(L,-3);
+        lua_pushstring(L,"integrated");
+        lua_pushinteger(L, (lua_Integer)(gpu->integrated));
+        lua_settable(L,-3);
+
+        lua_pushstring(L,"maxThreadsDim");
+        lua_newtable(L);
+        for (int j = 0; j < 3; j++)
+        {
+            lua_pushinteger(L, j+1);
+            lua_pushinteger(L, (lua_Integer)(gpu->maxThreadsDim[j]));
+            lua_settable(L,-3);
+        }
+        lua_settable(L,-3);
+
+        lua_pushstring(L,"maxGridSize");
+        lua_newtable(L);
+        for (int j = 0; j < 3; j++)
+        {
+            lua_pushinteger(L, j+1);
+            lua_pushinteger(L, (lua_Integer)(gpu->maxGridSize[j]));
+            lua_settable(L,-3);
+        }
+        lua_settable(L,-3);
+
+        lua_settable(L,-3);
+    }
+    lua_settable(L,-3);
+    return 1;
+}
+
+static int
+lua_likwid_putGpuTopology(lua_State* L)
+{
+    if (gputopology_isInitialized)
+    {
+        topology_gpu_finalize();
+    }
+    return 0;
+}
+
+static int
+lua_likwid_gpustr_to_gpulist(lua_State* L)
+{
+    int ret = 0;
+    char* gpustr = (char *)luaL_checkstring(L, 1);
+    if (!gputopology_isInitialized)
+    {
+        if (topology_gpu_init() == EXIT_SUCCESS)
+        {
+            gputopo = get_gpuTopology();
+            gputopology_isInitialized = 1;
+        }
+        else
+        {
+            lua_pushnumber(L, 0);
+            lua_pushnil(L);
+            return 2;
+        }
+    }
+    int* gpulist = (int*) malloc(gputopo->numDevices * sizeof(int));
+    if (gpulist == NULL)
+    {
+        lua_pushstring(L,"Cannot allocate data for the GPU list");
+        lua_error(L);
+    }
+    ret = gpustr_to_gpulist(gpustr, gpulist, gputopo->numDevices);
+    if (ret <= 0)
+    {
+        lua_pushstring(L,"Cannot parse GPU string");
+        lua_error(L);
+    }
+    lua_pushnumber(L, ret);
+    lua_newtable(L);
+    for (int i=0;i<ret;i++)
+    {
+        lua_pushinteger(L, (lua_Integer)( i+1));
+        lua_pushinteger(L, (lua_Integer)( gpulist[i]));
+        lua_settable(L,-3);
+    }
+    free(gpulist);
+    return 2;
+}
+
+static int
+lua_likwid_getGpuEventsAndCounters(lua_State* L)
+{
+    if (!gputopology_isInitialized)
+    {
+        if (topology_gpu_init() == EXIT_SUCCESS)
+        {
+            gputopo = get_gpuTopology();
+            gputopology_isInitialized = 1;
+        }
+        else
+        {
+            lua_pushnil(L);
+            return 1;
+        }
+    }
+
+    lua_newtable(L);
+    lua_pushstring(L,"numDevices");
+    lua_pushinteger(L, (lua_Integer)(gputopo->numDevices));
+    lua_settable(L,-3);
+
+    lua_pushstring(L,"devices");
+    lua_newtable(L);
+    for (int i = 0; i < gputopo->numDevices; i++)
+    {
+        NvmonEventList_t l;
+        GpuDevice* gpu = &gputopo->devices[i];
+        lua_pushinteger(L, gpu->devid);
+        lua_newtable(L);
+
+        int ret = nvmon_getEventsOfGpu(gpu->devid, &l);
+        if (ret == 0)
+        {
+            for (int j = 0; j < l->numEvents; j++)
+            {
+                lua_pushinteger(L, j+1);
+                lua_newtable(L);
+                lua_pushstring(L,"Name");
+                lua_pushstring(L, l->events[j].name);
+                lua_settable(L,-3);
+                if (l->events[j].desc)
+                {
+                    lua_pushstring(L,"Description");
+                    lua_pushstring(L, l->events[j].desc);
+                    lua_settable(L,-3);
+                }
+                lua_pushstring(L,"Limit");
+                lua_pushstring(L, l->events[j].limit);
+                lua_settable(L,-3);
+                lua_settable(L,-3);
+            }
+            lua_settable(L,-3);
+            nvmon_returnEventsOfGpu(l);
+        }
+    }
+    lua_settable(L,-3);
+    return 1;
+}
+
+
+static int
+lua_likwid_getGpuGroups(lua_State* L)
+{
+    int i, ret;
+    char** tmp, **infos, **longs;
+    int gpuId = lua_tonumber(L,1);
+    if (!gputopology_isInitialized)
+    {
+        if (topology_gpu_init() == EXIT_SUCCESS)
+        {
+            gputopo = get_gpuTopology();
+            gputopology_isInitialized = 1;
+        }
+        else
+        {
+            lua_pushnil(L);
+            return 1;
+        }
+    }
+    ret = nvmon_getGroups(gpuId, &tmp, &infos, &longs);
+    if (ret > 0)
+    {
+        lua_newtable(L);
+        for (i=0;i<ret;i++)
+        {
+            lua_pushinteger(L, (lua_Integer)( i+1));
+            lua_newtable(L);
+            lua_pushstring(L, "Name");
+            lua_pushstring(L, tmp[i]);
+            lua_settable(L,-3);
+            lua_pushstring(L, "Info");
+            lua_pushstring(L, infos[i]);
+            lua_settable(L,-3);
+            lua_pushstring(L, "Long");
+            lua_pushstring(L, longs[i]);
+            lua_settable(L,-3);
+            lua_settable(L,-3);
+        }
+        nvmon_returnGroups(ret, tmp, infos, longs);
+        return 1;
+    }
+    return 0;
+}
+
+
+static int
+lua_likwid_nvGetNameOfEvent(lua_State* L)
+{
+    int eventId, groupId;
+    char* tmp;
+    if (nvmon_initialized == 0)
+    {
+        return 0;
+    }
+    groupId = lua_tonumber(L,1);
+    eventId = lua_tonumber(L,2);
+    tmp = nvmon_getEventName(groupId-1, eventId-1);
+    lua_pushstring(L,tmp);
+    return 1;
+}
+
+static int
+lua_likwid_nvGetNameOfCounter(lua_State* L)
+{
+    int eventId, groupId;
+    char* tmp;
+    if (nvmon_initialized == 0)
+    {
+        return 0;
+    }
+    groupId = lua_tonumber(L,1);
+    eventId = lua_tonumber(L,2);
+    tmp = nvmon_getCounterName(groupId-1, eventId-1);
+    lua_pushstring(L,tmp);
+    return 1;
+}
+
+
+static int
+lua_likwid_nvGetNameOfMetric(lua_State* L)
+{
+    int metricId, groupId;
+    char* tmp;
+    if (nvmon_initialized == 0)
+    {
+        return 0;
+    }
+    groupId = lua_tonumber(L,1);
+    metricId = lua_tonumber(L,2);
+    tmp = nvmon_getMetricName(groupId-1, metricId-1);
+    lua_pushstring(L,tmp);
+    return 1;
+}
+
+static int
+lua_likwid_nvGetNameOfGroup(lua_State* L)
+{
+    int groupId;
+    char* tmp;
+    if (nvmon_initialized == 0)
+    {
+        return 0;
+    }
+    groupId = lua_tonumber(L,1);
+    tmp = nvmon_getGroupName(groupId-1);
+    lua_pushstring(L,tmp);
+    return 1;
+}
+
+static int
+lua_likwid_nvMarkerFile_read(lua_State* L)
+{
+    const char* filename = (const char*)luaL_checkstring(L, -1);
+    int ret = nvmon_readMarkerFile(filename);
+    lua_pushinteger(L, ret);
+    return 1;
+}
+
+static int
+lua_likwid_nvMarkerFile_destroy(lua_State* L)
+{
+    nvmon_destroyMarkerResults();
+    return 0;
+}
+
+static int
+lua_likwid_nvMarkerNumRegions(lua_State* L)
+{
+    lua_pushinteger(L, nvmon_getNumberOfRegions());
+    return 1;
+}
+
+static int
+lua_likwid_nvMarkerRegionGroup(lua_State* L)
+{
+    int region = lua_tointeger(L,-1);
+    lua_pushinteger(L, nvmon_getGroupOfRegion(region-1)+1);
+    return 1;
+}
+
+static int
+lua_likwid_nvMarkerRegionTag(lua_State* L)
+{
+    int region = lua_tointeger(L,-1);
+    lua_pushstring(L, nvmon_getTagOfRegion(region-1));
+    return 1;
+}
+
+static int
+lua_likwid_nvMarkerRegionEvents(lua_State* L)
+{
+    int region = lua_tointeger(L,-1);
+    lua_pushinteger(L, nvmon_getEventsOfRegion(region-1));
+    return 1;
+}
+
+static int
+lua_likwid_nvMarkerRegionMetrics(lua_State* L)
+{
+    int region = lua_tointeger(L,-1);
+    lua_pushinteger(L, nvmon_getMetricsOfRegion(region-1));
+    return 1;
+}
+
+
+static int
+lua_likwid_nvMarkerRegionGpus(lua_State* L)
+{
+    int region = lua_tointeger(L,-1);
+    lua_pushinteger(L, nvmon_getGpusOfRegion(region-1));
+    return 1;
+}
+
+static int
+lua_likwid_nvMarkerRegionGpulist(lua_State* L)
+{
+    int i = 0;
+    int region = lua_tointeger(L,-1);
+    int* gpulist;
+    int regionGPUs = 0;
+    if (gputopology_isInitialized == 0)
+    {
+        topology_gpu_init();
+    }
+    if ((gputopology_isInitialized) && (gputopo == NULL))
+    {
+        gputopo = get_gpuTopology();
+    }
+
+    gpulist = (int*)malloc(gputopo->numDevices * sizeof(int));
+    if (gpulist == NULL)
+    {
+        return 0;
+    }
+    regionGPUs = nvmon_getGpulistOfRegion(region-1, gputopo->numDevices, gpulist);
+    if (regionGPUs > 0)
+    {
+        lua_newtable(L);
+        for (i=0; i < regionGPUs; i++)
+        {
+            lua_pushinteger(L, i+1);
+            lua_pushinteger(L, gpulist[i]);
+            lua_settable(L, -3);
+        }
+        return 1;
+    }
+    return 0;
+}
+
+static int
+lua_likwid_nvMarkerRegionTime(lua_State* L)
+{
+    int region = lua_tointeger(L,-2);
+    int thread = lua_tointeger(L,-1);
+    lua_pushnumber(L, nvmon_getTimeOfRegion(region-1, thread-1));
+    return 1;
+}
+
+static int
+lua_likwid_nvMarkerRegionCount(lua_State* L)
+{
+    int region = lua_tointeger(L,-2);
+    int thread = lua_tointeger(L,-1);
+    lua_pushinteger(L, nvmon_getCountOfRegion(region-1, thread-1));
+    return 1;
+}
+
+static int
+lua_likwid_nvMarkerRegionResult(lua_State* L)
+{
+    int region = lua_tointeger(L,-3);
+    int event = lua_tointeger(L,-2);
+    int gpu = lua_tointeger(L,-1);
+    lua_pushnumber(L, nvmon_getResultOfRegionGpu(region-1, event-1, gpu-1));
+    return 1;
+}
+
+static int
+lua_likwid_nvMarkerRegionMetric(lua_State* L)
+{
+    int region = lua_tointeger(L,-3);
+    int metric = lua_tointeger(L,-2);
+    int gpu = lua_tointeger(L,-1);
+    lua_pushnumber(L, nvmon_getMetricOfRegionGpu(region-1, metric-1, gpu-1));
+    return 1;
+}
+
+
+static int
+lua_likwid_nvInit(lua_State* L)
+{
+    int ret;
+    int nrGpus = luaL_checknumber(L,1);
+    luaL_argcheck(L, nrGpus > 0, 1, "GPU count must be greater than 0");
+    int gpus[nrGpus];
+    if (!lua_istable(L, -1)) {
+      lua_pushstring(L,"No table given as second argument");
+      lua_error(L);
+    }
+    for (ret = 1; ret<=nrGpus; ret++)
+    {
+        lua_rawgeti(L,-1,ret);
+#if LUA_VERSION_NUM == 501
+        gpus[ret-1] = ((lua_Integer)lua_tointeger(L,-1));
+#else
+        gpus[ret-1] = ((lua_Unsigned)lua_tointegerx(L,-1, NULL));
+#endif
+        lua_pop(L,1);
+    }
+    if (gputopology_isInitialized == 0)
+    {
+        topology_gpu_init();
+    }
+    if ((gputopology_isInitialized) && (gputopo == NULL))
+    {
+        gputopo = get_gpuTopology();
+    }
+    if (nvmon_initialized == 0)
+    {
+        ret = nvmon_init(nrGpus, gpus);
+        if (ret != 0)
+        {
+            lua_pushstring(L,"Cannot initialize likwid perfmon");
+            nvmon_finalize();
+            lua_pushinteger(L,ret);
+            return 1;
+        }
+        nvmon_initialized = 1;
+        lua_pushinteger(L,ret);
+    }
+    return 1;
+}
+
+static int
+lua_likwid_nvAddEventSet(lua_State* L)
+{
+    int groupId, n;
+    const char* tmpString;
+    if (nvmon_initialized == 0)
+    {
+        return 0;
+    }
+    n = lua_gettop(L);
+    tmpString = luaL_checkstring(L, n);
+    luaL_argcheck(L, strlen(tmpString) > 0, n, "Event string must be larger than 0");
+
+    groupId = nvmon_addEventSet((char*)tmpString);
+    if (groupId >= 0)
+        lua_pushinteger(L, groupId+1);
+    else
+        lua_pushinteger(L, groupId);
+    return 1;
+}
+
+static int
+lua_likwid_nvFinalize(lua_State *L)
+{
+    if (nvmon_initialized)
+        nvmon_finalize();
+    return 0;
+}
+
+static int
+lua_likwid_nvSupported(lua_State *L)
+{
+    lua_pushboolean(L, 1);
+    return 1;
+}
+#else
+static int
+lua_likwid_nvSupported(lua_State *L)
+{
+    lua_pushboolean(L, 1);
+    return 0;
+}
+#endif /* LIKWID_WITH_NVMON */
 
 
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
@@ -2828,6 +3540,7 @@ luaopen_liblikwid(lua_State* L){
     lua_register(L, "likwid_getNumaInfo",lua_likwid_getNumaInfo);
     lua_register(L, "likwid_putNumaInfo",lua_likwid_putNumaInfo);
     lua_register(L, "likwid_setMemInterleaved", lua_likwid_setMemInterleaved);
+    lua_register(L, "likwid_setMembind", lua_likwid_setMembind);
     lua_register(L, "likwid_getAffinityInfo",lua_likwid_getAffinityInfo);
     lua_register(L, "likwid_putAffinityInfo",lua_likwid_putAffinityInfo);
     lua_register(L, "likwid_getPowerInfo",lua_likwid_getPowerInfo);
@@ -2886,6 +3599,7 @@ luaopen_liblikwid(lua_State* L){
     lua_register(L, "likwid_startRegion", lua_likwid_startRegion);
     lua_register(L, "likwid_stopRegion", lua_likwid_stopRegion);
     lua_register(L, "likwid_getRegion", lua_likwid_getRegion);
+    lua_register(L, "likwid_resetRegion", lua_likwid_resetRegion);
     // CPU feature manipulation functions
     lua_register(L, "likwid_cpuFeaturesInit", lua_likwid_cpuFeatures_init);
     lua_register(L, "likwid_cpuFeaturesGet", lua_likwid_cpuFeatures_get);
@@ -2905,10 +3619,15 @@ luaopen_liblikwid(lua_State* L){
     lua_register(L, "likwid_markerRegionResult", lua_likwid_markerRegionResult);
     lua_register(L, "likwid_markerRegionMetric", lua_likwid_markerRegionMetric);
     // CPU frequency functions
+    lua_register(L, "likwid_initFreq", lua_likwid_initFreq);
+    lua_register(L, "likwid_finalizeFreq", lua_likwid_finalizeFreq);
+    lua_register(L, "likwid_getCpuClockBase", lua_likwid_getCpuClockBase);
     lua_register(L, "likwid_getCpuClockCurrent", lua_likwid_getCpuClockCurrent);
     lua_register(L, "likwid_getCpuClockMin", lua_likwid_getCpuClockMin);
+    lua_register(L, "likwid_getConfCpuClockMin", lua_likwid_getConfCpuClockMin);
     lua_register(L, "likwid_setCpuClockMin", lua_likwid_setCpuClockMin);
     lua_register(L, "likwid_getCpuClockMax", lua_likwid_getCpuClockMax);
+    lua_register(L, "likwid_getConfCpuClockMax", lua_likwid_getConfCpuClockMax);
     lua_register(L, "likwid_setCpuClockMax", lua_likwid_setCpuClockMax);
     lua_register(L, "likwid_getGovernor", lua_likwid_getGovernor);
     lua_register(L, "likwid_setGovernor", lua_likwid_setGovernor);
@@ -2928,10 +3647,38 @@ luaopen_liblikwid(lua_State* L){
     lua_register(L, "likwid_seteuid", lua_likwid_seteuid);
     lua_register(L, "likwid_setresuid", lua_likwid_setresuid);
     lua_register(L, "likwid_setresuser", lua_likwid_setresuser);
+    // Nvidia GPU functions
+    lua_register(L, "likwid_nvSupported", lua_likwid_nvSupported);
+#ifdef LIKWID_WITH_NVMON
+    lua_register(L, "likwid_getGpuTopology", lua_likwid_getGpuTopology);
+    lua_register(L, "likwid_putGpuTopology", lua_likwid_putGpuTopology);
+    lua_register(L, "likwid_getGpuEventsAndCounters", lua_likwid_getGpuEventsAndCounters);
+    lua_register(L, "likwid_getGpuGroups", lua_likwid_getGpuGroups);
+    lua_register(L, "likwid_gpustr_to_gpulist", lua_likwid_gpustr_to_gpulist);
+    lua_register(L, "likwid_readNvMarkerFile", lua_likwid_nvMarkerFile_read);
+    lua_register(L, "likwid_destroyNvMarkerFile", lua_likwid_nvMarkerFile_destroy);
+    lua_register(L, "likwid_nvMarkerNumRegions", lua_likwid_nvMarkerNumRegions);
+    lua_register(L, "likwid_nvMarkerRegionGroup", lua_likwid_nvMarkerRegionGroup);
+    lua_register(L, "likwid_nvMarkerRegionTag", lua_likwid_nvMarkerRegionTag);
+    lua_register(L, "likwid_nvMarkerRegionEvents", lua_likwid_nvMarkerRegionEvents);
+    lua_register(L, "likwid_nvMarkerRegionMetrics", lua_likwid_nvMarkerRegionMetrics);
+    lua_register(L, "likwid_nvMarkerRegionGpus", lua_likwid_nvMarkerRegionGpus);
+    lua_register(L, "likwid_nvMarkerRegionGpulist", lua_likwid_nvMarkerRegionGpulist);
+    lua_register(L, "likwid_nvMarkerRegionTime", lua_likwid_nvMarkerRegionTime);
+    lua_register(L, "likwid_nvMarkerRegionCount", lua_likwid_nvMarkerRegionCount);
+    lua_register(L, "likwid_nvMarkerRegionResult", lua_likwid_nvMarkerRegionResult);
+    lua_register(L, "likwid_nvMarkerRegionMetric", lua_likwid_nvMarkerRegionMetric);
+    lua_register(L, "likwid_nvInit",lua_likwid_nvInit);
+    lua_register(L, "likwid_nvAddEventSet", lua_likwid_nvAddEventSet);
+    lua_register(L, "likwid_nvFinalize",lua_likwid_nvFinalize);
+    lua_register(L, "likwid_nvGetNameOfEvent",lua_likwid_nvGetNameOfEvent);
+    lua_register(L, "likwid_nvGetNameOfCounter",lua_likwid_nvGetNameOfCounter);
+    lua_register(L, "likwid_nvGetNameOfMetric",lua_likwid_nvGetNameOfMetric);
+    lua_register(L, "likwid_nvGetNameOfGroup",lua_likwid_nvGetNameOfGroup);
+#endif /* LIKWID_WITH_NVMON */
 #ifdef __MIC__
     setuid(0);
     seteuid(0);
-#endif
+#endif /* __MIC__ */
     return 0;
 }
-
