@@ -6,14 +6,14 @@
  *      Description:  Implementation of Linux NUMA interface. Selects between hwloc and
  *                    procfs/sysfs backends.
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
- *                Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -96,11 +96,92 @@ str2int(const char* str)
     return (int) val;
 }
 
+uint64_t
+proc_getFreeSysMem(void)
+{
+    FILE *fp;
+    uint64_t free = 0;
+    bstring freeString  = bformat("MemFree:");
+    int i;
+
+    if (!access("/proc/meminfo", R_OK))
+    {
+        if (NULL != (fp = fopen ("/proc/meminfo", "r")))
+        {
+            bstring src = bread ((bNread) fread, fp);
+            struct bstrList* tokens = bsplit(src,(char) '\n');
+            for (i=0;i<tokens->qty;i++)
+            {
+                if (binstr(tokens->entry[i],0,freeString) != BSTR_ERR)
+                {
+                     bstring tmp = bmidstr (tokens->entry[i], 10, blength(tokens->entry[i])-10  );
+                     bltrimws(tmp);
+                     struct bstrList* subtokens = bsplit(tmp,(char) ' ');
+                     free = str2int(bdata(subtokens->entry[0]));
+                     bdestroy(tmp);
+                     bstrListDestroy(subtokens);
+                }
+            }
+            bstrListDestroy(tokens);
+            bdestroy(src);
+            fclose(fp);
+        }
+    }
+    else
+    {
+        bdestroy(freeString);
+        ERROR;
+    }
+    bdestroy(freeString);
+    return free;
+}
+
+uint64_t
+proc_getTotalSysMem(void)
+{
+    int i;
+    FILE *fp;
+    uint64_t total = 0;
+    bstring totalString  = bformat("MemTotal:");
+
+    if (!access("/proc/meminfo", R_OK))
+    {
+        if (NULL != (fp = fopen ("/proc/meminfo", "r")))
+        {
+            bstring src = bread ((bNread) fread, fp);
+            struct bstrList* tokens = bsplit(src,(char) '\n');
+            for (i=0;i<tokens->qty;i++)
+            {
+                if (binstr(tokens->entry[i],0,totalString) != BSTR_ERR)
+                {
+                     bstring tmp = bmidstr (tokens->entry[i], 10, blength(tokens->entry[i])-10  );
+                     bltrimws(tmp);
+                     struct bstrList* subtokens = bsplit(tmp,(char) ' ');
+                     total = str2int(bdata(subtokens->entry[0]));
+                     bdestroy(tmp);
+                     bstrListDestroy(subtokens);
+                }
+            }
+            bstrListDestroy(tokens);
+            bdestroy(src);
+            fclose(fp);
+        }
+    }
+    else
+    {
+        bdestroy(totalString);
+        ERROR;
+    }
+
+    bdestroy(totalString);
+    return total;
+}
+
 int
 empty_numa_init()
 {
     printf("MEMPOLICY NOT supported in kernel!\n");
-    return 0;
+    return virtual_numa_init();
 }
 
 void
@@ -112,6 +193,15 @@ empty_numa_setInterleaved(const int* processorList, int numberOfProcessors)
     return;
 }
 
+void
+empty_numa_setMembind(const int* processorList, int numberOfProcessors)
+{
+    printf("MEMPOLICY NOT supported in kernel!\n");
+    if (!processorList || numberOfProcessors < 0)
+        printf("Invalid options\n");
+    return;
+}
+
 void
 empty_numa_membind(void* ptr, size_t size, int domainId)
 {
@@ -125,15 +215,21 @@ const struct numa_functions numa_funcs = {
 #ifndef HAS_MEMPOLICY
     .numa_init = empty_numa_init,
     .numa_setInterleaved = empty_numa_setInterleaved,
-    .numa_membind = empty_numa_membind
+    .numa_membind = empty_numa_membind,
+    .numa_setMembind = empty_numa_setMembind,
 #else
 #ifdef LIKWID_USE_HWLOC
+#if defined(__ARM_ARCH_7A__) || defined(__ARM_ARCH_8A__) || defined(__ARM_ARCH_8A) || defined(_ARCH_PPC64)
+    .numa_init = proc_numa_init,
+#else
     .numa_init = hwloc_numa_init,
+#endif
 #else
     .numa_init = proc_numa_init,
 #endif
     .numa_setInterleaved = proc_numa_setInterleaved,
-    .numa_membind = proc_numa_membind
+    .numa_membind = proc_numa_membind,
+    .numa_setMembind = proc_numa_setMembind,
 #endif
 };
 
@@ -159,10 +255,11 @@ numa_init(void)
     }
     else
     {
-        cpu_set_t cpuSet;
-        CPU_ZERO(&cpuSet);
-        sched_getaffinity(0,sizeof(cpu_set_t), &cpuSet);
-        if (cpuid_topology.activeHWThreads < cpuid_topology.numHWThreads)
+/*        cpu_set_t cpuSet;*/
+/*        CPU_ZERO(&cpuSet);*/
+/*        sched_getaffinity(0,sizeof(cpu_set_t), &cpuSet);*/
+        if (cpuid_topology.activeHWThreads < cpuid_topology.numHWThreads &&
+            getenv("HWLOC_FSROOT") == NULL)
         {
             ret = proc_numa_init();
         }
@@ -192,6 +289,14 @@ numa_membind(void* ptr, size_t size, int domainId)
     return;
 }
 
+void
+numa_setMembind(const int* processorList, int numberOfProcessors)
+{
+    const struct numa_functions funcs = numa_funcs;
+    funcs.numa_setMembind(processorList, numberOfProcessors);
+    return;
+}
+
 #ifndef HAS_MEMPOLICY
 void
 numa_finalize(void)
