@@ -5,14 +5,14 @@
  *
  *      Description:  Module implementing Intel RAPL interface
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
- *                Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -56,15 +56,25 @@ power_init(int cpuId)
     uint64_t flags;
     int i;
     int err;
+    int core_limits = 0;
     uint32_t unit_reg = MSR_RAPL_POWER_UNIT;
     int numDomains = NUM_POWER_DOMAINS;
+    Configuration_t config;
 
     /* determine Turbo Mode features */
     double busSpeed;
-    if (power_initialized || !lock_check())
+    if (power_initialized)
     {
         return 0;
     }
+    if (!lock_check())
+    {
+        ERROR_PLAIN_PRINT(Access to performance monitoring registers locked);
+        return -ENOLCK;
+    }
+    init_configuration();
+    config = get_configuration();
+
 
     power_info.baseFrequency = 0;
     power_info.minFrequency = 0;
@@ -76,6 +86,11 @@ power_init(int cpuId)
     power_info.uncoreMinFreq = 0;
     power_info.uncoreMaxFreq = 0;
     power_info.perfBias = 0;
+    if (config->daemonMode == ACCESSMODE_PERF)
+    {
+        ERROR_PRINT(RAPL in access mode 'perf_event' only available with perfmon);
+        return 0;
+    }
 
     switch (cpuid_info.family)
     {
@@ -107,9 +122,23 @@ power_init(int cpuId)
                     break;
                 case SKYLAKE1:
                 case SKYLAKE2:
-                case SKYLAKEX:
                 case KABYLAKE1:
                 case KABYLAKE2:
+                case CANNONLAKE:
+                case COMETLAKE1:
+                case COMETLAKE2:
+                case TIGERLAKE1:
+                case TIGERLAKE2:
+                case ICELAKE1:
+                case ICELAKE2:
+                case ROCKETLAKE:
+                    power_info.hasRAPL = 1;
+                    numDomains = NUM_POWER_DOMAINS;
+                    break;
+                case SKYLAKEX:
+                case ICELAKEX1:
+                case ICELAKEX2:
+                    core_limits = 1;
                     power_info.hasRAPL = 1;
                     numDomains = NUM_POWER_DOMAINS;
                     break;
@@ -128,7 +157,16 @@ power_init(int cpuId)
             }
             break;
         case ZEN_FAMILY:
-            if (cpuid_info.model == ZEN_RYZEN)
+        case ZEN3_FAMILY:
+            if (cpuid_info.model == ZEN_RYZEN ||
+                cpuid_info.model == ZENPLUS_RYZEN ||
+                cpuid_info.model == ZENPLUS_RYZEN2 ||
+                cpuid_info.model == ZEN2_RYZEN ||
+                cpuid_info.model == ZEN2_RYZEN2 ||
+                cpuid_info.model == ZEN2_RYZEN3 ||
+                cpuid_info.model == ZEN3_RYZEN ||
+                cpuid_info.model == ZEN3_RYZEN2 ||
+                cpuid_info.model == ZEN3_RYZEN3)
             {
                 cpuid_info.turbo = 0;
                 power_info.hasRAPL = 1;
@@ -147,6 +185,7 @@ power_init(int cpuId)
                     info_regs[i] = 0x0;
                 }
             }
+            break;
     }
 
     perfmon_init_maps();
@@ -197,7 +236,7 @@ power_init(int cpuId)
             }
             else
             {
-                if (cpuid_info.model != SKYLAKEX)
+                if (!core_limits)
                 {
                     unsigned long flag_vals[4];
                     int flag_idx = 0;
@@ -308,6 +347,8 @@ power_init(int cpuId)
                 (cpuid_info.model == BROADWELL_D) ||
                 (cpuid_info.model == BROADWELL_E) ||
                 (cpuid_info.model == SKYLAKEX) ||
+                (cpuid_info.model == ICELAKEX1) ||
+                (cpuid_info.model == ICELAKEX2) ||
                 (cpuid_info.model == XEON_PHI_KNL) ||
                 (cpuid_info.model == XEON_PHI_KML))
             {
@@ -400,8 +441,8 @@ power_init(int cpuId)
             err = HPMread(cpuId, MSR_DEV, MSR_UNCORE_FREQ, &flags);
             if (err == 0)
             {
-                power_info.uncoreMinFreq = ((double)((flags >> 8) & 0xFFULL)) * busSpeed;
-                power_info.uncoreMaxFreq = ((double)(flags & 0xFF)) * busSpeed;
+                power_info.uncoreMinFreq = ((double)((flags >> 8) & 0x3FULL)) * busSpeed;
+                power_info.uncoreMaxFreq = ((double)(flags & 0x3F)) * busSpeed;
             }
             err = HPMread(cpuId, MSR_DEV, MSR_ENERGY_PERF_BIAS, &flags);
             if (err == 0)
@@ -643,6 +684,7 @@ power_finalize(void)
     power_info.uncoreMinFreq = 0;
     power_info.uncoreMaxFreq = 0;
     memset(power_info.domains, 0, NUM_POWER_DOMAINS*sizeof(PowerDomain));
+    power_initialized = 0;
 }
 
 PowerInfo_t
@@ -650,4 +692,3 @@ get_powerInfo(void)
 {
     return &power_info;
 }
-
