@@ -5,13 +5,13 @@
  *
  *      Description:  Implementation of timer module
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -39,7 +39,9 @@
 #include <types.h>
 #include <error.h>
 #include <likwid.h>
+#if !defined(__ARM_ARCH_7A__) && !defined(__ARM_ARCH_8A)
 #include <cpuid.h>
+#endif
 
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
@@ -47,6 +49,7 @@ static uint64_t baseline = 0ULL;
 static uint64_t cpuClock = 0ULL;
 static uint64_t cyclesClock = 0ULL;
 static uint64_t sleepbase = 0ULL;
+static uint8_t fixedFreq = 0;
 static int timer_initialized = 0;
 
 void (*TSTART)(TscCounter*) = NULL;
@@ -136,6 +139,23 @@ fRDTSCP(TscCounter* cpu_c)
 #endif
 #endif
 
+#if defined(_ARCH_PPC)
+static void
+TIMER(TscCounter* cpu_c)
+{
+    uint32_t tbl, tbu0, tbu1;
+
+    do {
+        __asm__ __volatile__ ("mftbu %0" : "=r"(tbu0));
+        __asm__ __volatile__ ("mftb %0" : "=r"(tbl));
+        __asm__ __volatile__ ("mftbu %0" : "=r"(tbu1));
+    } while (tbu0 != tbu1);
+
+    (cpu_c)->int64 = (((uint64_t)tbu0) << 32) | tbl;
+}
+#endif
+
+
 static int os_timer(TscCounter* time)
 {
     int ret;
@@ -163,7 +183,7 @@ static void os_timer_stop(TscCounter* time)
 static void
 _timer_start( TimerData* time )
 {
-#if defined(__x86_64) || defined(__i386__)
+#if defined(__x86_64) || defined(__i386__) || defined(__ARM_ARCH_7A__) || defined(__ARM_ARCH_8A)
     if (TSTART)
         TSTART(&(time->start));
 #endif
@@ -183,7 +203,7 @@ _timer_start( TimerData* time )
 static void
 _timer_stop( TimerData* time )
 {
-#if defined(__x86_64) || defined(__i386__)
+#if defined(__x86_64) || defined(__i386__) || defined(__ARM_ARCH_7A__) || defined(__ARM_ARCH_8A)
     if (TSTOP)
         TSTOP(&(time->stop));
 #endif
@@ -213,6 +233,12 @@ _timer_printCycles( const TimerData* time )
     {
         cycles = (time->stop.int64 - time->start.int64 - baseline);
     }
+#if defined(__ARM_ARCH_7A__) || defined(__ARM_ARCH_8A)
+    if (fixedFreq == 1)
+    {
+        cycles *= 1E-6 * cpuClock;
+    }
+#endif
     return cycles;
 }
 
@@ -222,16 +248,11 @@ _timer_print( const TimerData* time )
 {
     uint64_t cycles = 0x0ULL;
     cycles = _timer_printCycles(time);
-    if (((time->stop.int64-baseline) < time->start.int64) ||
-        (time->start.int64 == time->stop.int64))
-    {
-        cycles = 0ULL;
-    }
-    else
-    {
-        cycles = time->stop.int64 - time->start.int64 - baseline;
-    }
+#if defined(__ARM_ARCH_7A__) || defined(__ARM_ARCH_8A)
+    return ((double) cycles) * 1E-6;
+#else
     return  ((double) cycles / (double) cyclesClock);
+#endif
 }
 
 static void
@@ -289,6 +310,7 @@ getCpuSpeed(void)
     }
 
     fgets(buff, 256, fpipe);
+    pclose(fpipe);
 
     cyclesClock = (uint64_t)   atoi(buff);
     if ( !(fpipe = (FILE*)popen(command2,"r")) )
@@ -303,39 +325,84 @@ getCpuSpeed(void)
     cpuClock *= 1E6;
     pclose(fpipe);
 #endif
+#if defined(__ARM_ARCH_7A__) || defined(__ARM_ARCH_8A)
+    uint64_t result = 0xFFFFFFFFFFFFFFFFULL;
+    TimerData data;
+    int i = 0;
+    struct timeval tv1;
+    struct timeval tv2;
+    struct timezone tzp;
+    struct timespec delay = { 1, 0 }; /* calibration time: 500 ms */
+/*    FILE *fpipe;*/
+/*    char *command="cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_cur_freq";*/
+/*    char *command2="cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor";*/
+/*    char buff[256];*/
+/*    buff[0] = '\0';*/
+/*    char* buffptr = NULL;*/
+/*    if ( !(fpipe = (FILE*)popen(command2, "r")))*/
+/*    {*/
+/*        perror("Problems with pipe, cannot read /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor");*/
+/*        exit(1);*/
+/*    }*/
+/*    buffptr = fgets(buff, 256, fpipe);*/
+/*    fclose(fpipe);*/
+/*    if ((strncmp(buff, "userspace", 9) == 0) || (strncmp(buff, "performance", 11) == 0))*/
+/*    {*/
+/*        fixedFreq = 1;*/
+/*    }*/
+/*    buff[0] = '\0';*/
+/*    buffptr = NULL;*/
+
+/*    if ( !(fpipe = (FILE*)popen(command, "r")))*/
+/*    {*/
+/*        perror("Problems with pipe, cannot read /sys/devices/system/cpu/cpu0/cpufreq/scaling_cur_freq");*/
+/*        exit(1);*/
+/*    }*/
+/*    buffptr = fgets(buff, 256, fpipe);*/
+/*    fclose(fpipe);*/
+    for (i=0;i<10;i++)
+    {
+        _timer_start(&data);
+        _timer_stop(&data);
+        result = MIN(result,_timer_printCycles(&data));
+    }
+    baseline = result;
+    result = 0xFFFFFFFFFFFFFFFFULL;
+    for (i=0; i< 2; i++)
+    {
+        _timer_start(&data);
+        gettimeofday( &tv1, &tzp);
+        gettimeofday( &tv2, &tzp);
+        double t = 0;
+        while (tv2.tv_sec == tv1.tv_sec)
+        {
+            for (int j = 0; j < 10 ; j++)
+                t += 1.0;
+            gettimeofday( &tv2, &tzp);
+        }
+        if (t == 0)
+            continue;
+        _timer_stop(&data);
+
+        result = MIN(result,(data.stop.int64 - data.start.int64));
+    }
+    cpuClock = (result) * 1000000 /
+        (((uint64_t)tv2.tv_sec * 1000000 + tv2.tv_usec) -
+         ((uint64_t)tv1.tv_sec * 1000000 + tv1.tv_usec));
+#endif
+
 }
 
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
-void
-init_sleep()
-{
-    int status;
-    TimerData timer;
-    struct timespec req = {0,1};
-    struct timespec rem = {0,0};
-    for (int i=0; i<10; ++i)
-    {
-        _timer_start(&timer);
-        status = clock_nanosleep(CLOCK_REALTIME,0,&req, &rem);
-        _timer_stop(&timer);
-        if (_timer_print(&timer)*1E6 > sleepbase)
-        {
-            sleepbase = _timer_print(&timer)*1E6 + 2;
-        }
-    }
-}
 
 void
-timer_init( void )
+_timer_init( void )
 {
     uint32_t eax = 0x0,ebx = 0x0,ecx = 0x0,edx = 0x0;
-    if (timer_initialized == 1)
-    {
-        return;
-    }
     if ((!TSTART) && (!TSTOP))
     {
+#if defined(__x86_64) || defined(__i386__)
         TSTART = fRDTSC;
         eax = 0x80000001;
         CPUID (eax, ebx, ecx, edx);
@@ -350,15 +417,54 @@ timer_init( void )
         }
 #else
         TSTOP = fRDTSC_CR;
+#endif
+#endif
+#if defined(__ARM_ARCH_7A__) || defined(__ARM_ARCH_8A)
+        TSTART = os_timer_start;
+        TSTOP = os_timer_stop;
+#endif
+#ifdef _ARCH_PPC
+        TSTART = TIMER;
+        TSTOP = TIMER;
 #endif
     }
     if (cpuClock == 0ULL)
     {
         getCpuSpeed();
     }
+}
+
+void
+init_sleep()
+{
+    int status;
+    TimerData timer;
+    _timer_init();
+    struct timespec req = {0,1};
+    struct timespec rem = {0,0};
+    for (int i=0; i<10; ++i)
+    {
+        _timer_start(&timer);
+        status = clock_nanosleep(CLOCK_REALTIME,0,&req, &rem);
+        _timer_stop(&timer);
+        if (_timer_print(&timer)*1E6 > sleepbase)
+        {
+            sleepbase = _timer_print(&timer)*1E6 + 2;
+        }
+    }
+}
+void timer_init(void)
+{
+    if (timer_initialized == 1)
+    {
+        return;
+    }
+    _timer_init();
+    init_sleep();
     timer_initialized = 1;
 }
 
+
 uint64_t
 timer_printCycles( const TimerData* time )
 {
@@ -418,6 +524,7 @@ timer_getCpuClockCurrent( int cpu_id )
     }
 
     rptr = fgets(buff, 256, fpipe);
+    pclose(fpipe);
     if (rptr != NULL)
     {
         clock = strtoull(buff, &eptr, 10);
@@ -478,6 +585,7 @@ timer_sleep(unsigned long usec)
 
     if (sleepbase == 0x0ULL)
     {
+        //fprintf(stderr, "Sleeping longer as likwid_sleep() called without prior initialization\n");
         init_sleep();
     }
     if (usec >= 1000000)
@@ -518,4 +626,3 @@ timer_reset( TimerData* time )
     time->start.int64 = 0;
     time->stop.int64 = 0;
 }
-
