@@ -5,14 +5,14 @@
  *
  *      Description:  Interface to the topology backends
  *
- *      Version:   4.3.4
- *      Released:  05.04.2019
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
- *                Thomas Roehl (tr), thomas.roehl@googlemail.com
+ *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -45,6 +45,7 @@
 #include <bitUtil.h>
 //#include <strUtil.h>
 #include <configuration.h>
+#include <topology_static.h>
 
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
@@ -81,12 +82,19 @@ static char* cascadelakeX_str = "Intel Cascadelake SP processor";
 static char* kabylake_str = "Intel Kabylake processor";
 static char* cannonlake_str = "Intel Cannonlake processor";
 static char* coffeelake_str = "Intel Coffeelake processor";
+static char* cometlake_str = "Intel Cometlake processor";
 static char* nehalem_ex_str = "Intel Nehalem EX processor";
 static char* westmere_ex_str = "Intel Westmere EX processor";
 static char* xeon_mp_string = "Intel Xeon MP processor";
 static char* xeon_phi_string = "Intel Xeon Phi (Knights Corner) Coprocessor";
 static char* xeon_phi2_string = "Intel Xeon Phi (Knights Landing) (Co)Processor";
 static char* xeon_phi3_string = "Intel Xeon Phi (Knights Mill) (Co)Processor";
+static char* icelake_str = "Intel Icelake processor";
+static char* tigerlake_str = "Intel Tigerlake processor";
+static char* icelakesp_str = "Intel Icelake SP processor";
+static char* rocketlake_str = "Intel Rocketlake processor";
+//static char* snowridgex_str = "Intel SnowridgeX processor";
+
 static char* barcelona_str = "AMD K10 (Barcelona) processor";
 static char* shanghai_str = "AMD K10 (Shanghai) processor";
 static char* istanbul_str = "AMD K10 (Istanbul) processor";
@@ -103,8 +111,26 @@ static char* athlon64_X2_g_str = "AMD Athlon64 X2 (AM2) Rev G 65nm processor";
 static char* athlon64_g_str = "AMD Athlon64 (AM2) Rev G 65nm processor";
 static char* amd_k8_str = "AMD K8 architecture";
 static char* amd_zen_str = "AMD K17 (Zen) architecture";
+static char* amd_zenplus_str = "AMD K17 (Zen+) architecture";
+static char* amd_zen2_str = "AMD K17 (Zen2) architecture";
+static char* amd_zen3_str = "AMD K19 (Zen3) architecture";
+static char* armv7l_str = "ARM 7l architecture";
+static char* armv8_str = "ARM 8 architecture";
+static char* cavium_thunderx2t99_str = "Cavium Thunder X2 (ARMv8)";
+static char* cavium_thunderx_str = "Cavium Thunder X (ARMv8)";
+static char* arm_cortex_a57 = "ARM Cortex A57";
+static char* arm_cortex_a53 = "ARM Cortex A53";
+static char* arm_cortex_a72 = "ARM Cortex A72";
+static char* arm_cortex_a73 = "ARM Cortex A73";
+static char* arm_neoverse_n1 = "ARM Neoverse N1";
+static char* fujitsu_a64fx = "Fujitsu A64FX";
+static char* power7_str = "POWER7 architecture";
+static char* power8_str = "POWER8 architecture";
+static char* power9_str = "POWER9 architecture";
+
 static char* unknown_intel_str = "Unknown Intel Processor";
 static char* unknown_amd_str = "Unknown AMD Processor";
+static char* unknown_power_str = "Unknown POWER Processor";
 
 static char* short_core2 = "core2";
 static char* short_atom = "atom";
@@ -128,15 +154,35 @@ static char* short_sandybridge_ep = "sandybridgeEP";
 static char* short_skylake = "skylake";
 static char* short_skylakeX = "skylakeX";
 static char* short_kabylake = "skylake";
-static char* short_cascadelakeX = "skylakeX";
+static char* short_cascadelakeX = "CLX";
 static char* short_cannonlake = "cannonlake";
+static char* short_tigerlake = "TGL";
 static char* short_phi = "phi";
 static char* short_phi2 = "knl";
+static char* short_icelake = "ICL";
+static char* short_rocketlake = "RKL";
+static char* short_icelakesp = "ICX";
+//static char* short_snowridgex = "SNR";
+
 static char* short_k8 = "k8";
 static char* short_k10 = "k10";
 static char* short_k15 = "interlagos";
 static char* short_k16 = "kabini";
 static char* short_zen = "zen";
+static char* short_zen2 = "zen2";
+static char* short_zen3 = "zen3";
+
+static char* short_arm7 = "arm7";
+static char* short_arm8 = "arm8";
+static char* short_arm8_cav_tx2 = "arm8_tx2";
+static char* short_arm8_cav_tx = "arm8_tx";
+static char* short_arm8_neo_n1 = "arm8_n1";
+static char* short_a64fx = "arm64fx";
+
+static char* short_power7 = "power7";
+static char* short_power8 = "power8";
+static char* short_power9 = "power9";
+
 static char* short_unknown = "unknown";
 
 /* #####  EXPORTED VARIABLES  ########################################## */
@@ -305,6 +351,10 @@ readTopologyFile(const char* filename, cpu_set_t cpuSet)
                 {
                     cpuid_topology.threadPool[thread].packageId = tmp;
                 }
+                else if (strcmp(value, "dieId") == 0)
+                {
+                    cpuid_topology.threadPool[thread].dieId = tmp;
+                }
                 else if (strcmp(value, "apicId") == 0)
                 {
                     cpuid_topology.threadPool[thread].apicId = tmp;
@@ -400,9 +450,10 @@ readTopologyFile(const char* filename, cpu_set_t cpuSet)
             }
             else if (strcmp(field, "osname") == 0)
             {
-                strcpy(value,&(line[strlen(structure)+strlen(field)+4]));
-                cpuid_info.osname = (char*) malloc((strlen(value)+1) * sizeof(char));
-                strncpy(cpuid_info.osname, value, strlen(value));
+                strncpy(value,&(line[strlen(structure)+strlen(field)+4]), 256);
+                int len = 257;
+                cpuid_info.osname = (char*) malloc(len * sizeof(char));
+                strncpy(cpuid_info.osname, value, len);
                 cpuid_info.osname[strlen(value)-1] = '\0';
             }
             else if (strcmp(field, "stepping") == 0)
@@ -462,8 +513,9 @@ readTopologyFile(const char* filename, cpu_set_t cpuSet)
             else if (strcmp(field, "features") == 0)
             {
                 strcpy(value,&(line[strlen(structure)+strlen(field)+4]));
-                cpuid_info.features = (char*) malloc((strlen(value)+1) * sizeof(char));
-                strncpy(cpuid_info.features, value, strlen(value));
+                int len = 257;
+                cpuid_info.features = (char*) malloc(len * sizeof(char));
+                strncpy(cpuid_info.features, value, len);
                 cpuid_info.features[strlen(value)-1] = '\0';
             }
         }
@@ -547,6 +599,97 @@ cpu_count(cpu_set_t* set)
     return s;
 }
 
+int
+likwid_cpu_online(int cpu_id)
+{
+    if (cpu_id < 0)
+        return 0;
+    int state = 0;
+    bstring bpath = bformat("/sys/devices/system/cpu/cpu%d/online", cpu_id);
+    FILE* fp = fopen(bdata(bpath), "r");
+    if (fp)
+    {
+        char buf[10];
+        int ret = fread(buf, sizeof(char), 9, fp);
+        fclose(fp);
+        if (ret > 0)
+        {
+            state = atoi(buf);
+        }
+    }
+    else
+    {
+        fp = fopen("/sys/devices/system/cpu/online", "r");
+        if (fp)
+        {
+            char buf[100];
+            int ret = fread(buf, sizeof(char), 99, fp);
+            fclose(fp);
+            if (ret > 0)
+            {
+                bdestroy(bpath);
+                buf[ret] = '\0';
+                bpath = bfromcstr(buf);
+                struct bstrList* first = bsplit(bpath, ',');
+                for (int i = 0; i < first->qty; i++)
+                {
+                    struct bstrList* second = bsplit(first->entry[i], '-');
+                    if (second->qty == 1)
+                    {
+                        int core = atoi(bdata(second->entry[0]));
+                        if (core == cpu_id)
+                            state = 1;
+                    }
+                    else if (second->qty == 2)
+                    {
+                        int s = atoi(bdata(second->entry[0]));
+                        int e = atoi(bdata(second->entry[1]));
+                        if (cpu_id >= s && cpu_id <= e)
+                            state = 1;
+                    }
+                    bstrListDestroy(second);
+                }
+                bstrListDestroy(first);
+
+            }
+        }
+    }
+    bdestroy(bpath);
+    return state;
+}
+
+int likwid_sysfs_list_len(char* sysfsfile)
+{
+    int i = 0;
+    int len = 0;
+    FILE* fp;
+    if (NULL != (fp = fopen (sysfsfile, "r")))
+    {
+        bstring src = bread ((bNread) fread, fp);
+        struct bstrList* first = bsplit(src, ',');
+        for (i = 0; i < first->qty; i++)
+        {
+            struct bstrList* second = bsplit(first->entry[i], '-');
+            if (second->qty == 1)
+            {
+                len++;
+            }
+            else
+            {
+                int s = atoi(bdata(second->entry[0]));
+                int e = atoi(bdata(second->entry[1]));
+                len += e - s + 1;
+            }
+            bstrListDestroy(second);
+        }
+        bstrListDestroy(first);
+        bdestroy(src);
+        fclose(fp);
+    }
+    return len;
+}
+
+
 
 int
 topology_setName(void)
@@ -599,6 +742,7 @@ topology_setName(void)
                     break;
 
                 case SANDYBRIDGE:
+                    cpuid_info.supportClientmem = 1;
                     cpuid_info.name = sandybridge_str;
                     cpuid_info.short_name = short_sandybridge;
                     break;
@@ -610,6 +754,7 @@ topology_setName(void)
                     break;
 
                 case IVYBRIDGE:
+                    cpuid_info.supportClientmem = 1;
                     cpuid_info.name = ivybridge_str;
                     cpuid_info.short_name = short_ivybridge;
                     break;
@@ -628,11 +773,13 @@ topology_setName(void)
                 case HASWELL:
                 case HASWELL_M1:
                 case HASWELL_M2:
+                    cpuid_info.supportClientmem = 1;
                     cpuid_info.name = haswell_str;
                     cpuid_info.short_name = short_haswell;
                     break;
 
                 case BROADWELL:
+                    cpuid_info.supportClientmem = 1;
                     cpuid_info.name = broadwell_str;
                     cpuid_info.short_name = short_broadwell;
                     break;
@@ -653,6 +800,7 @@ topology_setName(void)
 
                 case SKYLAKE1:
                 case SKYLAKE2:
+                    cpuid_info.supportClientmem = 1;
                     cpuid_info.name = skylake_str;
                     cpuid_info.short_name = short_skylake;
                     break;
@@ -668,23 +816,34 @@ topology_setName(void)
                         cpuid_info.name = cascadelakeX_str;
                         cpuid_info.short_name = short_cascadelakeX;
                     }
+                    cpuid_info.supportUncore = 1;
                     break;
 
                 case KABYLAKE1:
+                    cpuid_info.supportClientmem = 1;
                     cpuid_info.name = kabylake_str;
                     cpuid_info.short_name = short_skylake;
                     break;
 
                 case KABYLAKE2:
+                    cpuid_info.supportClientmem = 1;
                     cpuid_info.name = coffeelake_str;
                     cpuid_info.short_name = short_skylake;
                     break;
 
                 case CANNONLAKE:
+                    cpuid_info.supportClientmem = 1;
                     cpuid_info.name = cannonlake_str;
                     cpuid_info.short_name = short_cannonlake;
                     break;
 
+                case COMETLAKE1:
+                case COMETLAKE2:
+                    cpuid_info.supportClientmem = 1;
+                    cpuid_info.name = cometlake_str;
+                    cpuid_info.short_name = short_skylake;
+                    break;
+
                 case XEON_PHI_KNL:
                     cpuid_info.supportUncore = 1;
                     cpuid_info.name = xeon_phi2_string;
@@ -755,6 +914,38 @@ topology_setName(void)
                     cpuid_info.short_name = short_goldmontplus;
                     break;
 
+                case ICELAKE1:
+                case ICELAKE2:
+                    cpuid_info.supportClientmem = 1;
+                    cpuid_info.name = icelake_str;
+                    cpuid_info.short_name = short_icelake;
+                    break;
+
+                case ROCKETLAKE:
+                    cpuid_info.supportClientmem = 1;
+                    cpuid_info.name = rocketlake_str;
+                    cpuid_info.short_name = short_rocketlake;
+                    break;
+
+                case ICELAKEX1:
+                case ICELAKEX2:
+                    cpuid_info.supportUncore = 1;
+                    cpuid_info.name = icelakesp_str;
+                    cpuid_info.short_name = short_icelakesp;
+                    break;
+
+/*                case SNOWRIDGEX:*/
+/*                    cpuid_info.name = snowridgex_str;*/
+/*                    cpuid_info.short_name = short_snowridgex;*/
+/*                    break;*/
+
+                case TIGERLAKE1:
+                case TIGERLAKE2:
+                    //cpuid_info.supportClientmem = 1;
+                    cpuid_info.name = tigerlake_str;
+                    cpuid_info.short_name = short_tigerlake;
+                    break;
+
                 default:
                     cpuid_info.name = unknown_intel_str;
                     cpuid_info.short_name = short_unknown;
@@ -863,13 +1054,164 @@ topology_setName(void)
             cpuid_info.short_name = short_k16;
             break;
 
+        case PPC_FAMILY:
+            switch(cpuid_info.model)
+            {
+                case POWER7:
+                    cpuid_info.name = power7_str;
+                    cpuid_info.short_name = short_power7;
+                    break;
+                case POWER8:
+                    cpuid_info.name = power8_str;
+                    cpuid_info.short_name = short_power8;
+                    break;
+                case POWER9:
+                    cpuid_info.name = power9_str;
+                    cpuid_info.short_name = short_power9;
+                    break;
+                default:
+                    cpuid_info.name = unknown_power_str;
+                    cpuid_info.short_name = short_unknown;
+                    break;
+           }
+           break;
+
+
         case ZEN_FAMILY:
-            cpuid_info.name = amd_zen_str;
-            cpuid_info.short_name = short_zen;
+            switch (cpuid_info.model)
+            {
+                case ZEN_RYZEN:
+                    cpuid_info.name = amd_zen_str;
+                    cpuid_info.short_name = short_zen;
+                    break;
+                case ZENPLUS_RYZEN:
+                case ZENPLUS_RYZEN2:
+                    cpuid_info.name = amd_zenplus_str;
+                    cpuid_info.short_name = short_zen;
+                    break;
+                case ZEN2_RYZEN:
+                case ZEN2_RYZEN2:
+                case ZEN2_RYZEN3:
+                    cpuid_info.name = amd_zen2_str;
+                    cpuid_info.short_name = short_zen2;
+                    break;
+            }
+            break;
+        case ZEN3_FAMILY:
+            switch (cpuid_info.model)
+            {
+                case ZEN3_RYZEN:
+                    cpuid_info.name = amd_zen3_str;
+                    cpuid_info.short_name = short_zen3;
+                    break;
+                case ZEN3_RYZEN2:
+                    cpuid_info.name = amd_zen3_str;
+                    cpuid_info.short_name = short_zen3;
+                    break;
+                case ZEN3_RYZEN3:
+                    cpuid_info.name = amd_zen3_str;
+                    cpuid_info.short_name = short_zen3;
+                    break;
+            }
             break;
 
+        case ARMV7_FAMILY:
+            switch (cpuid_info.model)
+            {
+                case ARM7L:
+                case ARMV7L:
+                    cpuid_info.name = armv7l_str;
+                    cpuid_info.short_name = short_arm7;
+                    break;
+                case ARM_CORTEX_A57:
+                    cpuid_info.name = arm_cortex_a57;
+                    cpuid_info.short_name = short_arm7;
+                    break;
+                case ARM_CORTEX_A53:
+                    cpuid_info.name = arm_cortex_a53;
+                    cpuid_info.short_name = short_arm7;
+                    break;
+                case ARM_CORTEX_A72:
+                    cpuid_info.name = arm_cortex_a72;
+                    cpuid_info.short_name = short_arm7;
+                    break;
+                case ARM_CORTEX_A73:
+                    cpuid_info.name = arm_cortex_a73;
+                    cpuid_info.short_name = short_arm7;
+                    break;
+                default:
+                    return EXIT_FAILURE;
+                    break;
+            }
+            break;
+        case ARMV8_FAMILY:
+            switch (cpuid_info.vendor)
+            {
+                case DEFAULT_ARM:
+                    switch (cpuid_info.part)
+                    {
+                        case ARM_CORTEX_A57:
+                            cpuid_info.name = arm_cortex_a57;
+                            cpuid_info.short_name = short_arm8;
+                            break;
+                        case ARM_CORTEX_A53:
+                            cpuid_info.name = arm_cortex_a53;
+                            cpuid_info.short_name = short_arm8;
+                            break;
+                        case ARM_CORTEX_A72:
+                            cpuid_info.name = arm_cortex_a72;
+                            cpuid_info.short_name = short_arm8;
+                            break;
+                        case ARM_CORTEX_A73:
+                            cpuid_info.name = arm_cortex_a73;
+                            cpuid_info.short_name = short_arm8;
+                            break;
+                        case ARM_NEOVERSE_N1:
+                            cpuid_info.name = arm_neoverse_n1;
+                            cpuid_info.short_name = short_arm8_neo_n1;
+                            break;
+                        default:
+                            return EXIT_FAILURE;
+                            break;
+                    }
+                    break;
+                case CAVIUM1:
+                case CAVIUM2:
+                    switch (cpuid_info.part)
+                    {
+                        case CAV_THUNDERX2T99:
+                        case CAV_THUNDERX2T99P1:
+                            cpuid_info.name = cavium_thunderx2t99_str;
+                            cpuid_info.short_name = short_arm8_cav_tx2;
+                            break;
+                        case CAV_THUNDERX:
+                        case CAV_THUNDERX88:
+                        case CAV_THUNDERX81:
+                        case CAV_THUNDERX82:
+                            cpuid_info.name = cavium_thunderx_str;
+                            cpuid_info.short_name = short_arm8_cav_tx;
+                            break;
+                        default:
+                            return EXIT_FAILURE;
+                            break;
+                    }
+                    break;
+                case FUJITSU_ARM:
+                    switch (cpuid_info.part)
+                    {
+                        case FUJITSU_A64FX:
+                            cpuid_info.name = fujitsu_a64fx;
+                            cpuid_info.short_name = short_a64fx;
+                            break;
+                        default:
+                            return EXIT_FAILURE;
+                            break;
+                    }
+                default:
+                    return EXIT_FAILURE;
+                    break;
+            }
         default:
-            return EXIT_FAILURE;
             break;
     }
     return EXIT_SUCCESS;
@@ -884,11 +1226,19 @@ topology_functions topology_funcs = {
     .init_cacheTopology = proc_init_cacheTopology,
     .close_topology = NULL,
 #else
+#if !defined(__ARM_ARCH_8A)
     .init_cpuInfo = hwloc_init_cpuInfo,
     .init_nodeTopology = hwloc_init_nodeTopology,
     .init_cacheTopology = hwloc_init_cacheTopology,
     .init_cpuFeatures = proc_init_cpuFeatures,
     .close_topology = hwloc_close,
+#else
+    .init_cpuInfo = hwloc_init_cpuInfo,
+    .init_cpuFeatures = proc_init_cpuFeatures,
+    .init_nodeTopology = proc_init_nodeTopology,
+    .init_cacheTopology = proc_init_cacheTopology,
+    .close_topology = NULL,
+#endif
 #endif
     .init_fileTopology = initTopologyFile,
 };
@@ -913,6 +1263,12 @@ void topology_setupTree(void)
         }
         currentNode = tree_getNode(cpuid_topology.topologyTree,
                 hwThreadPool[i].packageId);
+        /*if (!tree_nodeExists(currentNode, hwThreadPool[i].dieId))
+        {
+            printf("Insert Die %d at Socket %d\n", hwThreadPool[i].dieId, hwThreadPool[i].packageId);
+            tree_insertNode(currentNode, hwThreadPool[i].dieId);
+        }
+        currentNode = tree_getNode(currentNode, hwThreadPool[i].dieId);*/
         if (!tree_nodeExists(currentNode, hwThreadPool[i].coreId))
         {
             //printf("Insert Core %d at Socket %d\n", hwThreadPool[i].coreId, hwThreadPool[i].packageId);
@@ -924,7 +1280,7 @@ void topology_setupTree(void)
             /*
                printf("WARNING: Thread already exists!\n");
                */
-            //printf("Insert HWThread %d from Core %d at Socket %d\n", hwThreadPool[i].apicId, hwThreadPool[i].coreId, hwThreadPool[i].packageId);
+            //printf("Insert HWThread %d at Core %d Socket %d\n", hwThreadPool[i].apicId, hwThreadPool[i].coreId, hwThreadPool[i].packageId);
             tree_insertNode(currentNode, hwThreadPool[i].apicId);
         }
 
@@ -949,7 +1305,6 @@ topology_init(void)
     int ret = 0;
     cpu_set_t cpuSet;
     struct topology_functions funcs = topology_funcs;
-
     if (topology_initialized)
     {
         return EXIT_SUCCESS;
@@ -965,13 +1320,30 @@ topology_init(void)
     {
 standard_init:
         CPU_ZERO(&cpuSet);
-        sched_getaffinity(0,sizeof(cpu_set_t), &cpuSet);
+        if (getenv("LIKWID_IGNORE_CPUSET") == NULL)
+        {
+            sched_getaffinity(0,sizeof(cpu_set_t), &cpuSet);
+        }
+        else
+        {
+            for (int i = 0; i < sysconf(_SC_NPROCESSORS_CONF); i++)
+            {
+                if (likwid_cpu_online(i))
+                {
+                    CPU_SET(i, &cpuSet);
+                }
+            }
+        }
         if (cpu_count(&cpuSet) < sysconf(_SC_NPROCESSORS_CONF))
         {
+#if !defined(__ARM_ARCH_7A__) && !defined(__ARM_ARCH_8A)
             cpuid_topology.activeHWThreads =
                 ((cpu_count(&cpuSet) < sysconf(_SC_NPROCESSORS_CONF)) ?
                 cpu_count(&cpuSet) :
                 sysconf(_SC_NPROCESSORS_CONF));
+#else
+            cpuid_topology.activeHWThreads = sysconf(_SC_NPROCESSORS_ONLN);
+#endif
         }
         else
         {
@@ -981,14 +1353,113 @@ standard_init:
         topology_setName();
         funcs.init_cpuFeatures();
         funcs.init_nodeTopology(cpuSet);
+        int activeCount = 0;
+        for (int i = 0; i < cpuid_topology.numHWThreads; i++)
+        {
+            if (cpuid_topology.threadPool[i].inCpuSet)
+                activeCount++;
+        }
+        if (activeCount > cpuid_topology.activeHWThreads)
+            cpuid_topology.activeHWThreads = activeCount;
         funcs.init_cacheTopology();
+        if (cpuid_topology.numCacheLevels == 0)
+        {
+            CacheLevel* cachePool = NULL;
+            switch(cpuid_info.family)
+            {
+                case ARMV8_FAMILY:
+                    switch (cpuid_info.vendor)
+                    {
+                        case CAVIUM2:
+                            switch (cpuid_info.part) {
+                                case CAV_THUNDERX2T99:
+                                    cachePool = (CacheLevel*) malloc(3 * sizeof(CacheLevel));
+                                    for(int i=0;i < 3; i++)
+                                    {
+                                        cachePool[i].level = caviumTX2_caches[i].level;
+                                        cachePool[i].size = caviumTX2_caches[i].size;
+                                        cachePool[i].lineSize = caviumTX2_caches[i].lineSize;
+                                        cachePool[i].threads = caviumTX2_caches[i].threads;
+                                        cachePool[i].inclusive = caviumTX2_caches[i].inclusive;
+                                        cachePool[i].sets = caviumTX2_caches[i].sets;
+                                        cachePool[i].associativity = caviumTX2_caches[i].associativity;
+                                    }
+                                    cpuid_topology.cacheLevels = cachePool;
+                                    cpuid_topology.numCacheLevels = 3;
+                                    break;
+                                default:
+                                    break;
+                            }
+                            break;
+                        case CAVIUM1:
+                            switch (cpuid_info.part) {
+                                case CAV_THUNDERX2T99P1:
+                                    cachePool = (CacheLevel*) malloc(3 * sizeof(CacheLevel));
+                                    for(int i=0;i < 3; i++)
+                                    {
+                                        cachePool[i].level = caviumTX2_caches[i].level;
+                                        cachePool[i].size = caviumTX2_caches[i].size;
+                                        cachePool[i].lineSize = caviumTX2_caches[i].lineSize;
+                                        cachePool[i].threads = caviumTX2_caches[i].threads;
+                                        cachePool[i].inclusive = caviumTX2_caches[i].inclusive;
+                                        cachePool[i].sets = caviumTX2_caches[i].sets;
+                                        cachePool[i].associativity = caviumTX2_caches[i].associativity;
+                                    }
+                                    cpuid_topology.cacheLevels = cachePool;
+                                    cpuid_topology.numCacheLevels = 3;
+                                    break;
+                                default:
+                                    break;
+                            }
+                        case FUJITSU_ARM:
+                            switch(cpuid_info.part) {
+                                case FUJITSU_A64FX:
+                                    cachePool = (CacheLevel*) malloc(2 * sizeof(CacheLevel));
+                                    for(int i=0;i < 2; i++)
+                                    {
+                                        cachePool[i].level = a64fx_caches[i].level;
+                                        cachePool[i].size = a64fx_caches[i].size;
+                                        cachePool[i].lineSize = a64fx_caches[i].lineSize;
+                                        cachePool[i].threads = a64fx_caches[i].threads;
+                                        cachePool[i].inclusive = a64fx_caches[i].inclusive;
+                                        cachePool[i].sets = a64fx_caches[i].sets;
+                                        cachePool[i].associativity = a64fx_caches[i].associativity;
+                                    }
+                                    cpuid_topology.cacheLevels = cachePool;
+                                    cpuid_topology.numCacheLevels = 2;
+                                    break;
+                                default:
+                                    break;
+                            }
+                            break;
+                        default:
+                            break;
+                    }
+                    break;
+                default:
+                    break;
+            }
+        }
         topology_setupTree();
         sched_setaffinity(0, sizeof(cpu_set_t), &cpuSet);
     }
     else
     {
         CPU_ZERO(&cpuSet);
-        sched_getaffinity(0, sizeof(cpu_set_t), &cpuSet);
+        if (getenv("LIKWID_IGNORE_CPUSET") == NULL)
+        {
+            sched_getaffinity(0, sizeof(cpu_set_t), &cpuSet);
+        }
+        else
+        {
+            for (int i = 0; i < sysconf(_SC_NPROCESSORS_CONF); i++)
+            {
+                if (likwid_cpu_online(i))
+                {
+                    CPU_SET(i, &cpuSet);
+                }
+            }
+        }
         DEBUG_PRINT(DEBUGLEV_INFO, Reading topology information from %s, config.topologyCfgFileName);
         ret = readTopologyFile(config.topologyCfgFileName, cpuSet);
         if (ret < 0)
@@ -1101,14 +1572,18 @@ print_supportedCPUs (void)
     printf("\t%s\n",broadwell_str);
     printf("\t%s\n",broadwell_d_str);
     printf("\t%s\n",broadwell_ep_str);
-    printf("\t%s\n",skylake_str);
     printf("\t%s\n",atom_goldmont_str);
     printf("\t%s\n",xeon_phi2_string);
+    printf("\t%s\n",skylake_str);
     printf("\t%s\n",skylakeX_str);
     printf("\t%s\n",xeon_phi3_string);
     printf("\t%s\n",kabylake_str);
     printf("\t%s\n",coffeelake_str);
     printf("\t%s\n",cascadelakeX_str);
+    printf("\t%s\n",tigerlake_str);
+    printf("\t%s\n",icelake_str);
+    printf("\t%s\n",rocketlake_str);
+    printf("\t%s\n",icelakesp_str);
     printf("\n");
     printf("Supported AMD processors:\n");
     printf("\t%s\n",opteron_sc_str);
@@ -1121,6 +1596,22 @@ print_supportedCPUs (void)
     printf("\t%s\n",interlagos_str);
     printf("\t%s\n",kabini_str);
     printf("\t%s\n",amd_zen_str);
+    printf("\t%s\n",amd_zen2_str);
+    printf("\t%s\n",amd_zen3_str);
+    printf("\n");
+    printf("Supported ARMv8 processors:\n");
+    printf("\t%s\n",arm_cortex_a53);
+    printf("\t%s\n",arm_cortex_a57);
+    printf("\t%s\n",cavium_thunderx_str);
+    printf("\t%s\n",cavium_thunderx2t99_str);
+    printf("\t%s\n",fujitsu_a64fx);
+    printf("\n");
+    printf("Supported ARMv7 processors:\n");
+    printf("\t%s\n",armv7l_str);
+    printf("\n");
+    printf("Supported POWER processors:\n");
+    printf("\t%s\n",power8_str);
+    printf("\t%s\n",power9_str);
     printf("\n");
 }
 
@@ -1141,4 +1632,3 @@ get_numaTopology(void)
 {
     return &numa_info;
 }
-
